% *************************************************************************
%
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
% 
% This exercise considers a cylindrical tank with constant cross-section A, 
% whose input flow (Fi) can be manipulated, while the output flow 
% is controlled by gravity, the water height (h), and the friction 
% coefficient of the output pipe Kv
% 
% Sects. 1.3.1 and 1.11.1, Figure 1.13, and Eq. (1.10)
%
% *************************************************************************

clear
close all

%% Define parameters
A    = 20;    % Tank cross-section                                   [cm^2]
Fi   = 1;     % Input variable flow                                 [L/min]
Kv   = 0.62;  % Outlet friction coefficient
Fb   = 2;     % Offset flow                                         [L/min]
w    = 0.001; % Square wave frequency                               [1/min]
tfin = 2000;  % Final simulation time                                 [min]

%% Set initial condition
ho   = (Fb/Kv)^2; % Initial condition (steady-state)

%% Launch the simulation
[t, x, y] = sim('tank', tfin);

%% Figures
figure(1)
set(1,'Position',[15 62 866 459])
subplot(2,1,1)
plot(t, h)
set(gca, 'FontName','Arial', 'FontSize',12)
hold on
plot([0 tfin], [ho ho], ':k')
ylabel('Height [cm]')

subplot(2,1,2)
set(gca, 'FontName','Arial', 'FontSize',12)
plot(t,Fin,':b', t,Fo)
hold on
plot([0 tfin], [Fb Fb], ':k')
ylabel('Flow [L/min]')
legend('Fi', 'Fo', 'location','northoutside', 'orientation','horizontal')
xlabel('time [min]')