%% ************************************************************************
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Parameter estimation of the Monod kinetics.
% The script uses the data generated with Fisher_contour_Monod.m which
% should be launched first, unless you want to use the data files 
% already available in the subfolder.
%
% Sect. 2.4.5
%
% *************************************************************************

clear
close all
clc
%%
% Global parameters definition
global Mu Ks Y bH So Xo


% Get the calibration data
%
g=uigetfile('Monod_*.mat','Load calibration data');
eval(['load ' g]);
%
% Remember! The data must be called Monod_*
%
   tx=Monod_data(:,1);tfin=100;
   Sx=Monod_data(:,2);Xx=Monod_data(:,3);
   NomeModello='Monod';
   Mu_vero=Mp(1);Ks_vero=Mp(2);Y=Mp(3);bH=Mp(4); % Store the true parameter values
   % Initial parameter values
      So=10;Xo=0.1; % Initial conditions
      Par=[0.4 23]';% Initial parameter guess

figure(2)
hold off;clf
set(2,'Position',[25 466 502 315])
plot(tx,Sx,'ob',tx,Xx,'db','MarkerSize',8,'MarkerFaceColor','b')
axis([0 tfin 0 So])

figure(3)
clf
set(3,'Position',[22 61 505 311])
contour(Mu1,Ks1,E1,[10 50 100 500],':b');
set(gca,'FontName','Arial','FontSize',12)
xlabel('{\mu}_m_a_x')
ylabel('K_s')
hold on
plot(Mp(1),Mp(2),'db','MarkerSize',8,'MarkerFaceColor','b')
%
% Trace the approximate confidence region
% computed via the FIM inherited from the
% previous Fisher_Contour_SP_DO.m
%
[v,e]=eig(inv(F));
L=sqrt(e);
fi = [0:0.01:2*pi];
pH = ones(2,length(fi));
for i = 1:length(fi);
    pH(:,i) = v*L*[cos(fi(i));-sin(fi(i))]+[Mp(1);Mp(2)];
end
plot(pH(1,:),pH(2,:),'b','LineWidth',1)
%%
%

options=foptions;
options(1)=1;
options(2)=0.00001;
options(3)=0.00001;
options(14)=5000;
[ParCal,options]=simplex('Ferr_Monod',Par,options,[],NomeModello,tfin,Monod_data);
ParBest=ParCal;
% Trace exact confidente region based on the minimum of the error
% functional
n_exp=length(tx); % Number of experimental data
Ec=options(8)*(1+(2/(n_exp-2))*finv(0.9,2,n_exp-2));
contour(Mu1,Ks1,E1,[Ec],'--b','LineWidth',2)
% Locate the true parameters
plot(Mu_vero,Ks_vero,'db','MarkerSize',8,'MarkerFaceColor','b')
% Locate the estimated parameters
plot(ParBest(1),ParBest(2),'db','MarkerSize',8,'MarkerFaceColor','y')
fprintf('\n')
fprintf('**********************\n')
fprintf(['Estimated parameter values:\n'])
fprintf('Mu  = %g\n',ParBest(1))
fprintf('Ks  = %g\n',ParBest(2))
fprintf('**********************\n')
% Simulation with optimal parameters
Mu=ParBest(1);Ks=ParBest(2); 

[t,x,y]=sim(NomeModello,tfin);
ys1=interp1(t,S,tx);
ys2=interp1(t,X,tx);
resid=sqrt((Sx-ys1).^2+(Xx-ys2).^2);
Residual_analysis(resid,12);
disp(' ')
disp('Residual analysis after estimation')
mum=mean(resid); sigmum=var(resid);
disp(['Residual mean value ',num2str(mum)])
disp(['Residual variance   ',num2str(sigmum)])

%
% Compute the estimation errors
%
dMu=Mu_vero-ParCal(1);
dKs=Ks_vero-ParCal(2);
disp('-------------------------------------------------------')
disp(['Estimated Mu = ',num2str(ParCal(1))])
disp(['Estimated Ks = ',num2str(ParCal(2))])
disp(['Estimation confidence band for Mu =  ',num2str(std_par(1))])
disp(['Estimation confidence band for Ks =  ',num2str(std_par(2))])
disp(['Estimation error for Mu ',num2str(dMu),' equal to ',num2str(100*abs(dMu/Mu_vero)),' %'])
disp(['Estimation error for Ks ',num2str(dKs),' equal to ',num2str(100*abs(dKs/Ks_vero)),' %'])
disp('-------------------------------------------------------')
