% 
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Data generation and FIM confidence region computation as a preliminary to
% the parameter estimation of the Monod kinetics (Cal_Mono.m).
%
% Exercise 2.4.5
%
% *************************************************************************

clear
close all
clc

prompt={'Mu_max (1/h)','Ks (mg/l)','Y','bh',...
    'Initial Substrate','Initial Biomass','Final time','Noise variance (%)','Number of samples'};
def={'0.45','30','0.5','0.01','10','0.1','100','15','30'};
dati=inputdlg(prompt,'Model parameters initial estimates',1,def,'on');
dati=char(dati);
Mu=str2num(dati(1,:));     %  Maximum growth rate
Ks=str2num(dati(2,:));     %  Half-velocity
Y=str2num(dati(3,:));      %  Yield
bh=str2num(dati(4,:));     %  Decay rate
So=str2num(dati(5,:));     %  Initial substrate
Xo=str2num(dati(6,:));     %  Initial biomass
tfin=str2num(dati(7,:));   %  Simulation time
n_var=str2num(dati(8,:));  %  Measurement noise
n_exp=str2num(dati(9,:));  % Number of samples
n_var=n_var/100;

% Group the parameters into a single vector
Monod_param=[Mu Ks Y bh];

% Launch the simulation

[tsim,x,y]=sim('Monod',tfin);
% Save nominal values
Mu_nom=Mu;Ks_nom=Ks;Y_nom=Y;bh_nom=bh;
tsim_nom=tsim;S_nom=S;X_nom=X;
nlength=length(tsim);
n=n_var*randn(nlength,2);

S_n=S_nom+n(:,1);  % Substrate data with additive noise
X_n=X_nom+n(:,2);  % Biomass data with additive noise

figure(1)
set(1,'Position',[120 59 698 682])
subplot(311)
plot(tsim,S_n,':b',tsim,X_n,'b')
ylabel('Concentration (mg/l)')
set(gca,'FontName','Arial','FontSize',12)
xlabel('time (h)')
legend('S','X','location','northeast','orientation','horizontal')
hold on

%%
% Sensitivity computation
%
% Compute the perturbed trajectory
ds=0.0001; % Perturbation
 % Sensitivity to Mu
p0=Mu; 
Mu=p0*(1+ds); % perturbed Mu
[tsim,x,y]=sim('Monod',tfin);
S_pert=interp1(tsim,S,tsim_nom);
X_pert=interp1(tsim,X,tsim_nom);
S_sens_Mu=(S_pert-S_nom)/(ds*p0); % Sensitivity of S to Mu
X_sens_Mu=(X_pert-X_nom)/(ds*p0); % Sensitivity of X to Mu
% Compute the cumulative sensitivity to Mu


% Sensitivity to Ks
Mu=Mu_nom;
Ks=Ks_nom;
p0=Ks; 
Ks=p0*(1+ds);
[tsim,x,y]=sim('Monod',tfin);
S_pert=interp1(tsim,S,tsim_nom);
X_pert=interp1(tsim,X,tsim_nom);
S_sens_Ks=(S_pert-S_nom)/(ds*p0); % Sensitivity of S to Ks
X_sens_Ks=(X_pert-X_nom)/(ds*p0); % Sensitivity of X to Ks
% Compute the cumulative sensitivity to Ks

subplot(312)
plot(tsim_nom,S_sens_Mu,':b',tsim_nom,X_sens_Mu,'b')
set(gca,'FontName','Arial','FontSize',12)
legend('Sens S to Mu','Sens X to Mu','location','northeast','orientation','horizontal')
xlabel('Time (h)')

subplot(313)
plot(tsim_nom,S_sens_Ks,':b',tsim_nom,X_sens_Ks,'b')
set(gca,'FontName','Arial','FontSize',12)
legend('Sens S to Ks','Sens X to Ks','location','northeast','orientation','horizontal')
xlabel('Time (h)')

% Acquire S,X sampling points
figure(1)
subplot(311)
Im=2;
switch Im
    case 1 % High-sensitivity measurements
Is=find(abs(X_sens_Mu)>0.4*abs(max(X_sens_Mu)));
t1=tsim_nom(Is(1));t2=tsim_nom(Is(end));
tempi=linspace(t1,t2,n_exp);
    case 2 % Uniform measurements
tempi=linspace(0,tfin,n_exp);
end
S_mis=interp1(tsim_nom,S_n,tempi);
X_mis=interp1(tsim_nom,X_n,tempi);

figure(1)
subplot(311)
hold on
plot(tempi,S_mis,'ob','MarkerSize',4,'MarkerFaceColor','b')
plot(tempi,X_mis,'db','MarkerSize',4,'MarkerFaceColor','b')
drawnow
SS_Mu=interp1(tsim,S_sens_Mu,tempi); % Compute the S sensitivity to Mu
SX_Mu=interp1(tsim,X_sens_Mu,tempi); % Compute the X sensitivity to Mu
subplot(312)
hold on
plot(tempi,SS_Mu,'ob','MarkerSize',4,'MarkerFaceColor','b')
plot(tempi,SX_Mu,'db','MarkerSize',4,'MarkerFaceColor','b')
drawnow
SS_Ks=interp1(tsim,S_sens_Ks,tempi); % Compute the S sensitivity to Ks
SX_Ks=interp1(tsim,X_sens_Ks,tempi); % Compute the S sensitivity to Ks
subplot(313)
hold on
plot(tempi,SS_Ks,'ob','MarkerSize',4,'MarkerFaceColor','b')
plot(tempi,SX_Ks,'db','MarkerSize',4,'MarkerFaceColor','b')
drawnow
% Compute the measurement errors
S_nom_mis=interp1(tsim,S_nom,tempi);
S_res=S_mis-S_nom_mis;
S_var=var(S_res);
S_std=sqrt(S_var);
X_nom_mis=interp1(tsim,X_nom,tempi);
X_res=X_mis-X_nom_mis;
X_var=var(X_res);
X_std=sqrt(X_var);
disp(['Measurement noise on S  ',num2str(S_var)])
disp(['Measurement noise on X  ',num2str(X_var)])
disp('Q matrix')
Q=[1/S_var 0;0 1/X_var]
%

%% Fisher Information Matrix
F=ones(2,2);
F(1,1)=sum((SS_Mu.*SS_Mu)/S_var+(SX_Mu.*SX_Mu)/X_var);
F(1,2)=sum((SS_Mu.*SS_Ks)/S_var+(SX_Mu.*SX_Ks)/X_var);
F(2,1)=F(1,2);
F(2,2)=sum((SS_Ks.*SS_Ks)/S_var+(SX_Ks.*SX_Ks)/X_var);
disp('Fisher Information Matrix')
disp(F)
disp(' ')
% Covariance matrix
disp('Covariance matrix')
Covar_Mat=inv(F); % F^-1 
disp(Covar_Mat)
disp(' ') 
Corr_Mat=[];
Corr_Mat(1,1)=Covar_Mat(1,1)/sqrt(Covar_Mat(1,1)*Covar_Mat(1,1));
Corr_Mat(1,2)=Covar_Mat(1,2)/sqrt(Covar_Mat(1,1)*Covar_Mat(2,2));
Corr_Mat(2,1)=Corr_Mat(1,2);
Corr_Mat(2,2)=Covar_Mat(2,2)/sqrt(Covar_Mat(2,2)*Covar_Mat(2,2));

%%
% Estimated parameters confidence bands
std_Mu=tinv(0.95,n_exp)*sqrt(Covar_Mat(1,1));
std_Ks=tinv(0.95,n_exp)*sqrt(Covar_Mat(2,2));
disp(['Estimation confidence band for Mu =  ',num2str(std_Mu)])
disp(['Estimation confidence band for Ks =  ',num2str(std_Ks)])


%%
% Compute the Error functional
%
i=0;j=0;E=[];E_do=[];
d1=0.85;d2=1.15; % Upper & lower parameter spread
Mup=linspace(d1*Mu_nom,d2*Mu_nom,25);
Ksp=linspace(d1*Ks_nom,d2*Ks_nom,25);
Par1=zeros(length(Mup),1);Par2=zeros(length(Ksp),1);
for Mu=Mup
           i=i+1;j=0;
           Par1(i)=Mu;
           for Ks=Ksp
              j=j+1;
              Par2(j)=Ks;
              [t,x,y]=sim('Monod',tfin);
              % Synchronize simulations
              S_pert=interp1(t,S,tsim_nom);X_pert=interp1(t,X,tsim_nom);
              % Full error functional
              E(i,j)=sum((S_pert-S_nom).^2)+sum((X_pert-X_nom).^2); 
           end
end
figure(10)
clf
set(10,'Position',[116 118 560 420])
% E Contour smoothing
Mu1=linspace(d1*Mu_nom,d2*Mu_nom,50);
Ks1=linspace(d1*Ks_nom,d2*Ks_nom,50);
E1=interp2(Par1,Par2,E,Mu1,Ks1','spline');
contour(Mu1,Ks1,E1,[10 50 100 500],':b');
set(gca,'FontName','Arial','FontSize',12)
xlabel('Mu')
ylabel('K_s')
hold on
plot(Mu_nom,Ks_nom,'db','MarkerSize',8,'MarkerFaceColor','b')

%
% Trace the approximate confidence region
% computed via the FIM inherited from the
% previous Fisher_contour_Monod.m
%
[v,e]=eig(inv(F));
L=sqrt(e);
fi = [0:0.01:2*pi];
pH = ones(2,length(fi));
for i = 1:length(fi);
    pH(:,i) = v*L*[cos(fi(i));-sin(fi(i))]+[Mu_nom;Ks_nom];
end
plot(pH(1,:),pH(2,:),'b','LineWidth',1)
%%
%
% Save the data for the estimation
%
sf=input('Do you want to save the data? (y/n) ','s');
if sf=='y',
    Fname=input('Name of data file (begin with Monod_)','s');
Fn=char(Fname);
Monod_data=[tempi' S_mis' X_mis'];
Mn='Monod_data';
Mp=[Mu_nom Ks_nom Y_nom bh_nom];
Mpn='Mp'; % Nominal parameters
std_par=[std_Mu std_Ks];
std_Par='std_par';
nv='n_var';
En='E1';FFn='F'; % Error functional & FIM
Par1='Mu1';Par2='Ks1'; % Values for plotting the error functional
eval(['save ',Fn,' ', Mn,' ',Mpn,' ',En,' ',Par1,' ',Par2,' ',FFn,' ',std_Par,' ',nv])
end