% *************************************************************************
%
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
% 
% Parameter estimation of the Streeter & Phelps model with two point
% discharges.
% 
% Sect. 2.4.4
%
% *************************************************************************

clear
close all
clc

%% Define global parameters for the simulation
global Kb Kc Csat Bo u L xs Bs

%% Load the calibration data
g = uigetfile('DO*.mat', 'Load calibration data');
eval(['load ' g]);

% System parameters loaded from the calibration data file
Bo   = DO_param(1);      % upstream BOD 
Csat = DO_param(2);      % DO saturation
Kb   = DO_param(3);      % BOD decay constant
Kc   = DO_param(4);      % reoxigenation constant
u    = DO_param(5);      % stream velocity (m(s)
L    = DO_param(6);      % river length
xs   = DO_param(7);      % location of the second point source
Bs   = DO_param(8);      % BOD of the second point source

%% Exctract information from the DO_Sperim data matrix
t_s = DO_sperim(:,1); % experimental sampling locations
DO_s = DO_sperim(:,2);% experimental DO samples
N = length(DO_s);     % Number of experimental data
tfin = L;             % Final simulation time equal to river length
Kb_vero = Kb;
Kc_vero = Kc; % "True" parameter values

prompt = {'Kb BOD decay [1/h]',...
          'Kc oxigenation kinetics [1/h]'};
def = {'0.1','0.12'}; % Default initial parameter values
data = inputdlg(prompt, 'Initial estimates', 1, def,'on');
data = char(data);
Kb0 = str2num(data(1,:));% BOD decay constant
Kc0 = str2num(data(2,:));% reoxigenation constant


Par = [Kb0 Kc0]';  % Parameters to be estimated
ModelName = 'SP'; 
%% Load the pre-computed error functional contour 

figure(3)
clf
set(3, 'Position',[24 55 503 316])
set(gca, 'FontName','Arial', 'FontSize',10)
contour(Par1, Par2, E, [0.1 0.5 1 2.5 5 10 25], 'b')
hold on

xlabel('K_b')
ylabel('K_c')
plot(Kb_nom, Kc_nom, 'or', 'MarkerFaceColor','r', 'MarkerSize',8)

%% Start Optimization
%  Set SIMPLEX options (see function help for details)
options = foptions;
options(1)  = 1;
options(2)  = 1e-8;
options(3)  = 1e-8;
options(14) = 1500;
[ParCal, options] = simplex('Ferr_SP', Par,options, [], ModelName,...
                            tfin, DO_sperim);

figure(3)
plot(ParCal(1), ParCal(2), 'db', 'MarkerFaceColor','y', 'MarkerSize',8)
xlabel('K_b')
ylabel('K_c')

% options(8) is the minimum value of the error functional
Ec = options(8)*(1+(2/(N-2))*finv(0.95,2,N-2));
contour(Par1, Par2, E, [Ec], ':r', 'LineWidth',2)
hold on

% Trace the approximate confidence region
% computed via the FIM inherited from the
% previous Fisher_Contour_SP_DO.m

[v, e] = eig(inv(F));
L = sqrt(e);
fi = [0:0.01:2*pi];
pH = ones(2, length(fi));
for i = 1:length(fi);
    pH(:,i) = v*L*[cos(fi(i)); -sin(fi(i))] + ParCal;
end
plot(pH(1,:), pH(2,:), ':r', 'LineWidth',2)


%% Plot calibrated model response
figure(1)
% Nominal model
Kb = Kb_nom;
Kc = Kc_nom;
[t, x, y] = sim('SP', tfin);
plot(t, DO, 'b', 'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
hold on
% Data
plot(DO_sperim(:,1), DO_sperim(:,2), 'or', 'MarkerFaceColor','r')
xlabel('river length [km]')
ylabel('DO Conc. [mg/L]')
legend('Data', 'Calibr. model', 'True model', 'location','north',...
       'orientation','horizontal')
title('Calibration with DO measurements')
axis([0 tfin 4 10]);
a = axis;
start_text = a(2)*1/3;

text(start_text, a(3) + 1.5, ['True K_b = ', num2str(Kb_vero),...
                              '   Est. K_b = ',num2str(ParCal(1))],...
                              'FontName','Arial', 'FontSize',12)
text(start_text, a(3) + 1.1, ['True K_c = ', num2str(Kc_vero),...
                              '   Est. K_c = ', num2str(ParCal(2))],...
                              'FontName','Arial','FontSize',12)

%% Downstream processing

% Compute the Variance Accounted For (VAF)

[vaf, DO1] = Vaffa(t_s, DO_s, t,DO); 
% DO1 = model DO interpolated at the sampling points

disp('----------------------------')
disp('---  Estimation Results  ---')
disp('----------------------------')
disp(' ')
disp(['Variance Accounted For ', num2str(vaf), ' %'])
disp(' ')

% Compute the residuals
resid = DO_s - DO1;
resid = resid(2:end);

% Compute the residual analysis

figure(4)
set(4, 'Position', [543 467 498 312])
Residual_analysis(resid,4)
figure(5)
set(5, 'Position', [546 59 498 312])

disp(' ')
disp('Residual analysis after estimation')
mu  = mean(resid); 
sig = var(resid);
disp(['Residual mean value ',num2str(mu)])
disp(['Residual variance   ',num2str(sig)])

% Compute the estimation errors

dKb = Kb_vero - ParCal(1);
dKc = Kc_vero - ParCal(2);
disp('-------------------------------------------------------')
disp(['Estimated Kb = ', num2str(ParCal(1))])
disp(['Estimated Kc = ', num2str(ParCal(2))])
disp(['Estimation error for Kb ', num2str(dKb),...
      ' equal to ', num2str(100*abs(dKb/Kb_vero)), ' %'])
disp(['Estimation error for Kc ', num2str(dKc),...
      ' equal to ', num2str(100*abs(dKc/Kc_vero)), ' %'])
disp('-------------------------------------------------------')

%% Perform a F-test on the regression line
figure(8)
set(8, 'Position', [294 231 502 318])
clf
[r, ME, SE, NC, Ftest] = F_test(DO_s, DO1);