% *************************************************************************
%
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
%  Parameter estimation of the Streeter & Phelps model
%  with two point discharges
%  Preliminary identifiability assessment
%  with sensitivity and FIM computation
%
%  Sect. 2.4.4
% 
% ********************************************************

clear
close all
clc

prompt={'Kb BOD decay (1/h)',...
      'Kc reoxigenation (1/h)','Velocity u (m/s)',...
      'Reach length (Km)','Second point source (Km)', 'BOD second source (mg/l)',...
      'BOD after dilution'};
def={'0.12','0.2','0.1','40','20','40','10'};
dati=inputdlg(prompt,'Model parameters initial estimates',1,def,'on');
dati=char(dati);
Kb=str2num(dati(1,:)); %  BOD decay constant
Kc=str2num(dati(2,:)); %  reoxigenation constant
u=str2num(dati(3,:));  %  stream velocity (m/s)
u=u*3.6;               %  Conversion m/s -> Km/h
L=str2num(dati(4,:));  %  river length
xs=str2num(dati(5,:)); %  location of the second point source
Bs=str2num(dati(6,:)); %  BOD of the second point source
Bo=str2num(dati(7,:)); %  upstream BOD after dilution
Csat=8.6;              %  DO saturation

%% Group the parameters into a single vector
DO_param=[Bo Csat Kb Kc u L xs Bs];

%% Launch the simulation
[tsim,x,y]=sim('SP',[0 L]);

%% Save nominal values
Kb_nom=Kb;Kc_nom=Kc;
tsim_nom=tsim;BOD_nom=BOD;DO_nom=DO;

figure(1)
clf
set(1,'Position',[98 64 1070 634])
subplot(211)
plot(tsim,DO,'b')
set(gca,'FontName','Arial','FontSize',12)
ylabel('mg O_2/L')
xlabel('River length (Km)')
ylabel('DO (mg/l)')
title('Acquire DO data by clicking on the blue curve (right-click to end)')
hold on
plot([0 L],[Csat Csat],':b')

%% Sensitivity computation
% Compute the perturbed trajectory
ds=0.0001; % Perturbation
 % Sensitivity to Kb
p0=Kb; 
Kb=p0*(1+ds); % perturbed Kb
[t,x,y]=sim('SP',[0 L]);
BOD_pert=interp1(tsim,BOD,tsim_nom);
DO_pert=interp1(tsim,DO,tsim_nom);
BOD_sens_Kb=(BOD_pert-BOD_nom)/(ds*p0); % Sensitivity of BOD to Kb
DO_sens_Kb=(DO_pert-DO_nom)/(ds*p0);    % Sensitivity of DO to Kb
S_Kb=DO_sens_Kb; % Vector of sensitivities in the sampling points


% Compute the cumulative sensitivity to Kb
Sens_Kb=sqrt(sum(BOD_sens_Kb.^2)+sum(DO_sens_Kb.^2));

% Sensitivity to Kc
Kb=Kb_nom;
p0=Kc; 
Kc=p0*(1+ds);
[t,x,y]=sim('SP',[0 L]);
BOD_pert=interp1(tsim,BOD,tsim_nom);
DO_pert=interp1(tsim,DO,tsim_nom);
DO_sens_Kc=(DO_pert-DO_nom)/(ds*p0);% Sensitivity of DO to Kc
S_Kc=DO_sens_Kc;% Vector of sensitivities in the sampling points
% Compute the cumulative sensitivity

Sens_Kc=sqrt(sum(DO_sens_Kc.^2));
disp(' ')
disp(['Global sensitivity of Kb to DO = ',num2str(Sens_Kb)])
disp(['Global sensitivity of Kc to DO = ',num2str(Sens_Kc)])

subplot(212)
plot(tsim_nom,BOD_sens_Kb,':r',tsim_nom,DO_sens_Kb,':b',tsim_nom,DO_sens_Kc,'-.b')
set(gca,'FontName','Arial','FontSize',12)
legend('Sens BOD to K_b','Sens DO to K_b','Sens DO to K_c','location','north','orientation','horizontal')
xlabel('Flow time (h)')
% Acquire DO sampling points
figure(1)
subplot(211)
tempi=[];DO_mis=[];i=0;butt=1;
while butt==1
i=i+1;
[x ,y,butt]=ginput(1);
plot(x,y,'dr','MarkerFaceColor','r')
tempi(i)=x;DO_mis(i)=y;
end

[tempi,I]=sort(tempi);
DO_mis=DO_mis(I);
n_exp=length(DO_mis);

SS_Kb=interp1(tsim,S_Kb,tempi); % Compute the Kb sensitivity at the sampling points
SS_Kc=interp1(tsim,S_Kc,tempi); % Compute the Kc sensitivity at the sampling points
%% Compute the measurement errors
%
% Compute the measurement errors
DO_nom_mis=interp1(tsim,DO_nom,tempi);
DO_res=DO_mis-DO_nom_mis;
% Measurement variance
DO_var=var(DO_res);
DO_std=std(DO_res);
fprintf(['\nVariance of DO meas. = ',num2str(DO_var),'\n\n'])

%

%% Fisher Information Matrix
F=ones(2,2);
F(1,1)=sum(SS_Kb.*SS_Kb);
F(1,2)=sum(SS_Kb.*SS_Kc);
F(2,1)=F(1,2);
F(2,2)=sum(SS_Kc.*SS_Kc);
disp('Fisher Information Matrix')
disp(F)
disp(' ')
% Covariance matrix
disp('Covariance matrix')
Covar_Mat=DO_var*inv(F); % F^-1 multiplied by the measurement error variance
disp(Covar_Mat)
disp(' ') 
Corr_Mat=[];
Corr_Mat(1,1)=Covar_Mat(1,1)/sqrt(Covar_Mat(1,1)*Covar_Mat(1,1));
Corr_Mat(1,2)=Covar_Mat(1,2)/sqrt(Covar_Mat(1,1)*Covar_Mat(2,2));
Corr_Mat(2,1)=Corr_Mat(1,2);
Corr_Mat(2,2)=Covar_Mat(2,2)/sqrt(Covar_Mat(2,2)*Covar_Mat(2,2));

%%
% Estimated parameters confidence bands
std_Kb=tinv(0.95,n_exp)*sqrt(Covar_Mat(1,1));
std_Kc=tinv(0.95,n_exp)*sqrt(Covar_Mat(2,2));
disp(['Estimation confidence band for Kb =  ',num2str(std_Kb)])
disp(['Estimation confidence band for Kc =  ',num2str(std_Kc)])

DO_sperim=zeros(n_exp,2);
DO_sperim(:,1)=tempi';
DO_sperim(:,2)=DO_mis';
%%
% Compute the Error functional
%
i=0;j=0;E=[];
Kbp=linspace(Kb_nom/2,2*Kb_nom,25);
Kcp=linspace(Kc_nom/2,2*Kc_nom,25);
for Kb=Kbp
           i=i+1;j=0;
           Par1(i)=Kb;
           for Kc=Kcp
              j=j+1;
              Par2(j)=Kc;
              [t,x,y]=sim('SP',L);
              DO_pert=interp1(t,DO,tsim_nom);
              E(i,j)=sum((DO_pert-DO_nom).^2);         
           end
end
figure(10)
clf
set(10,'Position',[116 118 560 420])
set(gca,'FontName','Arial','FontSize',12)
% E Contour smoothing
Kb1=linspace(0.5*Kb_nom,1.5*Kb_nom,250);
Kc1=linspace(0.5*Kc_nom,1.5*Kc_nom,250);
E1=interp2(Par1,Par2,E,Kb1,Kc1','spline');
[C,h]=contour(Kb1,Kc1,E1,[0.25 0.5 1 2],'b');
clabel(C,h)
hold on
xlabel('K_b')
ylabel('K_c')
hold on
plot(Kb_nom,Kc_nom,'or','MarkerSize',8,'MarkerFaceColor','r')
Ec = DO_var*(1+(2/(n_exp-2))*finv(0.95,2,n_exp-2));
% Tracing the 95% confidence region
[C, h] = contour(Kb1, Kc1, E1, Ec, 'r', 'LineWidth',2);
clabel(C, h)
axis([0.1 0.15 0.14 0.28])
legend('E','Opt','E95')

figure(12)
set(12,'Position',[694 117 560 420])
set(gca,'FontName','Arial','FontSize',12)
surfc(Par1,Par2,E)
view([35 38])
shading interp
hold on
plot3(Kb_nom,Kc_nom,min(min(E)),'or','MarkerSize',8,'MarkerFaceColor','y')
xlabel('K_b')
ylabel('K_c')
zlabel('E')
rotate3d on


%%
%
% Save the data for the estimation
%
sf=input('Do you want to save the data? (y/n) ','s');
if sf=='y',
Fname=input('Name of data file (begin with DO_)','s');
Fn=char(Fname);
E=E1;Par1=Kb1;Par2=Kc1;
eval(['save ',Fn,' DO_sperim DO_param E F Par1 Par2 Kb_nom Kc_nom'])
end