% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% This exercise demonstrates the PCA reconstruction with full or reduced 
% dimension using random data.
% %
% Sect. 3.5.2
%
% *************************************************************************

clear
close all
clc

%% Add Tool path
curr_path = pwd; % Get current path
i_rp = strfind(curr_path, 'Exercises'); % Find root level
root_path = curr_path(1:i_rp + 8); % Get root level path
addpath([root_path '/Tools']) % Add Tools path
clear curr_path i_rp root_path

%% Generate the input values
N = 150;
X = zeros(N, 2);
%  Random synthetic observations (with unit variance)
for i = 1:N-1
    e = randn(1);
    X(i+1,1) = 0.75*X(i,2) -  0.2*X(i,2) - 0.25*e;
    X(i+1,2) = -0.2*X(i,1) + 0.52*X(i,1) + 0.45*e;
end

% Compute the average
XM = mean(X); 
% Substract the average
X(:,1) = X(:,1) - XM(1);
X(:,2) = X(:,2) - XM(2);
[N p] = size(X);

% Figure
hh = figure(1);
set(hh, 'Position',[40 108 1110 477])

subplot(221)
plot(X(:,1))
hold on
plot([0 N], [0 0], 'k')
axis tight
set(gca, 'FontName','Arial', 'FontSize',12)
xlabel('sample')
ylabel('x_1')

subplot(223)
plot(X(:,2))
hold on
plot([0 N], [0 0], 'k')
axis tight
set(gca, 'FontName','Arial', 'FontSize',12)
xlabel('sample')
ylabel('x_2')


%% Trace the covariance ellipse
%  Compute the covariance
A = cov(X)
%  Compute the correlation
B = corr(X)
[v, e] = eig(A);
L(1) = 1.96*sqrt(e(1,1));
L(2) = 1.96*sqrt(e(2,2));
E = diag(L);
fi = [0:0.001:2*pi];
pH = ones(2, length(fi));
for i = 1:length(fi);
    pH(:,i) = v'*E*[cos(fi(i)); -sin(fi(i))];
end

figure(1)
subplot(2,2,[2 4])
fill(pH(1,:), pH(2,:), 'c')
hold on
plot([0,L(1)*v(1,1)], [0,L(1)*v(1,2)], ':r')
plot([0,L(2)*v(2,1)], [0,L(2)*v(2,2)], ':r')
for i = 1:N
    plot(X(i,1), X(i,2), '*b')
end
set(gca, 'FontName','Arial', 'FontSize',12)
axis equal
az = axis;
plot([az(1) az(2)], [0 0], ':k')
plot([0 0], [az(3) az(4)], ':k')
xlabel('x_1'), ylabel('x_2')
title('Covariance ellipse')

%% Perform PCA transform
[P, Z, L, T2] = princomp(X);
scree(L, 2)

figure(3)
vl = {' x_1',' x_2'};
biplot(P, 'scores', Z, 'VarLabels',vl)
grid off
hold on
unit_circle(1, 1)
axis equal
xlabel('PC_1')
ylabel('PC_2')

%% Hotelling's T^2
hh = figure(4);
set(hh, 'Position',[296 132 731 372])
clf
plot(T2, 'b', 'LineWidth',2)
hold on
T_lim = (p*(N-1)/(N-p))*finv(0.95, p, N-p);
plot([0 length(T2)], [T_lim T_lim], '--k', 'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
xlabel('samples')
ylabel('T^2')
legend('T^2', 'T^2_{lim}', 'Location','northeast',...
       'orientation','horizontal')
	   
%% Check the duality of the transformations
hh = figure(5);
set(hh, 'Position',[36 58 910 536])
%  Inverse PCA transform
X1 = Z*P';

[resid,R1]=pcares(X,1);

subplot(211)
plot(X(:,1), '.b')
hold on
plot(X1(:,1), ':b')
plot(R1(:,1), 'dk')
plot([0 N], [0 0], 'k')
set(gca, 'FontName','Arial', 'FontSize',12)
title('Reconstructed variables')
xlabel('samples')
ylabel('X_1')
legend('X_1', 'R_2 X_1', 'R_1 X_1', 'location','north',...
       'orientation','horizontal')

subplot(212)
plot(X(:,2), '.b')
hold on
plot(X1(:,2), ':b')
plot(R1(:,2), 'dk')
plot([0 N], [0 0], 'k')
set(gca, 'FontName','Arial', 'FontSize',12)
legend('X_2', 'R_2 X_2', 'R_1 X_2', 'location','north',...
       'orientation','horizontal')
xlabel('samples')
ylabel('X_2')