% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% This exercise demonstrates the use of the Contribution Variables in 
% detecting the variables that stray from the normal pattern.
%
% The example uses the water quaity data from the Arno river upstream of the
% city of Florence and simulates a pH probe failure.
%
% Sect. 3.5.3.2
%
% *************************************************************************

clear
close all
clc

%% Load water quality data
load Ros_dip
[n p] = size(Ros_Data);
X = zeros(n,p);
Mr  =mean(Ros_Data);

for i = 1:p
    X(:,i) = Ros_Data(:,i) - Mr(i);
end

% Plot the original data
hh = figure(1);
set(hh, 'Position',[296 132 731 372])
clf
hh = subplot(311);
plot(Ros_Data(:,1), 'LineWidth',2);
set(gca, 'FontName','Arial', 'FontSize',12)
set(hh, 'XLim',[0 72])
set(gca, 'XTickLabels',[ ])
ylabel('DO [%]')
title('Original variables')

hh = subplot(312);
plot(Ros_Data(:,2), 'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
set(gca, 'XTickLabels',[ ])
ylabel('pH')
set(hh, 'XLim',[0 72])

hh = subplot(313),
plot(Ros_Data(:,3), 'LineWidth',2)
axis([0 72 20 26])
set(gca, 'FontName','Arial', 'FontSize',12)
set(gca, 'XTick',[0 12 24 36 48 60 72])
ylabel('T [C]')
set(hh, 'XLim',[0 72])

%% Standardize variables
Xz = zscore(Ros_Data);
[S, scores, eig, t2] = princomp(Xz); % Standardized data PCA
[Sd, scoresd, eigd, T2] = princomp(X); % Zero-mean Raw data PCA

%% Analysis of the PCA results
%  Scree plot
scree(eigd, 2)
%  Biplot
figure(3)
vl = {' DO',' pH',' T'};
biplot(S, 'Scores', scores, 'Varlabels',vl)

%% Contribution variables
L  = diag(sqrt(eigd));
CV = scoresd*inv(L)*Sd';

%  Plot contribution variables
hh = figure(4);
set(hh, 'Position',[296 132 731 372])
plot(CV(:,1), 'b', 'LineWidth',2)
hold on
plot(CV(:,2), '--b', 'LineWidth',2)
plot(CV(:,3), ':b', 'LineWidth',2)
legend('CV 1', 'CV 2', 'CV 3', 'orientation','horizontal',...
       'location','southeast')
a = axis;
axis([0 72 a(3) a(4)])
set(gca, 'XTick',[0 12 24 36 48 60 72])
set(gca, 'FontName','Arial', 'FontSize',12)
ylabel('CV')
xlabel('time [h]')
title('Contribution Variables')

% Plot PCA scores
hh = figure(5);
clf
set(hh, 'Position',[296 132 731 372])
plot(scoresd(:,1), 'b', 'LineWidth',2)
hold on
plot(scoresd(:,2), '--b', 'LineWidth',2)
plot(scoresd(:,3), ':b', 'LineWidth',2)
a = axis;
axis([0 72 a(3) a(4)])
set(gca, 'FontName','Arial', 'FontSize',12)
set(gca, 'XTick',[0 12 24 36 48 60 72])
legend('z_1', 'z_2', 'z_3', 'orientation','horizontal', 'location','south')
xlabel('time [h]')
ylabel('PCA scores')
title('PCA transformed variables')

hh = figure(6);
set(hh, 'Position',[296 132 731 372])
clf
plot(T2, 'b','LineWidth',2)
hold on
T_lim = (p*(n-1)/(n-p))*finv(0.95,p,n-p);
plot([0 72], [T_lim T_lim], '--k', 'LineWidth',2)
a=axis;
axis([0 72 a(3) a(4)])
set(gca, 'FontName','Arial', 'FontSize',12)
set(gca, 'XTick',[0 12 24 36 48 60 72])
xlabel('time [h]')
ylabel('T^2')
legend('T^2', 'T^2_{lim}', 'Location','northeast',...
       'orientation','horizontal')
title('Hotellng T^2')

%% Reconstruction
[Xres, Xr] = pcares(scoresd, 3);
for i = 1:p
    Xrr(:,i) = Xr(:,i) + Mr(i);
end
hh = figure(7);
clf
set(hh, 'Position',[21 58 731 569])
hh = subplot(311);
plot(Ros_Data(:,1), 'LineWidth',2);
hold on
plot(Xrr(:,1), ':b', 'LineWidth',2);
set(gca, 'FontName','Arial', 'FontSize',12)
set(hh, 'XLim',[0 72])
set(gca, 'XTickLabel',[ ])
ylabel('DO [%]')
legend('Original variables','Reconstructed variables',...
    'orientation','horizontal', 'location','northoutside')

hh = subplot(312);
plot(Ros_Data(:,2), 'LineWidth',2)
hold on
plot(Xrr(:,2), ':b', 'LineWidth',2);
set(gca, 'FontName','Arial', 'FontSize',12)
set(gca, 'XTickLabel',[ ])
ylabel('pH')
set(hh, 'XLim',[0 72])

hh = subplot(313);
plot(Ros_Data(:,3), 'LineWidth',2)
hold on
plot(Xrr(:,3), ':b', 'LineWidth',2);
axis([0 72 20 26])
set(gca, 'FontName','Arial', 'FontSize',12)
set(gca, 'XTick',[0 12 24 36 48 60 72])
xlabel('time [h]')
ylabel('T [C]')
set(hh, 'XLim',[0 72])