% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% This exercise demonstrates the Wavelet Denoising by several thresholding
% methods. Discrete Meyer wavelet is used.
%
% Sect. 3.4.2
%
% *************************************************************************

clear
close all
clc
%% Load data
load DO_apr
x = DO_apr;
N = length(x);
t = 0:length(x)-1;
wname = 'dmey'; % Discrete Meyer wavelet
lev = 3;

%% Wavelet decomposition
[C, L] = wavedec(x, lev, wname);
A2_base = wrcoef('a', C, L, wname, 2);
A1_base = wrcoef('a', C, L, wname, 1);
D2_base = wrcoef('d', C, L, wname, 2);
D1_base = wrcoef('d', C, L, wname, 1);

%% De-noise noisy signal using soft heuristic SURE thresholding
%  and scaled noise option, on detail coefficients obtained
%  from the decomposition of x.
%  RIGRSURE uses the principle of Stein's Unbiased Risk Estimate.
%  HEURSURE is an heuristic variant of  RIGRSURE.
%  With option 'MLN' rescaling is done using level dependent estimation of
%  level noise.
xd1 = wden(x, 'rigrsure', 's', 'mln', lev, wname);
xd2 = wden(x, 'heursure', 's', 'mln', lev, wname);

%% De-noise noisy signal using fixed form threshold 
%  with a single level estimation of noise standard deviation.
%  Universal threshold sqrt(2*log(.)).
%  With option 'MLN' rescaling is done using level dependent estimation of
%  level noise.
xd3 = wden(x, 'sqtwolog', 's', 'mln', lev, wname);

%% De-noise noisy signal using minimax threshold
%  with a multiple level estimation of noise standard deviation.
%  With option 'SLN' 
%  With option 'MLN' rescaling is done using a single estimation of level
%  noise based on first level coefficients.
xd4 = wden(x, 'minimaxi', 's', 'sln', lev, wname);

%% Plot signals.
figure(1)
set(1, 'Position',[26 424 628 350])

plot(x, 'ob', 'MarkerFaceColor','b', 'MarkerSize',2)
hold on
plot(xd1, ':b')
plot(xd2, '.-b', 'LineWidth',2)
plot(xd3, '--k', 'LineWidth',2)
plot(xd4, '-.k', 'LineWidth',2)
set(gca, 'FontName','Arial','FontSize',10)
legend('Data', 'rigrsure', 'heursure', 'sqtwolog', 'minimax',...
       'location','northoutside', 'orientation','horizontal')
xlabel('time [h]')
ylabel('DO [mg/L]')
xlim([0 720])

%% FFT comparison
figure(3)
set(3,'Position',[29 91 627 233])
Ts = 1; % Sampling period 1 hour

% Plot frequency analysis of original signal
Fs = Freq_spectrum(t, x, Ts, 3, 1);
set(gca, 'FontName','Arial', 'FontSize',10)
t24 = 1/24; % hourly sampling freq.
t12 = 1/12; % half-hourly sampling freq.
a = axis;
hold on
semilogy([t24 t24], [a(3) a(4)], ':k')
semilogy([t12 t12], [a(3) a(4)], ':k')

Fs = Freq_spectrum(t, xd1, Ts, 3, 2);
Fs = Freq_spectrum(t, xd2, Ts, 3,3);
Fs = Freq_spectrum(t, xd3, Ts, 3,4);
Fs = Freq_spectrum(t, xd4, Ts, 3,5);
legend('Data', 'rigrsure', 'heursure', 'sqtwolog', 'minimax',...
       'location','northoutside', 'orientation','horizontal')

% Comparison with Approximations
figure(2)
set(2, 'Position', [670 420 630 353])
clf
subplot(211)
plot(DO_apr, '.r', 'MarkerSize',8')
hold on
xlabel('time [h]')
ylabel('DO [mg/L]')
plot([A1_base; A2_base]')
xlim([0 720])
legend('Data', 'A_1', 'A_2', 'location','north',...
       'orientation','horizontal')
title('Approximations')

subplot(212)
% Plot frequency analysis of original signal
Fs = Freq_spectrum(t, x, Ts, 2, 1);
t24 = 1/24; % hourly sampling freq.
t12 = 1/12; % half-hourly sampling freq.
a = axis;
hold on
semilogy([t24 t24], [a(3) a(4)], ':k')
semilogy([t12 t12], [a(3) a(4)], ':k')


Fs = Freq_spectrum(t,xd1,Ts,2,2);
Fs = Freq_spectrum(t,xd2,Ts,2,3);
Fs = Freq_spectrum(t,xd3,Ts,2,4);
legend('Data', 'A_1', 'A_2', 'location','north',...
       'orientation','horizontal')

figure(4)
set(4,'Position',[673 93 627 232])
dx = deriv1(x);
dx1 = deriv1(xd1);
dx2 = deriv1(xd2);
dx3 = deriv1(xd3);
dx4 = deriv1(xd4);
plot(dx, 'b')
hold on
plot(dx1, ':b')
plot(dx2, '.-b', 'LineWidth',2)
plot(dx3, '--k', 'LineWidth',2)
plot(dx4, ':k', 'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',10)
xlim([0 720])
legend('Data', 'rigrsure', 'heursure', 'sqtwolog', 'minimax',...
       'location','northoutside', 'orientation','horizontal')
xlabel('time [h]')
ylabel('dDO/dt')