% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% This exercise demonstrates the thresholding alternatives 
% in Wavelet Denoising.
% Hard and Soft thresholding are compared in denoising a monthly record of
% DO data.
%
% Sect. 3.4.2 and Figures 3.34-3.35
%
% *************************************************************************

clear
close all
clc

load DO_apr
x = DO_apr(50:100);
t = 0:length(x)-1;
wname = 'dmey'; % Discrete Meyer wavelet
lev = 2;

% Plot signals.
figure(1)
set(1, 'Position',[26 494 622 280])
clf
plot(x, '.r', 'MarkerSize',8')
hold on
xlabel('time [h]')
ylabel('DO [mg/L]')
title('Denoising')

%% Wavelet decomposition
[C, L]  = wavedec(x, lev, wname);
A2_base = wrcoef('a', C, L, wname, 2);
A1_base = wrcoef('a', C, L, wname, 1);
D2_base = wrcoef('d', C, L, wname, 2);
D1_base = wrcoef('d', C, L, wname, 1);

mtd = 3;
switch mtd
    case 1
        method = 'rigrsure';
    case 2
        method = 'heursure';
    case 3
        method = 'sqtwolog';
    case 4
        method = 'minimaxi';
end
thr = thselect(x, method);
[xd3s, cs, ls] = wden(x, method, 's', 'sln', lev, wname);
[xd3h, ch, lh] = wden(x, method, 'h', 'sln', lev, wname);

% plot([A1_base; xd3s; xd3h]')
plot([A2_base; xd3s; xd3h]')
legend('Data', 'A2', 'Minimax soft', 'Minimax hard', 'location','north',...
       'orientation','horizontal')
title(method)

% Details
figure(2)
clf
set(2,'Position',[26 55 627 344])
subplot(211)
L1 = L(1) + L(2);
L2 = L1 + L(3);
plot(C(L1:L2), 'ob', 'MarkerFaceColor','b', 'MarkerSize',3)
hold on
plot(cs(L1:L2), 'sr', 'MarkerFaceColor','r', 'MarkerSize',5)
legend('Original D coeff', 'Thresholded coeff.', 'location','north',...
       'orientation','horizontal')
title('Soft thresholding')
subplot(212)
plot(C(L1:L2), 'ob', 'MarkerFaceColor','b', 'MarkerSize',3)
hold on
plot(ch(L1:L2), 'dr', 'MarkerFaceColor','r', 'MarkerSize',5)
legend('Original D coeff', 'Thresholded coeff.', 'location','north',...
       'orientation','horizontal')
title('Hard thresholding')
