% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% This exercise demonstrates the ANFIS approach to the Tank modelling with
% a first-order model.
%
% Sect. 4.4.3.3
%
% *************************************************************************

clear
close all
clc

%% Load data
load CSTR_Data

%% Data decimation
tic
decima = 0;
if decima == 1,
    no = length(t_dot); dec = 1;
    t = []; F = []; h = [];
    
    %% Data organization for ANFIS modelling
    for k=1:dec:no
        t=[t t_dot(k)]; F=[F F_dot(k)]; h=[h h_dot(k)];
    end
else
    t = t_dot; F =F_dot; h = h_dot;
end
save CSTR_data t_dot F_dot h_dot
clear  t_dot F_dot h_dot
n=length(F);
n1=round(n/2);
for k=4:n
    Data(k-3,:) = [F(k-1) h(k-1) h(k)];
end

%% Split the data between training and checking
TrnData = Data(1:n1,:);
chkData = Data(n1+1:end,:);

%% Select the ANFIS structure
numMFs = 3;
mfType = 'gauss2mf';
epoch_n = 30;
trnOpt = [epoch_n 0 0 0.98 1.12];
in_fis = genfis1(TrnData,numMFs,mfType);

%% Generate the ANFIS Tank model
[CSTR_Sug1,error1] = anfis(TrnData,in_fis,trnOpt,[],chkData,1);

%% Save the ANFIS to be used in the Simulink model
writefis(CSTR_Sug1, 'CSTR2_Sug1');
toc

%% Figures
figure(1)
set(1, 'Position',[26 42 669 379])
clf
h_anfis = evalfis([TrnData(:,1:2); chkData(:,1:2)],CSTR_Sug1);
index = 5:n1;
% Plot training data
plot(t(index), h(index), 'ob', 'MarkerFaceColor','b', 'MarkerSize',3)   
hold on
plot(t(index), h_anfis(index-3), 'b');
index1 = n1+1:n;
 % Plot Checking data
plot(t(index1), h(index1), 'ob', 'MarkerFaceColor','b', 'MarkerSize',3)
plot(t(index1), h_anfis(index1-3), '-.b');
set(gca, 'FontName','Arial', 'FontSize',14)
xlabel('time [min]')
ylabel('h [m]')
legend('Train h', 'ANFIS Train h', 'Valid h', 'ANFIS Valid h',...
       'location','northoutside', 'orientation','horizontal');
a = axis;
plot([t(n1) t(n1)], [a(3) a(4)], ':k')

figure(2) % Plot the modelling error
set(2, 'Position',[714 41 669 379])
clf
nh = length(h_anfis);
dh = h(3:nh) - h_anfis(3:nh)';
plot(t(3:nh), dh, '.-b')
sse = sum(dh.^2); % Total model error
set(gca, 'FontName','Arial', 'FontSize',14)
xlabel('time [min]')
ylabel('{\Delta}h [m]')
title(['Sum of Squared Errors = ',num2str(sse)])