% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Fuzzy approximation of a mathematical function defined point-wise by
% (x,y) couples with a Sugeno Fuzzy Inference System.
% The antecedents are fixed and the consequents are estimated according to
% a least-squares criterion.
%
% Sect. 4.3.3
%
% *************************************************************************

clear
close all
clc

%% Define global parameters for the simulation
global Mu x y n ym y_fit

%% Load the data to be approximated
load Fun_up_down

figure(1)
clf
set(1,'Position',[593 60 550 350])
subplot(211)
plot(Mis_exp(:,1), Mis_exp(:,2), 'o','MarkerFaceColor','b')
xlabel('x')
ylabel('y')
subplot(212)
xx = 0:0.05:10;
Pm1 = [1.36 -0.4 1.19 0.78];
Pm2 = [0.793 2.51 0.966 3.53];
Pm3 = [0.941 5.804 1.36 10.4];

%% Trace the membership functions
a1 = gauss2mf(xx, Pm1);
a2 = gauss2mf(xx, Pm2);
a3 = gauss2mf(xx, Pm3);
plot(xx,a1, xx,a2, xx,a3)

%% Compute the degree of truth of the given data (x) to the mfs
x = Mis_exp(:,1);
y = Mis_exp(:,2);
n = length(x);
Mu = zeros(3,n); % Mu is the matrix of the degrees of truth
Mu(1,:) = gauss2mf(x, Pm1);
Mu(2,:) = gauss2mf(x, Pm2);
Mu(3,:) = gauss2mf(x, Pm3);

figure(1)
hold on
subplot(212)
plot(x,Mu','o')
xlabel('x')
ylabel('Degree of truth')

%% Generate initial linear consequents
i1 = find(x < 1.8);
p1 = polyfit(x(i1), y(i1), 1);
y1 = polyval(p1, x(i1));
i2 = find((x > 2.2) & (x < 5));
p2 = polyfit(x(i2), y(i2), 1);
y2 = polyval(p2, x(i2));
i3 = find(x > 5);
p3 = polyfit(x(i3), y(i3), 1);
y3 = polyval(p3, x(i3));

figure(1)
subplot(211)
hold on
plot(x(i1), y1, '--r', 'LineWidth',2)
plot(x(i2), y2, '--r', 'LineWidth',2)
plot(x(i3), y3, '--r', 'LineWidth',2)
legend('Data','Initial approx. lines','location','north', 'orientation','horizontal')

%% Simplex optimization
Par = [p1; p2; p3];     % Consequent parameters
options(2) = 0.01;      % Termination tolerance for the solution
options(3) = 0.01;      % Termination tolerance on function
options(14) = 10000;    % Maximum number of function evaluations
ParBest = simplex('Ferr_fun', Par, options)

figure(2)
set(2, 'Position',[24 60 550 350])
hold on
y1f = polyval(ParBest(1,:), x(i1));
y2f = polyval(ParBest(2,:), x(i2));
y3f = polyval(ParBest(3,:), x(i3));
plot(x(i1), y1f, '--k', 'LineWidth',2)
plot(x(i2), y2f, '--k', 'LineWidth',2)
plot(x(i3), y3f, '--k', 'LineWidth',2)
xlabel('x')
ylabel('y')
legend('Data', 'Fit','Final approx. lines','location','north', 'orientation','horizontal')