% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
%  This exercise demonstrates the automatic generation of membership
%  functions starting from experimental mfs generated by FCM data clustering.
%  The user may chose to process two data sets:
%  1 - Turbidity data from a potabilization plant
%  2 - Dissolved oxygen data from the Orbetello lagoon
%
%  Alice & SML, March 2011
%  Revised by  El Basri & Plotegher, Oct. 2011
%
%  Sect. 4.5.7
%
% *************************************************************************

clear
close all
clc

%% Data loading
data_type = 1;
switch data_type
    case 1
        load Turbidity
        X = Turb;
        C_labels = 'Turb [FTU]';
    case 2
        load DO_apr
        X = DO_apr';
        C_labels = 'DO [mg/L]';
end
[n p] = size(X);

% Plot data
figure(2)
set(2,'Position',[649 61 618 420])
set(gca, 'FontName','Arial', 'FontSize',12)
st = 1:length(X);
plot(st, X)
ylabel(C_labels)

%% Cluster parameters
m = input('Set fuzzy exponent: ');
c = input('Set number of clusters: ');

% Create the cell array where the mf data are stored one for each variable
Memb = cell(p,1);
for i=1:p
    x = X(:,i);
    mu = [];
    Xz = [];
    [ui, vvi] = FCmeans(x, m, c); % FCM clustering of the one-dimensional variables
    vi = sort(vvi);               % Sort the centroids from the smallest to the largest
    x1 = vi(1); x2 = vi(end);     % Extreme values to generate the support vector
    for xz = x1:(x2-x1)/50:x2;    % Generate the support vector
        uz = classif(xz, vi, m);  % Numerical generation of memberships
        mu = [mu uz];
        Xz = [Xz xz];
    end
    MMu = [mu; Xz];   % Pack membership and support
    Memb{i} = {MMu};  % Memb is a cell array with the numerical mf data
    % Plot numerical memberships
    figure(1)
    set(1, 'Position',[13 61 618 420])
    subplot(p, 1, i)
    for j = 1:c
        Plot.numerical = plot(Xz,mu(j,:),'b');
        hold on
        Plot.centroids = plot(vi(j),1,'sb','MarkerFaceColor','b');
        xlabel(C_labels(i,:))
        axis tight
    end
end

%% Fitting mf's to the data
for w = 1:c
    if w == 1
        ord = 1;     % Left sigmoid membership (zmf)
    elseif w == c
        ord = 3;     % Right sigmoid membership (smf)
    else
        ord = 2;     % Gauss2 membership (gauss2mf)
    end
    [est, model] = fit_mf(Xz, mu(w,:), ord, c);
    Param{i,w} = {est}; % Load the cell with the parameter vector
    dXz = Xz(2) - Xz(1);
    Xzz = [0.85*Xz(1):dXz:Xz(1)-dXz, Xz, Xz(end)+dXz:dXz:1.15*Xz(end)];
    
    figure(1)
    subplot(p,1,i)
    if ord == 2,
        y_fit = gauss2mf(Xzz, est); % Fit the middle mfs
    end
    if ord == 1,
        y_fit = zmf(Xzz, est);      % Fit the left mf
    end
    if ord==3,
        y_fit = smf(Xzz, est);      % Fit the right mf
    end
    Plot.fitted = plot(Xzz, y_fit, ':r');
    set(gca, 'FontName','Arial', 'FontSize',12)
    xlabel(C_labels(i,:))
    ylabel('mf')
    axis tight
    drawnow
    hold on
end
legend([Plot.numerical, Plot.centroids, Plot.fitted],...
       'mf data', 'Centroids','Fitted mf', 'location','northoutside',...
       'orientation','horizontal')

%% Figure
figure(2)
set(2,'Position',[649 61 618 420])
set(gca, 'FontName','Arial', 'FontSize',12)
st = 1:length(X);
plot(st, X)
ylabel(C_labels)
hold on
for i = 1:c
    plot([st(1) st(end)],[vi(i) vi(i)], ':r')
end
axis tight
legend('Data','Centroids', 'location','north', 'orientation','horizontal')
xlabel('time (h)')
ylabel(C_labels)