function [estimates, model] = fit_mf(xdata, ydata, ord, c)
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Function to fit an analytical mf to the FCM-generated cluster data.
%
% *************************************************************************
%
%      [estimates, model] = fit_mf(xdata,ydata,ord,c)
%
%    Input:
%           xdata         Support of the membership.
%
%           ydata         Numerical membership values (from FCmeans.m).
%
%           ord           Position of the memmbership function:
%                          - ord = 1 --> Left membership.
%                          - ord = 2 --> Middle membership.
%                          - ord = 3 --> Right membership.
%
%           c             Number of clusters.
%
%    Output:
%           estimates     Estimated parameters of the fitted membership
%                         function.
%
%           model         Type of Matlab predefined membership funtion used
%                         for the fitting.
%
n = length(xdata);

if ord == 2,  % Estimate middle mf
    c1 = 0.9*mean(xdata); % Initial parameters estimates
    c2 = 1.1*mean(xdata);
    sig1 = 0.5*var(xdata);
    sig2 = 0.7*var(xdata);
    start_point = [sig1 c1 sig2 c2]; % Form the initial parameters vector
    model = @gausmf; % Define the function handle
end
if ord == 1   % Estimate left mf
    dx = abs(max(xdata) - min(xdata));
    a = 0.1*min(xdata);
    g = min(xdata)+0.5*dx;
    start_point = [a g]; % Form the initial parameters vector
    model = @zmf_est;    % Define the function handle
end
if ord == 3,  % Estimate right mf
    dx = abs(max(xdata) - min(xdata));
    a = min(xdata)+0.5*dx;
    g = 0.95*max(xdata);
    start_point = [a g]; % Form the initial parameters vector
    model = @smf_est;    % Define the function handle
end

%% Actual parameters optimization
options = optimset('MaxIter', 1e8, 'MaxFunEvals',1e8, 'TolFun',1e-6,...
                   'TolX',1e-6);
estimates = fminsearch(model, start_point, options);
% Model accepts curve parameters as inputs, and outputs sse,
% the sum of squares error between data and fitted curve.
% FMINSEARCH only needs sse, but we want to
% plot the FittedCurve at the end.

%% Estimation of the middle mf
%  Objective function for parameter fitting
    function [sse, FittedCurve] = gausmf(par)
        % Fit the data to the selected analytical mf (gauss2mf)
        FittedCurve = gauss2mf(xdata, par);
        % Difference between numerical and analytical mfs
        ErrorVector = FittedCurve - ydata;
        % Sum of squared errors
        sse = sum(ErrorVector.^2);          
    end

%% Estimation of the left mf
%  Objective function for parameter fitting
    function [sse, FittedCurve] = smf_est(par)
        % Fit the data to the selected analytical mf (smf)
        FittedCurve = smf(xdata, par);
        % Difference between numerical and analytical mfs
        ErrorVector = FittedCurve - ydata;
        % Sum of squared errors
        sse = sum(ErrorVector.^2);
    end

%% Estimation of the right mf
%  Objective function for parameter fitting
    function [sse, FittedCurve] = zmf_est(par)
        % Fit the data to the selected analytical mf (zmf)
        FittedCurve = zmf(xdata, par);
        % Difference between numerical and analytical mfs
        ErrorVector = FittedCurve - ydata;
        % Sum of squared errors
        sse = sum(ErrorVector.^2);
    end

end