% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Demonstration of the constant harvesting policy by selecting the
% harvesting effort (h) and the intial harvesting time. 
% The fertility coefficient was adjusted to obtain a stable population.
%
% Sect. 5.3.1
%
% *************************************************************************

clear
close all
clc

r  = 0.2;
K  = 100;
x0 = 5;
h  = 2.5;

H_msy = r*K/4;             % Maximum Sustainable Yield (MSY)
disp('Maximum Sustainable Yield (MSY)')
fprintf('%4.2f\n', H_msy)
rr = roots([1 -K K*h/r]);  % Equilibria
disp('Equilibria')
fprintf('%4.2f\n', rr)
Tstep = input('Initial harvesting time Tin = ');
if Tstep < 6,
    Tstep = 6;
end
tend = 100;

%% Simulation
[t, x, y] = sim('harvest', tend);

%% Figures
figure(1) % Derivative plotting
set(1, 'Position',[18 40 560 420])
set(gca, 'FontName','Arial', 'FontSize',12)
xf1 = 0:K/20:K;
dx = r*xf1.*(1-xf1./K);
Plot.curve=plot(xf1,dx,'LineWidth',2);
hold on
plot([0 K], [h h], ':w')
axis(axis)
xlabel('Population')
ylabel('x \cdot F(x)')
ax = axis;
plot([ax(1) ax(2)], [h h], ':k')

x  = Out(:,1);
x1 = Out(:,2);
N  = length(x);
for i = 1:N
    if t(i) > Tstep
        % Plot after harvesting starts
        Plot.after=plot(x(i), x1(i), 'ob', 'MarkerFaceColor','b', 'MarkerSize',6);
    else
        % Plot before harvesting starts
        Plot.before=plot(x(i), x1(i), 'sr', 'MarkerFaceColor','r', 'MarkerSize',6);
    end
    hold on
end
legend([Plot.curve,Plot.before',Plot.after],...
    'Growth curve','Samples before harvest','Samples after harvest',...
    'location','south')
figure(2)
set(2, 'Position',[598 39 560 420])
set(gca, 'FontName','Arial', 'FontSize',12)
plot(t, x)
hold on
plot([Tstep Tstep], [0 K], ':k')
text(Tstep,0.2*K,['  Harvest starts at T = ',num2str(Tstep)])
xlabel('time')
ylabel('Population')