% *************************************************************************
% 
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Simulation of a pasture-herbivores ecosystem
% using Kolmogorov Prey-Predator model.
%
% Sect. 5.5.3.1
% 
% *************************************************************************

clear
close all
clc

%% Load rainfall data
load Rainfall
a = mean(Rainfall(:,2));
tend = max(Rainfall(:,1));
Limit = 2500;
Decim = 10;
r = 0.1;
s = 0.12;
q = 0.4;
p = 0.4;
Mu = 1.6;
Kv = 180;
sigma = 5; % Rainfall characteristics
Tdelay = 5;  

Vo = Kv*(r/s)/(Mu-(r/s));        % Equilibrium Vegetation
Ho = (a-p*Vo)*(Kv+Vo)/(q*Mu*Vo); % Equilibrium Herbivores
v = [5:5:1200];                  % Computing isoclines
ho = (a-p*v).*(Kv+v)./(q*Mu*v);  % Trace Phase Plane and Isoclines

%% Simulation
Vego  = 20; % Initial conditions
Herbo = 10;
Start = 10;
[y, x, t] = sim('Veg_Herb', tend);
Veg = Veg(Start:length(Veg));
Herb = Herb(Start:length(Herb));
rain = rain(Start:length(rain));
tsim = tsim(Start:length(tsim));

%% Figures
figure(1)
set(1, 'Position',[18 40 560 420])
plot(v, ho, ':r')
axis([100 350 0 200])
limits = axis;
axis(axis);
hold on
Hmin = limits(3);
Hmax = limits(4);
plot([Vo Vo], [Hmin Hmax], ':b')
plot(Veg(10), Herb(10), 'ob')
plot(Vo, Ho, 'ob', 'MarkerFacecolor','b', 'MarkerSize',6)
plot(Veg(10:end), Herb(10:end), 'b')
title('Population Equilibrium')
xlabel('Vegetation')
ylabel('Herbivores')
legend('Vegetation', 'Herbivores', 'Initial state', 'Equilibrium state',...
       'Trajectory')

figure(2) % Time plots
set(2, 'Position',[598 39 560 420])
subplot(211)
plot(tsim(10:end), rain(10:end), ':k')
ylabel('Rainfall [mm/week]')
subplot(212)
plot(tsim(10:end),Veg(10:end),'r', tsim(10:end),Herb(10:end),'b')
xlabel('time [weeks]')
ylabel('Densities')
legend('Vegetation', 'Herbivores', 'location','northoutside',...
       'orientation','horizontal')