% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Demonstration of the generalized 
% Kolmogorov Prey-Predator model
%
% Sect. 5.5.3
%
% *************************************************************************

clear
close all
clc

disp('Select the kind of behaviour')
rsp = input('Slow-fast response (1), Oscillatory stable response (2) ');
switch rsp
    case 1 % Monotonic stable response
        r  = 1;
        Kx = 15;
        p  = 4.2;
        n  = 1.05;
        Kp = 7;
        A  = 5;
        c  = 0.1;
        s  = 0.2;
        tend = 80;
    case 2 % Oscillatory stable response
        r  = 1;
        Kx = 15;
        p  = 1;
        Kp = 7;
        A  = 5;
        n  = 1;
        c  = 0.1;
        s  = 0.2;
        tend = 300;
end
Pred_o = 5; % Initial conditions
Prey_o = 8;
i = 0;

for x=0.05:0.25:Kx;
    i=i+1;
    xx(i)=x;
    sy=(Kp^n+x^n)/(x^(n-1));
    y1(i)=(r/A)*(1-(x/Kx)^p)*sy;
end;

%% Equilibrium value
disp('-------------------')
disp('Equilibrium point')
xstar = Kp*((s/(c*A-s))^(1/n))
sy = (Kp^n+xstar^n)/(xstar^(n-1));
ystar = (r/A)*(1-(xstar/Kx)^p)*sy
disp('-------------------')

%% Jacobian
J = zeros(2,2);
J(1,1) = r*(1-(p+1)*(xstar/Kx)^p)...
    -A*ystar*n*(Kp^n)*xstar^(n-1)/((Kp^n+xstar^n)^2);
J(1,2) = -A*xstar^n/(Kp^n+xstar^n);
J(2,1) = c*A*ystar*n*(Kp^n)*xstar^(n-1)/((Kp^n+xstar^n)^2);
J(2,2) = 0;
disp('Jacobian')
J
Eigenvalues = roots([1 -(J(1,1)+J(2,2)) J(1,1)*J(2,2)-J(1,2)*J(2,1)])

%% Simulation
[t, x, y] = sim('Kolmogoro', tend);
x_fin = prey(end);
y_fin = pred(end);

%% Figures
figure(1)
set(1, 'Position',[18 40 560 420])
plot(prey, pred,'LineWidth',2)
hold on
a = axis;
yz=[0 a(4)];
hold on
plot([xstar xstar], yz, ':r','LineWidth',2) % Predator isocline
plot(xx, y1, ':b','LineWidth',2)            % Prey isocline
xlabel('Prey')
ylabel('Predator')
plot(Prey_o,Pred_o,'ob')
plot(x_fin, y_fin, 'ob', 'MarkerFacecolor','b', 'MarkerSize',6)
legend('Trajectory', 'Predator isocl.', 'Prey isocl.', 'Start', 'End',...
    'location','northoutside', 'orientation','horizontal')

figure(2)
set(2,'Position',[598 39 560 420])
plot(t,prey,'b',t,pred,'--r','LineWidth',2)
legend('Prey', 'Predator','location','northoutside','orientation','horizontal')