% *************************************************************************
% 
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Demonstration of the Lotka-Volterra Prey-Predator model with external
% food supply to avoid the predator extinction.
%
% Sect. 5.5.1.1
% 
% *************************************************************************

clear
close all
clc

%% Extinction parameters
Fmax = 15; 
r    = 1.4;
m    = 4.6;
K    = 10;
b12  = 0.3;
b21  = 0.4;
tfin = 5; 

%% Equilibrium 
Y = [];
x_equil = m /b21
y_equil = input('Insert desired predator population: ');
F = b12*x_equil*y_equil - r*x_equil*(1-x_equil/K)


%% Jacobian
J(1,1) = r*(1-2*x_equil/K)-b12*y_equil;
J(1,2) = -b12*x_equil;
J(2,1) = b21*x_equil;
J(2,2) = 0;
disp('Eigenvalues')
eig(J)

%% Figure
figure(1)
set(1, 'Position',[18 40 560 420])
x = 0.01:K/50:1.5*K;           % Prey isocline
y = (r*x.*(1-x/K)+F)./(b12*x); % Prey isocline with additional food
yo = (r*x.*(1-x/K))./(b12*x);  % Original Prey isocline
plot(x,yo,'b', x,y,':b','LineWidth',2)
axis([0 1.5*K 0 3*y_equil])
hold on
plot([m/b21 m/b21], [0 3*y_equil], ':r','LineWidth',2) % Predator isocline
legend('Original Prey isocl.', 'Prey isocl. with food','Predator isocl.',...
       'location','northoutside', 'orientation','horizontal')
plot(x_equil, y_equil, 'ob', 'MarkerFacecolor','b', 'MarkerSize',6)
TTL = title('Click to select starting point');
[Xo, Yo] = ginput(1);
plot(Xo, Yo, 'ob')
delete(TTL); clear TTL
xlabel('Prey')
ylabel('Predator')

%% Simulation
tend = 8;
[t,x,y] = sim('LV', tend);
plot(Xv, Yv, 'b')

%% Figure
figure(2)
set(2, 'Position',[598 39 560 420])
plot(t,Xv,'b', t,Yv,':r','LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
xlabel('time')
ylabel('Populations')
legend('Prey', 'Predator')