% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Demonstration of the Lotka-Volterra Prey-Predator model with three
% possible outcomes
% 1 - Prey/Predator coexistence with oscillatory response
% 2 - Prey/Predator coexistence with monotonic response
% 3 - Predator extinction 
%
% Sect. 5.5.1
%
% *************************************************************************

clear
close all
clc

F = 0;
disp('Select the kind of behaviour')
rsp = input('Oscillatory response (1), Monotonic response (2), Predator extinction (3)');
switch rsp
    case 1 % Oscillatory response parameters
        r    = 0.4;
        m    = 0.5;
        K    = 20;
        b12  = 0.33;
        b21  = 0.1;
        tend = 80;
        Xo   = 10;
        Yo   = 2;
    case 2 % Monotonic response parameters
        r    = 30;
        m    = 5;
        K    = 18;
        b12  = 1.5;
        b21  = 0.5;
        tend = 1;
        Xo   = 20;
        Yo   = 25;
    case 3 % Extinction parameters
        r    = 1.4;
        m    = 4.6;
        K    = 10;
        b12  = 0.3;
        b21  = 0.4;
        tend = 5;
        Xo   = 18;
        Yo   = 8;
end

%% Equilibrium
disp('Equilibrium')
x_equil = m/b21
y_equil = (r/b12)*(1-x_equil/K)

%% Jacobian and eigenvalues
J = zeros(2,2);
J(1,1) = -m*r/(b21*K);
J(1,2) = -m*b12/b21;
J(2,1) = (b21*r/b12)*(1-m/(b21*K));
disp('Eigenvalues')
Jacob = [1 m*r/(b21*K) m*r*(1-m/(b21*K))];
eig(J)

%% Simulation
[t, x, y] = sim('LV', [0 tend]);

figure(1)
set(1, 'Position',[18 40 560 420])
plot([0 K], [r/b12 0], 'b','LineWidth',2) % Prey isocline
set(gca, 'FontName','Arial', 'FontSize',12)
Xlim = max(Xo,K);
Ylim = max(Yo,r/b12);
axis([0 1.2*Xlim 0 1.6*Ylim]);
limite = axis
hold on
axis(axis)
plot([m/b21 m/b21], [0 limite(4)], '--r','LineWidth',2) % Predator isocline
plot(Xv, Yv, 'b','LineWidth',2)
plot(Xo, Yo, 'ob','LineWidth',2)
text(1.03*Xo, Yo, '(X_0, Y_0)', 'fontsize',12)
xlabel('Prey')
ylabel('Predator')
legend('Prey', 'Predator')

figure(2)
set(2, 'Position',[598 39 560 420])
plot(t,Xv,'b', t,Yv,':r','LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
a = axis;
axis([a(1) a(2) a(3) a(4)*1.3])
xlabel('time')
ylabel('Populations')
legend('Prey', 'Predator')