% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Demonstration of the Leslie age class model where
% the fertility coefficients were adjusted to obtain
% a stable population
%
% Sect. 5.2.2.1
%
% *************************************************************************

clear
close all
clc

Nclass = 5;
Xold = [0 0 5 3 1]';
s(1) = 0.69;
s(2) = 0.5;
s(3) = 0.4;
s(4) = 0.17;
s(5) = 0.15; % Survival rates
P = 0;
for i = 1:Nclass
    P = P + prod(s(1:i))
end
for i = 1:Nclass
    pai(i) = prod(s(1:i))/P;
end
% Fertility coefficients adjustment
% suppose that f(1) = f(5) = 0
% the central fertilities are symmetrical: f(2) = f(4)
% the middle age class has maximum fertility: f(3) = 2f(2)
f(2) = 1/(P*(pai(2)+2*pai(3)+pai(4)));
f(3) = 2*f(2);
f(4) = f(2);
f(1) = 0;
f(5) = 0;
%f1 = 0.45; f2 = 0.8; f3 = 0.73; f4 = 0.2; f5 = 0.02; % Fertility
A = [s(1)*f(1), s(1)*f(2), s(1)*f(3), s(1)*f(4), s(1)*f(5) ;
          s(2),         0,         0,         0,         0 ;
             0,      s(3),         0,         0,         0 ;
             0,         0,      s(4),         0,         0 ;
             0,         0,         0,      s(5),         0];
eig(A)
Tmax = 15;
Xnew = zeros(Nclass, Tmax+1);
Xnew(:,1) = Xold;
for i = 2:Tmax+1;
    Xnew(:,i) = A*Xold;
    Xold = Xnew(:,i);
end;
t = 0:Tmax;

%% Figures
figure(1)
set(1, 'Position',[18 40 560 420])
set(gca, 'FontName','Arial', 'FontSize',12)
subplot(211)
bar(pai)
ylabel('Stationary distribution')
xlabel('Age class')
subplot(212)
bar(f)
ylabel('Fertility')
xlabel('Age class')

figure(2)
set(2, 'Position',[598 39 560 420])
set(gca, 'FontName','Arial', 'FontSize',12)
bar3(Xnew')
colormap spring
grid on
ylabel('Time')
xlabel('Age group')
zlabel('Individuals in each group')
title('Leslie model for age-group population')
rotate3d on