% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Demonstration of the Leslie age class model
% applied to Great tit (Parus major) census data
% The fertility coefficient was adjusted to obtain
% a stable population
%
% Sect. 5.2.2.2
%
% *************************************************************************

clear
close all
clc

Nclass = 6;
% Survival rates
s(1) = 1000;
s(2) = 90;
s(3) = 60;
s(4) = 20;
s(5) = 9;
s(6) = 6;
s(7) = 1;
s(8) = 0;
fert = 10.416; % Fertility coefficient which makes the population stable
% Fertility
f(1) = 0;
f(2) = fert;
f(3) = fert;
f(4) = fert;
f(5) = fert;
f(6) = fert;
f(7) = fert;
f(8) = fert;
P = 0;

%% Computation of survivals
for i = 1:Nclass
    sigma(i) = s(i+1)/s(i);
end

%% Initial population
Xold = [100 0 0 0 0 0]';

%% Leslie matrix
A = [sigma(1)*f(1), sigma(1)*f(2), sigma(1)*f(3), sigma(1)*f(4), sigma(1)*f(5), sigma(1)*f(6);
     sigma(2),      0,             0,             0,             0,             0            ;
     0,             sigma(3),      0,             0,             0,             0            ;
     0,             0,             sigma(4),      0,             0,             0            ;
     0,             0,             0,             sigma(5),      0,             0            ;
     0,             0,             0,             0,             sigma(6),      0           ];

%% Compute the eigenvalues
eig(A)
Tmax = 25;

XX = [];
XX(1,:) = Xold;

%% Model simulation
for i = 2:Tmax+1;
    Xnew = A*Xold;
    XX = [XX; Xnew'];
    Xold = Xnew;
end;

t = 0:Tmax;
XX = fliplr(XX);

%% Figures
figure(1)
set(1, 'Position',[18 40 560 420])
set(gca, 'FontName','Arial', 'FontSize',12)
subplot(211)
bar(sigma)
xlabel('Age class')
ylabel('Survival coefficients')
subplot(212) % Display the last population distribution
bar(Xnew)
xlabel('Age class')
ylabel('Females in each class')
title('Stable age distribution')

figure(2)
set(2, 'Position',[598 39 560 420])
set(gca, 'FontName','Arial', 'FontSize',12)
bar3(t, XX)
colormap spring
grid on
ylabel('time')
xlabel('Age class')
zlabel('Females in each class')
title('Leslie model for the Great tit')
axis tight
rotate3d on