% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Demonstration of the period-doubling property of the discrete-time
% Logistic equation.
%
% Iterative computation of the equilibrium of the difference equation
%       x(t+1) = r.x(t).(1-x(t)/a)
% for r = 1 only the x = 0 equilibrim is possible because thethe curve is
% tangent to the equilibrium line.
% The iteration converges only if xo > 0.
%
% Sect. 5.2.1.1 
%
% *************************************************************************

clear
close all
clc

resp = input('Selecty kind of response (1 - 5): ' );
if resp == 1
    r = 1.4;         % Monotonic
    disp('Monotonic response')
end
if resp == 2
    r = 3;           % Oscillatory
    disp('Oscillatory response')
end
if resp == 3
    r = 3.5;         % Limit cycle
    disp('Limit cycle')
end
if resp == 4
    r = 1.1+sqrt(6); % Period doubling
    disp('Period doubling')
end
if resp == 5
    r = 3.8;         % Chaotic behaviour
    disp('Chaotic behaviour')
end
x = 0.1;
toll = 1e-3;
err = 1;
k = 1;
z = 0:0.01:1;
y = (r*r)*z .*(1-(r+1)*z+2*r*(z.^2)-r.*(z.^3));

%% Figures
figure(1)
set(1, 'Position',[18 40 560 420])
set(gca, 'FontName','Arial', 'FontSize',14)
plot(z, z, '-.k')
axis([0 1.0 0 1.0]);
hold on
plot(z, y, 'LineWidth',3)
yo = (r*r)*x*(1-(r+1)*x+2*r*(x^2)-r*(x^3));
plot([x x], [0 yo], 'b')
xlabel('z_k')
ylabel('z_{k+1}')
IT_max = 30;
Y(1)=x;
while err > toll && k < IT_max
    k = k + 1;
    x_new = (r*r)*x*(1-(r+1)*x+2*r*(x^2)-r*(x^3));
    Y(k) = x_new;
    err = abs(x_new - x);
    plot(x, x_new, 'b')
    plot([x x_new], [x_new x_new], 'b')
    if k > 1
        plot([x x], [x x_new], 'b');
    end
    x = x_new;
end
text(0.7, 0.18, ['z_{equil.} = ', num2str(x,'%6.3f')],...
    'FontName','Arial', 'FontSize',12);
text(0.7, 0.1, ['\alpha = ', num2str(r,'%6.3f')],...
    'FontName','Arial', 'FontSize',12);

figure(2) % Time behaviour
set(2, 'Position',[598 39 560 420])
set(gca, 'FontName','Arial', 'FontSize',12)
plot(Y)
a = axis;
axis([0 a(2) 0 1])
xlabel('Samples')
ylabel('Norm. Pop.')