% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Demonstration of the discrete-time Logistic equation with differing
% behavious.
%
% Iterative computation the difference equation
%       x(t+1) = r.x(t).(1-x(t)/a)
% The iteration converges to the equilibrium only if xo > 0.
%
% Sect. 5.2.1.1 
%
% *************************************************************************

clear
close all
clc

resp=input('Select kind of response (1 - 5): ' );
if resp == 1
    r = 1.5;         % Monotonic
    disp('Monotonic response')
end
if resp == 2
    r = 2.85;        % Oscillatory
    disp('Oscillatory response')
end
if resp == 3
    r = 3.133;       % Limit cycle
    disp('Limit cycle')
end
if resp == 4
    r = 1.1+sqrt(6); % Period doubling
    disp('Period doubling')
end
if resp == 5
    r = 4;           % Chaotic behaviour
    disp('Chaotic behaviour')
end

x = 0.2;
toll = 1e-4;
err = 1;
k = 0;
z = 0:0.01:1;
y = r.*z.*(1-z);

%% Figures
figure(1)
set(1, 'Position',[18 40 560 420])
set(gca, 'FontName','Arial', 'FontSize',12)
plot(z, y,'b','LineWidth',2)
hold on
plot(z, z, ':k')
axis([0 1.0 0 1.0]);
yo = r*x*(1-x);
plot([x x], [0 yo], 'r')
xlabel('x(k)')
ylabel('x(k+1)')
TTL = title('Press the space bar to advance the simulation by one step');
IT_max = 20;
while err > toll && k < IT_max
    k = k + 1;
    x_new = r*x*(1-x);
    Y(k) = x_new;
    err = abs(x_new - x);
    plot(x, x_new, 'or','MarkerFaceColor','r')
    plot([x x_new], [x_new x_new], 'r')
    if k > 1
        plot([x x], [x x_new], 'r');
    end
    drawnow
    x = x_new;
    pause
end
delete(TTL); clear TTL
title('Discrete-time Logistic iterations')
li = axis;
set(gca, 'FontName','Arial', 'FontSize',12)
text(0.04*li(2), 0.95*li(4), ['Equilibrium    = ', num2str(x)]);
text(0.04*li(2), 0.88*li(4), ['Growth rate r = ', num2str(r)]);
pause

figure(2)
set(2, 'Position',[598 39 560 420])
set(gca, 'FontName','Arial', 'FontSize',12)
plot(Y, '-ob','MarkerFaceColor','b')
xlabel('time')
ylabel('Population')
title('Discrete-time Logistic behaviour in the time domain')