% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Demonstration of the Reicker discrete-time Logistic equation with
% differing behaviours depending on Rmax.
%
% Sect. 5.2.1.2
%
% *************************************************************************

clear
close all
clc

resp=input('Select kind of response (1 - 3): ' );
switch resp
    case 1
        Rmax_s = 6.8;  % Damped oscillation
        disp('Damped oscillation')
    case 2
        Rmax_s = 8;    % Limit cycle
        disp('Limit cycle')
    case 3
        Rmax_s = 15.9; % Chaotic behaviour
        disp('Chaotic behaviour')
end

K = 25; % Carrying capacity
N = 0:1:2*K;
Rr = Rmax_s.^(1-N./K);
N1 = Rr.*N;
x = 8; % Initial population

%% Figures
figure(1)
set(1, 'Position',[18 40 560 420])
set(gca, 'FontName','Arial', 'FontSize',12)
plot(N, N1, 'b', 'LineWidth',2)
hold on
plot(N, N, ':k')
xlabel('z_k')
ylabel('z_{k+1}')
TTL = title('Press the space bar to advance the simulation by one step');
set(gca, 'FontName','Arial', 'FontSize',12)

% Scramble iterations
e  = 0.05;
xo = 0;
it = 1;
IT_max = 20;

Xs = zeros(1, IT_max);
Xs(1) = x;
x1 = x*Rmax_s^(1-x/K);
plot([x x], [0 x1])
while abs(xo - x) > e && it < IT_max
    it = it + 1;
    x1 = x*Rmax_s^(1-x/K);
    plot([x x],[x x1],'r')
    plot([x x1],[x1 x1],'r')
    plot(x, x1, 'or','MarkerFaceColor','r')
    drawnow
    xo = x;
    x  = x1;
    Xs(it) = x;
    pause
end
delete(TTL);clear TTL
title(['Discrete-time Ricker iterations with R_{max} = ', num2str(Rmax_s, '%4.2f')])
figure(2)
set(2, 'Position',[598 39 560 420])
set(gca, 'FontName','Arial', 'FontSize',12)
plot(Xs(1:it), '-ob','MarkerFaceColor','b')
set(gca, 'FontName','Arial', 'FontSize',12)
xlabel('time')
ylabel('Population')
title('Discrete-time Ricker behaviour in the time domain')