% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% An example of CSTRs coupled by diffusion of the Bay-Lake interaction in
% the system of the Saginaw Bay - Lake Huron system.
% Model parameters from Chapra, 1997.
%
% Sect 6.3.1.1 and Fig. 6.22
%
% *************************************************************************

clear
close all
clc

% Subscript legend:
%   1 = Lake
%   2 = Bay

% Set parameters
C1ss = 5.4;  % Steady-sate chlorides conc. in the Lake (Chapra, 1997)
C2ss = 15.2; % Steady-sate chlorides conc. in the Bay  (Chapra, 1997)
C2o  = 30;   % [g/m^3] Steady-state Conc. Bay
C1o  = 20;
C2o  = 30;

W2 = 0.353e12; % [g/year] Bay loading

F2 = 7e9;       % [m^3/year] Input flow from lake to bay
F_out = 161e9;  % [m^3/year] Output flow. NB: F_out = F1 + F2
C1o = W2/F_out; % [g/m^3]  Steady-state Conc. Lake
V1 = 3507e9;    % [m^3] Lake volume
V2 = 8e9;       % [m^3] Bay volume

E = (W2-F2*C2ss)/(C2ss-C1ss); % Computation of Diffusion coeff.

A(1,1) = -(F_out+E)/V1;
A(1,2) = (E+F2)/V1;
A(2,1) = E/V2;
A(2,2) = -(E+F2)/V2;
lambda_iniz = eig(A) %Initial eigenvalues

C_bar = -inv(A)*[0; W2/V2]

%% Run simulation with Chapra's data
D = E;
tfin = 150;
[t, x, y]=sim('Bay_Lake_C1', tfin);

%% Diffusion sensitivity

% tfin=5; % [years]
G = [];
CC = [];
for g = 1:50
    D = g*1e9;
    [t, x, y] = sim('Bay_Lake_C1', tfin);
    G = [G g];
    CC = [CC; C1(end) C2(end)];
end

%% Figures
figure(1)
set(gcf, 'Position', [24 50 1138 495])

subplot(121)
plot(t,C1,'b', t,C2, '--b', 'LineWidth',2)
xlim([0 2])
set(gca, 'FontName','Arial', 'FontSize',12)
xlabel('time [yr]')
ylabel('Concentration [g/m^3]')
legend('Lake Conc.', 'Bay Conc.', 'location','north',...
    'orientation','horizontal')
title(['Fast mode behaviour with computed D',''''])
subplot(122)
plot(G,CC(:,1),'b', G,CC(:,2),'--b', 'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
hold on
axis([0 50 0 30])
ax = axis;
plot([E*1e-9 E*1e-9], [ax(3) ax(4)], ':k')
xlabel('Diffusion coeff. D\times10^-^9 [m^3/yr]')
ylabel('Concentration [g/m^3]')
legend('Lake Conc.', 'Bay Conc.', 'location','north',...
    'orientation','horizontal')
title(['Sensitivity of the steady-state concentrations to D',''''])