% *************************************************************************
% 
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Linearization of a CSTR with second-order kinetics
%
%
% Sect. 6.1.1.3
% 
% *************************************************************************

clear
close all
clc

K      = 1.5;       % Second order kinetics constant
Ci     = 3;         % Input concentration
t_fin  = 6;         % Simulation end time
T_step = 3;         % Dilution change instant
qo1    = 0.25;      % Initial dilution
qo2    = 0.35;      % Final dilution

%% Insert data
prompt={'Kinetic constant K', 'Input concentration', 'Initial dilution',...
        'Final dilution', 'Dilution switch time', 'Simulation length'};
def = {'1.5', '3', '0.35', '0.45', '3', '6'};
data = inputdlg(prompt, 'CSTR with 2nd order kinetics', 1, def);
data = char(data);
K      = str2num(data(1,:));
Ci     = str2num(data(2,:));
qo1    = str2num(data(3,:));
qo2    = str2num(data(4,:));
T_step = str2num(data(5,:));
t_fin  = str2num(data(6,:));

equ = roots([K qo1 -qo1*Ci]); % Equilibrium Solution
eq  = find(equ > 0);
Co1 = equ(eq);  % Steady-state initial concentration (with qo1)
equ = roots([K qo2 -qo2*Ci]);
eq  = find(equ > 0);
Co2 = equ(eq);  % Steady-state final concentration (with qo2)
clear eq equ

%% Simulation
[y, x, t] = sim('CSTR_2', t_fin);
it = find(t_sim >= T_step-0.001);
tt = t_sim(it) - T_step;
CC_out = C_out(it);
CC_lin = C_lin(it);

%% Figures
figure(1)
clf
set(1, 'Position',[18 40 959 459])
plot(tt, CC_out,'b', tt,CC_lin,':r')
axis([0 max(tt) min(Co1,Co2)-0.02 max(Co1,Co2)+0.02]);
axis(axis)
title('Simulation of a CSTR with 2^{nd} order kinetics')
xlabel('time')
ylabel('Concentration')
limits = axis;
hold on
plot([limits(1) limits(2)], [Co1 Co1], ':b')
plot([limits(1) limits(2)], [Co2 Co2], ':b')
text(0.5, Co1, 'Old regime', 'VerticalAlignment', 'bottom')
text(0.5, Co2, 'New regime', 'VerticalAlignment', 'bottom')
legend('Nonlinear model', 'Linearized model', 'location','east')