% *************************************************************************
% 
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Approximation of the pulse response of a diffusive reactor with a
% Normal pdf to estimate the Diffusion coefficient.
%
% Sect. 6.3.3.4
% 
% *************************************************************************

clear
close all
clc

%% Set parameters
L = 6000;          % Length                                            [cm]
t = 0.1:1:1001;    % Elapsed time                                       [s]
F = 0.8;           % Flow                                           [L/min]
M = 182;
S = pi*(1.3/2)^2;  % Cross section                                   [cm^2]
u = 1000*F/(60*S); % Velocity                                        [cm/s]
D = 600;           % Diffusion coeff.                           [cm^2 s^-1]

% Initialize figure
figure(1)
set(gcf, 'Position', [25 39 748 590])
set(gca, 'FontName', 'Arial','FontSize', 12)
Nfig = 3; % Number of subplots

for k = 1:Nfig
    
    L = 2*k*1000;
    C = (M./(2*S*sqrt(pi*D.*t)).*exp(-((L-u*t).^2)./(4*D*t)));
    Cm = max(C);
    Im = find(C == Cm);
    tm = t(Im);
    Sig = 2*D*tm; % Estimated variance
    D_est = Sig/(2*tm);
    
    % Fitted Gaussian distribution
    G  = Cm.*exp(-(u^2*(t-tm).^2)./(2*Sig));
    t1 = tm-sqrt(Sig/(u^2));
    G1 = Cm.*exp(-(u^2*(t1-tm).^2)./(2*Sig));
    t2 = tm+sqrt(Sig/(u^2));
    G2 = Cm.*exp(-(u^2*(t2-tm).^2)./(2*Sig));
    
    % Plot
    subplot(3,1,k)
    plot(t, C, 'b', t, G, ':b', 'LineWidth', 2)
    hold on
    plot([tm tm], [0 Cm], ':k')
    plot(t1, G1, 'ob', 'MarkerFaceColor', 'b', 'MarkerSize', 8)
    plot(t2, G2, 'ob', 'MarkerFaceColor', 'b', 'MarkerSize', 8)
    plot([t1,t2], [G1,G2], ':k')
    axis tight
    title(['Length = ', num2str(L/100), 'm'])
    xlabel('time [s]')
    ylabel('Conc. [mg/L]')
    legend('Pulse response', 'Normal approx.')
end