%**************************************************************************
% 
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Demonstration of the numerical solution of the mono-directional diffusive
% river reach, partitioned into N cells
% 
%
% Sect. 6.3.6.1
% 
% *************************************************************************

clear
close all
clc
% Loading initial concentration profile
load Pulse_data
zmax=ceil(max(t_data));
L=zmax; % Rescaling river length (m)
N_cell=30;
dz=L/(N_cell-1);
z=0:dz:L;
C_iniz=csaps(t_data,Y_data,0.9,z);
ind0=find(C_iniz<=0.001);
C_iniz(ind0)=0.001;


% Loading velocity data
load U_data
u=csaps(t_data,Y_data,0.9,z); % Smoothing velocity for each cell


uo=mean(u);   % mean velocitya (m/s)
w=15;         % river width (m)
s=0.0001;     % slope
h=1.5;        % mead depth (m)

% Computing diffusion via the Fisher formula with a modified coeff. (0.011)
Dif=0.005*uo^2*w^2/(h*sqrt(9.81*s*h))

% Kinetics
k=0.03;


% Loading upstream conditions
load INPUT_data
tfin=max(t_data);    % Simulation time horizon
C_in=Y_data;         % Experimental upstream input

% No upstream input
% C_in=zeros(length(t_data),1);
C_in=C_in';t_data=t_data';

%% Figures
figure(1)
set(1, 'Position',[18 40 560 420])
subplot(311)
plot(z,C_iniz,':r',z,C_iniz,'or')
xlabel('distanza (km)')
ylabel('C_i_n_i_z (mg/L)')

subplot(312)
plot(z,u,':r',z,u,'or')
xlabel('distanza (km)')
ylabel('u (m/s)')

subplot(313)
plot(t_data,C_in)
xlabel('tempo (h)')
ylabel('C_i_n (mg/L)')


% Costruzione del modello
A=zeros(N_cell,N_cell);
B_mat=zeros(N_cell,1);
C_mat=ones(1,N_cell);

for i=1:N_cell
   a(i)=u(i)/(2*dz)+Dif/(dz*dz);
   b(i)=-2*Dif/(dz*dz);
   c(i)=Dif/(dz*dz)-u(i)/(2*dz);
end
A(1,1)=b(1)-c(1);A(1,2)=c(1);
B_mat(1)=a(1);

for i=2:N_cell-1
 A(i,i-1)=a(i-1);  
 A(i,i)=b(i)-k;
 A(i,i+1)=c(i-1);
end

A(N_cell,N_cell-1)=a(N_cell-1);
A(N_cell,N_cell)=b(N_cell)-k+c(N_cell);



[t,x,y]=sim('modello_fiume',tfin);

figure(2)
set(2, 'Position',[598 39 560 420])
surf(t,z,x')
axis tight
xlabel('tempo (h)')
ylabel('z (km)')
shading interp 
view([140 58])
grid off
zlabel('Conc. (mg/L)')
rotate3d on

 

