% *************************************************************************
%
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Sensitivity of the eigenvalues of the diffusive river model with the
% stream velocity u.
%
% Sect. 6.3.6.1
% 
% *************************************************************************

clear
close all
clc

% River data
L = 25;      % River length [km];
N_cell = 30; % Number of cells
dz = L/(N_cell-1); % Length/cells ratio [m]
z = 0:dz:L;  % Spatial coordinates used in the plot
C_iniz = zeros(1,N_cell);

% Fisher diffusion
w  = 4;      % River width [m]
s  = 0.005;  % Slope
h  = 0.2;    % Mean depth  [m]
k  = 0.2; 
uo = linspace(0.01, 0.2, 10);   % velocity [m/s]

% Initialize figure
figure(1)
set(1,'Position', [24 51 847 530])

EIGV = [];
for kk = 1:length(uo)
    u = uo(kk)*ones(1, N_cell);
    Dif = 0.011*uo(kk)^2*w^2/(h*sqrt(9.81*s*h));
    % Model construction
    A = zeros(N_cell, N_cell);
    B_mat = zeros(N_cell, 1);
    C_mat = ones(1, N_cell);
    for i = 1:N_cell
        a(i) = u(i)/(2*dz)+(Dif/dz*dz);
        b(i) = -2*Dif/(dz*dz);
        c(i) = (Dif/dz*dz)-(u(i)/(2*dz));
    end
    % First row
    A(1,1) = b(1)-k;A(1,2)=c(1);
    B_mat(1) = a(1);
    % Intermediate rows
    for i = 2:N_cell-1
        A(i,i-1) = a(i);  
        A(i,i)   = b(i)-k;
        A(i,i+1) = c(i);
    end
    % Final row
    A(N_cell,N_cell-1) = a(N_cell);
    A(N_cell,N_cell) = b(N_cell) - k + c(N_cell);
    eigenval = eig(A);
    EIGV = [EIGV eigenval];
end
[nn pp] = size(EIGV);

for jj = 1:N_cell
    for kj = 1:pp
        if abs(imag(EIGV(jj, kj))) > 1e-4
            IM = plot(real(EIGV(jj, kj)), imag(EIGV(jj, kj)), 'ob',...
                 'MarkerSize',6, 'MarkerFaceColor','b');
            set(gca, 'FontName','Arial', 'FontSize',12)
            
        else
            RE = plot(real(EIGV(jj, kj)), imag(EIGV(jj, kj)), 'sb',...
                 'MarkerSize',8, 'MarkerFaceColor','r');
            set(gca, 'FontName','Arial', 'FontSize',12)
            
        end
        
        hold on
        
    end
end
 
a = axis;
axis([-0.6 0.1 a(3) a(4)]);
plot([-0.6 0.1], [0 0], 'k')
plot([0 0], [a(3) a(4)], 'k')
plot([-k -k], [a(3) a(4)], ':b')
xlabel('Real axis')
ylabel('Imag. axis')
title('Eigenvalues with velocity variations')
legend([RE IM], 'Real eigenvalues', 'Complex eigenvalues', 'location','sw')