%**************************************************************************
% 
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Demonstration of the numerical solution of the mono-directional diffusive
% equation applied to a river reach, partitioned into N_cell
% The comparison is made between the diffusive model 
% and a chain of cascaded CSTRs
%
% Sect. 6.3.6.1
% 
% *************************************************************************

clear
close all
clc

%% Load boundary conditions
%    In the River_* the data are organized as follows:
%               z x_u u t_sim C_init C_in
%    where:
%      t_sim  C_in      are the input data.
%      z and C_init     are river length at which the C_init are given;
%      x_u and u        are the coordinate and the values of the velocity 
%                       along the river;

load River_Data

% Select min. length between C_init and u data
zmax = min(z(end), x_u(end)); 

% Compute the length of each cell
N_cell = 30;
dz = zmax/(N_cell-1); % Length of each cell
z_cell = 0:dz:zmax;
C_init = csaps(z, C_init, 0.9, z_cell);
C_init(1) = C_in(1);

%% Sync the velocity data
u = csaps(x_u, u, 0.9, z_cell);

%% Set params
uo = mean(u);    % Mean velocity [m/s]
w  = 5;          % River width [m]
s  = 0.0001;     % Slope
h  = 0.5;        % Mean water depth [m]
k = 0.003;       % Kinetics

% Compute diffusion using modified Fisher equation
Dif = 0.011*uo^2*w^2/(h*sqrt(9.81*s*h));

%% Discrete-space diffusional equation
A = zeros(N_cell, N_cell);
B_mat = zeros(N_cell, 1);
C_mat = ones(1, N_cell);

for i = 1:N_cell
    a(i) = u(i)/(2*dz) + Dif/(dz*dz);
    b(i) = -2*Dif/(dz*dz);
    c(i) = Dif/(dz*dz) - u(i)/(2*dz);
end
A(1,1) = b(1) - k;
A(1,2) = c(1);
B_mat(1) = a(1);

for i = 2:N_cell-1
    A(i,i-1) = a(i-1);  
    A(i,i)   = b(i) - k;
    A(i,i+1) = c(i-1);
end

A(N_cell,N_cell-1) = a(N_cell);
A(N_cell,N_cell) = b(N_cell) - k + c(N_cell);

%% CSTR chain - Discrete-space diffusional equation

q = u./dz; % Equivalent dilution rate
Acstr = zeros(N_cell, N_cell);
Bcstr = zeros(N_cell, 1);
Ccstr = ones(1, N_cell); 

for i = 1:N_cell
    Acstr(i,i) = -(q(i) + k);
end

for i = 2:N_cell-1
    Acstr(i,i-1) = q(i);
end

Bcstr(1) = q(1);

%% Run simulations
% Diffusive model
[tdiff, Xdiff, y] = sim('Diffusive_model', t_sim(end));

% CSTR chain
[tcstr, Xcstr, y] = sim('CSTR_chain', t_sim(end));

%% Figures

figure(1)
set(1, 'Position',[21 432 755 313])
subplot(311)
plot(z, C_init, ':or','LineWidth',2,'MarkerFaceColor','r')
xlabel('River length [km]')
ylabel('C_{init} [mg/L]')

subplot(312)
plot(z, u, ':or','LineWidth',2,'MarkerFaceColor','r')
xlabel('River length [km]')
ylabel('u [m/s]')

subplot(313)
plot(t_sim, C_in)
xlabel('time [h]')
ylabel('C_i_n [mg/L]')

figure(2)
set(2, 'Position', [20 39 757 299])

% Diffusive model
subplot(121)
surf(tdiff, z(2:end-1), Xdiff(:, 2:end-1)')
set(gca, 'FontName', 'Arial', 'FontSize', 10)
axis tight
xlabel('time [h]')
ylabel('z [km]')
shading interp 
view([140 58])
grid off
zlabel('Conc. [mg/L]')
rotate3d on
title('(A) - Diffusive model with 30 cells')

% CSTR chain
subplot(122)
surf(tcstr, z(2:end-1), Xcstr(:, 2:end-1)')
set(gca, 'FontName',' Arial', 'FontSize', 10)
axis tight
xlabel('time [h]')
ylabel('z [km]')
shading interp 
view([140 58])
grid off
zlabel('Conc. [mg/L]')
rotate3d on
title('(B) - CSTR chain with 30 elements')