% *************************************************************************
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% This file defines the prameters and the initial conditions for all the
% CSTR of the plant.
% The plant is composed as follows:  1 anoxic tank
%                                    1 aerobic tank
%                                    1 secondary settler
% 2-step nitrification is employed.
%
% *************************************************************************

temp = 20;                                      % Plant temperature    [C]
Csat = exp(7.7117 - 1.31403*log(temp + 45.93)); % DO sat. conc.      [mg/L]

%% CSTR parameters
V_anox1 = 1000*4; % Volume anox 1 tank                                 [mc]
V_aero1 = 1330*6; % Volume aero 1 tank                                 [mc]

Ka = 0.15;        % Aeration system efficiency                       [1/mc]
Ua_anox1 = 0;     % Uair anox 1 tank                                 [mc/d]

Ua_aero1 = 1800;  % Uair aero 1 tank                                 [mc/d]

% % Set-points
% DO_setpoint = 2;
% NO3_setpoint = 1;

%% Hydrolysis parameters
Kh = 3.0;         % Hydrolysis constant ratio                         [1/d]
KX = 1;           % Particulate COD saturation coefficient      [gCOD/gCOD]

p_idrolisi = [Kh,KX,];

%% Heterotrophic biomass parameters
kSTO=7.38;   %  0) Stock ratio constant                               [1/d]
ETANO3=0.63; %  1) Anoxic reduction factor                              [-]
KO2=0.1;     %  2) Saturation/Inhibition SO2 constant             [g-O2/mc]
KNOX=0.5;    %  3) SNO3+SNO2 saturation constant                   [g-N/mc]
KS=3;        %  4) Soluble substrate saturation constant     v   [g-COD/mc]
KSTO=1;      %  5) XSTO saturation constant                    [g-COD/gCOD]
MUH=1;       %  6) XH maximum growth ratio                            [1/d]
KNH4=0.01;   %  7) NH4 saturation constant                         [g-N/mc]
KALK=0.1;    %  8) Alkalinity saturation constant            [mole HCO3/mc]
bHO2=0.1;    %  9) Aerobic endogenous respiration constant ratio      [1/d]
bHNO3=0.05;  % 10) Anoxic endogenous respiration constant ratio       [1/d]
bSTOO2=0.2;  % 11) XSTO aerobic respiration ratio                     [1/d] 
bSTONO3=0.1; % 12) XSTO anoxic respiration ratio                      [1/d]

p_eterotrofi = [kSTO, ETANO3, KO2, KNOX, KS, KSTO, MUH, KNH4, KALK,...
                bHO2, bHNO3, bSTOO2, bSTONO3];

%% Autotrophic biomass parameters
MUns = 0.47*exp(0.095*(temp-15)); % 0) Xns max growth rate            [1/d]
MUnb  = 0.78*exp(0.059*(temp-15));% 1) Xnb max growth rate            [1/d]
KANH4 = 2;                        % 2) Ammonia sat. coeff.         [g-N/mc] 
KAO2  = 0.5;                      % 3) Nitrites sat. coeff.        [g-N/mc]
KAALK = 0.5;                      % 4) Alkalinity sat. coeff.[mole HCO3/mc]
bnsO2 = 0.061;                    % 5) Xns aero. end. resp. ratio     [1/d]
bANO2 = 0.05;                     % 6) Xns anox. end. resp. ratio     [1/d]
bANO3 = 0.05;                     % 7) Xnb anox. end. resp. ratio     [1/d]
KANO2 = 0.5;                      % 8) Nitrites sat./inib. coeff.  [g-N/mc]
KNH4I = 5;                        % 9) Ammonia inhibition coeff.   [g-N/mc] 
bnbO2 = 0.061;                    %10) Xnb aero. end. reps ratio      [1/d]
KANOX = 0.5;                      %11) SNO2+SNO3 saturation coeff. [g-N/mc]

p_autotrofi = [MUns, MUnb, KANH4, KAO2, KAALK, bnsO2, bANO2, bANO3,...
               KANO2, KNH4I, bnbO2, KANOX];

%% Stoichiometric constants
fxi    = 0.2;  % XI prod. in end. respiration                 [g-COD/g-COD]
fsi    = 0;    % Inert COD fraction in the partic. substrate  [g-COD/g-COD]
YSTOO2 = 0.85; % Ss stock. prod. aero. yield coeff.           [g-COD/g-COD]
YSTONOX= 0.8;  % Ss stock. prod. anox. yield coeff.           [g-COD/g-COD]
YHO2   = 0.835;% Heterotrophics anox yield coefficient        [g-COD/g-COD]
YHNOX  = 0.74; % Heterotrophics aero yield coefficient        [g-COD/g-COD]
YAnb   = 0.09; % Nitrobacter yield coefficient                  [g-COD/g-N] 
YAns   = 0.14; % Nitrosomonas yield coefficient                 [g-COD/g-N]    

%% Conversion factors
INSS = 0.03;   % N in SS                                        [g-N/g-COD]
INXI = 0.02;   % N in XI                                        [g-N/g-COD]
INBM = 0.07;   % N in biomass                                   [g-N/g-COD]
INSI = 0.01;   % N in SI                                        [g-N/g-COD]
INXS = 0.04;   % N in XS                                        [g-N/g-COD]

%% STATE VARIABLES OF THE BIOLOGICAL MODEL
% x(1)  = SO2   Dissolved oxygen                                 [g/m^3 DO] 
% x(2)  = SS    Rapidely biodegradable organic substrate        [g/m^3 COD]
% x(3)  = SN2g  Gas organic nitrogen                              [g/m^3 N]
% x(4)  = SNH4  Ammoniacal nitrogen + ammonia ions                [g/m^3 N]
% x(5)  = SNO2  Nitrites                                          [g/m^3 N]
% x(6)  = SNO3  Nitrates                                          [g/m^3 N]
% x(7)  = SI    Inert soluble organic material                  [g/m^3 COD]
% x(8)  = SALK  Alkalinity (=392 gr/m^3)                     [Mol HCO3/m^3]
% x(9)  = XI    Inert particulate material                      [g/m^3 COD]
% x(10) = XS    Slowly biodegradable substrate                  [g/m^3 COD]
% x(11) = XH    Heterotrophic biomass                           [g/m^3 COD]
% x(12) = XSTO  Internal biomass stocking product               [g/m^3 COD]
% x(13) = Xns   Nitrosomonas autotrophic biomass                [g/m^3 COD]
% x(14) = Xnb   Nitrobacter autotrophic biomass                 [g/m^3 COD]

%% Stoichiometric matrix
matrice = [     YSTOO2-1,    -1,                         0,                         INSS,                        0,                        0,          0,                               (1/14)*INSS,   0,  0,   0,     YSTOO2,  0,  0 ;   %1)  SS aerobic storage
                       0,    -1,   -(YSTONOX-1)*(1/(2.86)),                         INSS,       0*(YSTONOX-1)/2.86,       (YSTONOX-1)/(2.86),          0,         (INSS/14)-(YSTONOX-1)/((2.86)*14),   0,  0,   0,    YSTONOX,  0,  0 ;   %2)  SS anoxic storage
              1-(1/YHO2),     0,                         0,                        -INBM,                        0,                        0,          0,                                -(INBM/14),   0,  0,   1,  -(1/YHO2),  0,  0 ;   %3)  Aerobic growth
                       0,     0, (1/(1.71))*(-1+(1/YHNOX)),                        -INBM, (1/(1.71))*(1-(1/YHNOX)),                        0,          0, -(INBM/14)+(1/((1.71)*14))*(-1+(1/YHNOX)),   0,  0,   1, -(1/YHNOX),  0,  0 ;   %4)  Nitrite denitrification
                       0,     0, (1/(2.86))*(-1+(1/YHNOX)),                        -INBM,                        0, (1/(2.86))*(1-(1/YHNOX)),          0, -(INBM/14)+(1/((2.86)*14))*(-1+(1/YHNOX)),   0,  0,   1, -(1/YHNOX),  0,  0 ;   %5)  Nitrato denitrification
                   fxi-1,     0,                         0,             -(fxi*INXI)+INBM,                        0,                        0,          0,                 (1/14)*(-(fxi*INXI)+INBM), fxi,  0,  -1,          0,  0,  0 ;   %6)  Aerobic end. resp.
                       0,     0,       -(fxi-1)*(1/(2.86)),              INBM-(fxi*INXI),           0*(fxi-1)/2.86,           (fxi-1)/(2.86),          0,   (1/14)*(INBM-(fxi*INXI)-(fxi-1)/(2.86)), fxi,  0,  -1,          0,  0,  0 ;   %7)  Anoxic end. resp.
                      -1,     0,                         0,                            0,                        0,                        0,          0,                                         0,   0,  0,   0,         -1,  0,  0 ;   %8)  XSTO aerobic resp
                       0,     0,                (1/(2.86)),                            0,              -0*(1/2.86),                -(1/2.86),          0,                         (1/14)*(1/(2.86)),   0,  0,   0,         -1,  0,  0 ;   %9)  XSTO anoxic resp.
           1-(3.43/YAns),     0,                         0,               -INBM-(1/YAns),                   1/YAns,                        0,          0,               -(1/7)*(1/YAns)-(1/14)*INBM,   0,  0,   0,          0,  1,  0 ;   %10) Nitrif. I step
           1-(1.14/YAnb),     0,                         0,                        -INBM,                -(1/YAnb),                   1/YAnb,          0,                                -(INBM/14),   0,  0,   0,          0,  0,  1 ;   %11) Nitrif. II step 
                   fxi-1,     0,                         0,                INBM-fxi*INXI,                        0,                        0,          0,                  (1/14)*(INBM-(fxi*INXI)), fxi,  0,   0,          0, -1,  0 ;   %12) Xns aerobic end. resp. 
                   fxi-1,     0,                         0,                INBM-fxi*INXI,                        0,                        0,          0,                  (1/14)*(INBM-(fxi*INXI)), fxi,  0,   0,          0,  0, -1 ;   %13) Xnb aerobic end. resp. 
                       0,     0,         -(fxi-1)*(1/2.86),                INBM-fxi*INXI,             (fxi-1)/2.86,                        0,          0,     (1/14)*(INBM-(fxi*INXI)-(fxi-1)/2.86), fxi,  0,   0,          0, -1,  0 ;   %14) Xns anoxic end. resp. 
                       0,     0,         -(fxi-1)*(1/2.86),                INBM-fxi*INXI,                        0,             (fxi-1)/2.86,          0,     (1/14)*(INBM-(fxi*INXI)-(fxi-1)/2.86), fxi,  0,   0,          0,  0, -1 ;   %15) Xnb anoxic end. resp
                       0, 1-fsi,                         0, INXS-INSS*(1-fsi)-(fsi*INSI),                        0,                        0,        fsi,     (1/14)*(INXS-INSS*(1-fsi)-(fsi*INSI)),   0, -1,   0,          0,  0,  0];   %16) Hydrolysis
                                
matrice = matrice';