/* ========================================================================
 * Environmental Systems Analysis with Matlab 
 * S. Marsili-Libelli
 *
 * (c) CRC Press 2016
 * ISBN 9781498706353
 * Matlab code revised by Emanuele El Basri and Giacomo Barni
 * ========================================================================
 *
 * ASM3_2N model.
 * C code for a library to be used in an S-function (MEX file) in the 
 * Matlab environment. Includes the vector used to compute the partial 
 * ratios of the chemical reactions.
 *
 * ========================================================================
 */

#define S_FUNCTION_NAME Cstr3_2step
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define S(element)    (*uPtrs_S[element])    /* Input pointer Soluble Components     (Port0) */
#define X(element)    (*uPtrs_X[element])    /* Input pointer Particulate components (Port1) */
#define F(element)    (*uPtrs_F[element])    /* Input pointer Discharge              (Port2) */
#define Uair(element) (*uPtrs_Uair[element]) /* Input pointer Air                    (Port3) */

#define Dim_S    8    /* Dimension Soluble Components                    */
#define Dim_X    6    /* Dimension Particulate components                */
#define Dim_F    1    /* Dimension Discharge                             */
#define Dim_Uair 1    /* Dimension Air                				     */

#define X0_P(S)    ssGetSFcnParam(S,0)    /* Initial conditions          */
#define IDR_P(S)   ssGetSFcnParam(S,1)    /* Parameters hydrolysis       */
#define ETE_P(S)   ssGetSFcnParam(S,2)    /* Parameters heterotroph      */
#define AUT_P(S)   ssGetSFcnParam(S,3)    /* Parameters Aautotroph       */
#define MAT_P(S)   ssGetSFcnParam(S,4)    /* Matrix (already transposed) */
#define Ka_P(S)    ssGetSFcnParam(S,5) 
#define Csat_P(S)  ssGetSFcnParam(S,6)
#define V_P(S)     ssGetSFcnParam(S,7) 

#define NumeroParametri  8	/* Total number of Parameters                */
#define NumeroStati      14 /* Number of continuous states of the system */
#define NumeroRatei      16

/* Function: mdlInitializeSizes ___________________________________________
 * Initializes the differential equation system, defines the number of 
 * states, inputs, outputs and parameters.
 */
static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumSFcnParams(S, NumeroParametri);  /* Expected number of params*/
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, NumeroStati);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, 4)) return;
    ssSetInputPortWidth(S, 0, Dim_S);    /*Input soluble components S    */
    ssSetInputPortWidth(S, 1, Dim_X);    /*Input Particulate components X*/
    ssSetInputPortWidth(S, 2, Dim_F);    /*Input discharge F             */
    ssSetInputPortWidth(S, 3, Dim_Uair); /*Input air Uair                */

    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
	ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortDirectFeedThrough(S, 3, 1);

    if (!ssSetNumOutputPorts(S, 3)) return;
    ssSetOutputPortWidth(S, 0, Dim_S);
    ssSetOutputPortWidth(S, 1, Dim_X);
    ssSetOutputPortWidth(S, 2, Dim_F);
 
    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);
}
/*______________________________________________________________________ */


/* Function: mdlInitializeSampleTimes ___________________________________*/
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, CONTINUOUS_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}
/*______________________________________________________________________ */


#define MDL_INITIALIZE_CONDITIONS
/* Function: mdlInitializeConditions ______________________________________
 * Initializes state x.
 */
static void mdlInitializeConditions(SimStruct *S)
{
    real_T *x0 = ssGetContStates(S); /* Pointer to the initial condition vector     */
    int_T  i;						 /* Variable used for the loops                 */

	if (mxGetM(X0_P(S))!=0)		     /* If the initial condition vector is provided */
	{
		real_T *pr=mxGetPr(X0_P(S)); /* Pointer to the initial condition            */
		for (i=0;i<NumeroStati;i++)
		{
			*x0++=*pr++;             /* Get data form the previous vector           */
		}
	}
	else	
	{
		for(i=0;i<NumeroStati;i++)
		{
			*x0++=0.0;				 /* Else the initial conditions are zero        */
		}
	}
}
/*______________________________________________________________________ */


/* Function: mdlOutputs ___________________________________________________
 * Computes the outputs y = x
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
	real_T            *y1     = ssGetOutputPortRealSignal(S,0);    /* Pointer to the soluble components output    */
	real_T            *y2     = ssGetOutputPortRealSignal(S,1);    /* Pointer to the particulate component output */
	real_T            *y3     = ssGetOutputPortRealSignal(S,2);    /* Pointer to the discharge output             */
	real_T            *x      = ssGetContStates(S);                /* Pointer to the state                        */
	int_T             i;
	InputRealPtrsType uPtrs_F = ssGetInputPortRealSignalPtrs(S,2); /* Pointer to the inputs                       */

   	for(i=0;i<Dim_S;i++)
		{
			if (x[i]<0) x[i]=0; /*x[i]=0;x[i]=-x[i]; To avoid negative
			                      approximations of zero setting the state
								  as zero or changing the sign.          */
							    /*if (x[i]<0 and x[i]>-10^4)             */

			y1[i]=x[i];			/* Soluble components                    */
		}
	
	for(i=0;i<Dim_X;i++)
		{
			y2[i]=x[i+Dim_S];	/* Particulate components                */
		}
	*y3=*uPtrs_F[0];
}
/*______________________________________________________________________ */


#define MDL_DERIVATIVES
/* Function: mdlDerivatives _______________________________________________
 * Solves the non-linear differential system
 */
static void mdlDerivatives(SimStruct *S)
{
	real_T            q;										  	       /* Dilution                          */
	real_T            *dx        = ssGetdX(S);					           /* Pointer to the derivative         */
	real_T            *x         = ssGetContStates(S);				       /* Pointer to the state              */
	int_T			      i,j ;									           /* Variables used in the loops       */
	InputRealPtrsType uPtrs_S    = ssGetInputPortRealSignalPtrs(S,0);      /* Pointer to the inputs             */
	InputRealPtrsType uPtrs_X    = ssGetInputPortRealSignalPtrs(S,1);      /* Pointer to the inputs             */
	InputRealPtrsType uPtrs_F    = ssGetInputPortRealSignalPtrs(S,2);      /* Pointer to the inputs             */
	InputRealPtrsType uPtrs_Uair = ssGetInputPortRealSignalPtrs(S,3);      /* Pointer to the inputs             */
	real_T			  accum;                                               /* Accumulation variable             */

	/* Process variables */
	real_T				*idr  = mxGetPr(IDR_P(S));                         /* Pointer hydrolysis parameters      */
	real_T				*ete  = mxGetPr(ETE_P(S));                         /* Pointer heterotrophics' parameters */	
	real_T				*aut  = mxGetPr(AUT_P(S));                         /* Pointer autotrophics' parameters   */
	real_T				*mat  = mxGetPr(MAT_P(S));                         /* Pointer matrix                     */
	real_T				*Ka   = mxGetPr(Ka_P(S));                          /* Pointer Air                        */
	real_T				*Csat = mxGetPr(Csat_P(S));
	real_T				*Vol  = mxGetPr(V_P(S));
	real_T				ratei [NumeroRatei];                               /* Ratios vector                      */
	real_T				vettore [NumeroStati];                             /* Output ratios vector               */
	real_T				parz [18] ;                                        /* Partial ratios vector              */

    /* Compute the partial rations of reactions */
	parz[0]=x[0]/(ete[2]+x[0]);                                            /* SO2/(KO2+SO2)                      */
	parz[1]=x[1]/(ete[4]+x[1]);                                            /* SS/(KS+SS)                         */
	parz[2]=ete[2]/(ete[2]+x[0]);                                          /* KO2/(KO2+SO2)                      */
	parz[3]=(x[4]+x[5])/(ete[3]+x[4]+x[5]);                                /* (SNO2+SNO3)/(KNOX+SNO2+SNO3)       */
	parz[4]=x[3]/(ete[7]+x[3]);                                            /* SNH4/(KNH4+SNH4)                   */
	parz[5]=x[7]/(ete[8]+x[7]);                                            /* SALK/(KALK+SALK)                   */
	parz[6]=(x[11]/x[10])/(ete[5]+(x[11]/x[10]));                          /* (XSTO/XH)/[KSTO+(XSTO/XH)]         */
	parz[7]=x[0]/(aut[3]+x[0]);                                            /* SO2/(KAO2+SO2)                     */
	parz[8]=x[3]/(aut[2]+x[3]);                                            /* SNH4/(KANH4+SNH4)                  */
	parz[9]=x[7]/(aut[4]+x[7]);                                            /* SALK/(KAALK+SALK)                  */
	parz[10]=x[4]/(ete[3]+x[4]);                                           /* SNO2/(KNO2+SNO2)                   */
	parz[11]=(x[9]/x[10])/(idr[1]+(x[9]/x[10]));                           /* (XS/XH)/[KX+(XS/XH)]               */
    parz[12]=aut[9]/(aut[9]+x[3]);                                         /* KNH4I/(KNH4I+SNH4)                 */
	parz[13]=aut[3]/(aut[3]+x[0]);                                         /* KAO2/(KAO2+SO2)                    */  
    parz[14]=(x[4]+x[5])/(aut[11]+x[4]+x[5]);                              /* (SNO2+SNO3)/(KANOX+SNO2+SNO3)      */
    parz[15]=x[4]/(aut[11]+x[4]);                                          /* SNO2/(KANOX+SNO2)                  */
    parz[16]=x[5]/(aut[11]+x[5]);                                          /* SNO3/(KANOX+SNO3)                  */
   	parz[17]=x[5]/(ete[3]+x[5]);                                           /* SNO3/(KNO3+SNO3)                   */
    
    /* Compute reaction ratios */    
         /* Heterotrophic organisms */
    ratei[0]=ete[0]*parz[0]*parz[1]*x[10];                                 /* SS aerobic storage                 */
	ratei[1]=ete[0]*ete[1]*parz[2]*parz[3]*parz[1]*x[10];                  /* SS anoxic storage                  */
	ratei[2]=ete[6]*parz[0]*parz[4]*parz[5]*parz[6]*x[10];                 /* Aerobic growth                     */
    ratei[3]=ete[1]*ete[6]*parz[2]*parz[10]*parz[4]*parz[5]*parz[6]*x[10]; /* Nitrite denitrification            */	
	ratei[4]=ete[1]*ete[6]*parz[2]*parz[17]*parz[4]*parz[5]*parz[6]*x[10]; /* Nitrate denitrification            */
	ratei[5]=ete[9]*parz[0]*x[10];                                         /* Endogenous respiration aero        */
	ratei[6]=ete[10]*parz[2]*parz[3]*x[10];                                /* Endogenous respiration anox        */
	ratei[7]=ete[11]*parz[0]*x[11];                                        /* Xsto respiration aero              */
	ratei[8]=ete[12]*parz[2]*parz[3]*x[11];                                /* Xsto respiration                   */
	
	     /* Autotrophic organisms */
	ratei[9]=aut[0]*parz[7]*parz[8]*parz[9]*x[12];                         /* 1st step nitrification             */
	ratei[10]=aut[1]*parz[7]*parz[12]*parz[9]*parz[15]*x[13];              /* 2nd step nitrification             */
	ratei[11]=aut[5]*parz[7]*x[12];                                        /* Xns endogenous respiration aero    */
	ratei[12]=aut[10]*parz[7]*x[13];                                       /* Xnb endogenous respiration aero    */
	ratei[13]=aut[6]*parz[13]*parz[15]*x[12];                              /* Xns endogenous respiration anox    */
	ratei[14]=aut[7]*parz[13]*parz[16]*x[13];                              /* Xnb endogenous respiration anox    */
	
	     /* Hydrolysis */
	ratei[15]=idr[0]*parz[11]*x[10];                                       /* Hydrolysis                          */ 

	/* Compute vector */
	/* Takes into account oxygen saturation*/	
	for(i=0;i<Dim_S;i++)
		{
			vettore[i]=*uPtrs_S[i];                                        /* Soluble components                  */
		}
	
	for(i=0;i<Dim_X;i++)
		{
			vettore[i+Dim_S]=*uPtrs_X[i];                                  /* Particulate components               */
		}
	q=*uPtrs_F[0]/Vol[0];

	/* Compute the derivatives */
	for (i=0;i<NumeroStati;i++)
	{
		accum=0.0;
		for (j=0;j<NumeroRatei;j++)
		{
			accum+=mat[i+NumeroStati*j]*ratei[j];                          /* Matrix-ratios product                */
		}
		accum+=q*vettore[i]-q*x[i];                                        /* Add the part computed in the vector  */
		dx[i]=accum;
	}
        dx[0]+=*Ka*(*uPtrs_Uair[0])*(*Csat-x[0]);                          /* Originally dx[0]+=*Ka*Uair(0)*(*Csat-x[0])*/
}
/*______________________________________________________________________ */


/* Function: mdlTerminate _________________________________________________
 * Block to be executed when closing the S-function (MEX file).
 */
static void mdlTerminate(SimStruct *S)
{
}
#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism               */
#else
#include "cg_sfun.h"       /* Code generation registration function      */
#endif