/* ========================================================================
 * Environmental Systems Analysis with Matlab 
 * S. Marsili-Libelli
 *
 * (c) CRC Press 2016
 * ISBN 9781498706353
 * Matlab code revised by Emanuele El Basri and Giacomo Barni
 * ========================================================================
 *
 * Two influents mixing model.
 * C code for a library to be used in an S-function (MEX file) in the 
 * Matlab environment.
 *
 * ========================================================================
 */

#define S_FUNCTION_NAME Mixer3_2step
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define Dim_S    8          /* Dimension Soluble Components              */
#define Dim_X    6          /* Dimension Particulate components          */
#define Dim_F    1          /* Dimension Discharge                       */

#define NumeroParametri  0	/* Total number of Parameters                */
#define NumeroStati      0  /* Number of continuous states of the system */

/* Function: mdlInitializeSizes ___________________________________________
 * Initializes the differential equation system, defines the number of 
 * states, inputs, outputs and parameters.
 */
static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumSFcnParams(S, NumeroParametri);  /* Expected number of params*/
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink       */
    }

    ssSetNumContStates(S, NumeroStati);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, 6)) return;
    ssSetInputPortWidth(S, 0, Dim_S);  /*Input soluble components S      */
    ssSetInputPortWidth(S, 1, Dim_X);  /*Input Particulate components X  */
    ssSetInputPortWidth(S, 2, Dim_F);  /*Input discharge F               */

    ssSetInputPortWidth(S, 3, Dim_S);  /*Input soluble components S1     */
    ssSetInputPortWidth(S, 4, Dim_X);  /*Input Particulate components X1 */
    ssSetInputPortWidth(S, 5, Dim_F);  /*Input discharge F1              */


    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
	ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortDirectFeedThrough(S, 3, 1);
    ssSetInputPortDirectFeedThrough(S, 4, 1);
	ssSetInputPortDirectFeedThrough(S, 5, 1);

    if (!ssSetNumOutputPorts(S, 3)) return;
    ssSetOutputPortWidth(S, 0, Dim_S);
    ssSetOutputPortWidth(S, 1, Dim_X);
    ssSetOutputPortWidth(S, 2, Dim_F);
 
    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);
}
/*______________________________________________________________________ */


/* Function: mdlInitializeSampleTimes ___________________________________*/
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, CONTINUOUS_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}
/*______________________________________________________________________ */


#define MDL_INITIALIZE_CONDITIONS
/* Function: mdlInitializeConditions ______________________________________
 * Initialize state x
 */
static void mdlInitializeConditions(SimStruct *S)
{
}
/*______________________________________________________________________ */


/* Function: mdlOutputs ___________________________________________________
 * Compute output y = x
*/
static void mdlOutputs(SimStruct *S, int_T tid)
{
    real_T            *y1    = ssGetOutputPortRealSignal(S,0);       /* Pointer to the soluble components output    */
	real_T            *y2    = ssGetOutputPortRealSignal(S,1);       /* Pointer to the particulate component output */
	real_T            *y3    = ssGetOutputPortRealSignal(S,2);       /* Pointer to the discharge output             */
	real_T            Ftot;
    int_T              i;

	InputRealPtrsType uPtrs_S = ssGetInputPortRealSignalPtrs(S,0);   /* Pointer to input  S */
	InputRealPtrsType uPtrs_X = ssGetInputPortRealSignalPtrs(S,1);   /* Pointer to input  X */
	InputRealPtrsType uPtrs_F = ssGetInputPortRealSignalPtrs(S,2);   /* Pointer to input  F */
	InputRealPtrsType uPtrs_S1 = ssGetInputPortRealSignalPtrs(S,3);  /* Pointer to input S1 */
	InputRealPtrsType uPtrs_X1 = ssGetInputPortRealSignalPtrs(S,4);  /* Pointer to input X1 */
	InputRealPtrsType uPtrs_F1 = ssGetInputPortRealSignalPtrs(S,5);  /* Pointer to input F1 */

	Ftot=*uPtrs_F[0]+*uPtrs_F1[0];

 /* Soluble components */ 
	for (i=0;i<Dim_S;i++)
	{
	        if (i!=2)  {y1[i]=(*uPtrs_S[i]*(*uPtrs_F[0])+*uPtrs_S1[i]*(*uPtrs_F1[0]))/Ftot;}
	}
		
 /* Particulate components */
      	for (i=0;i<Dim_X;i++)
	{
		    y2[i]=(*uPtrs_X[i]*(*uPtrs_F[0])+*uPtrs_X1[i]*(*uPtrs_F1[0]))/Ftot;
	}
	     
 /* Discharge */
			y3[0]=Ftot;

}
/*______________________________________________________________________ */


#define MDL_DERIVATIVES
/* Function: mdlDerivatives _______________________________________________
 * Compute the non-linear differential system.
 */
static void mdlDerivatives(SimStruct *S)
{  
}
/*______________________________________________________________________ */


/* Function: mdlTerminate _________________________________________________
 * Block to be executed when closing the S-function (MEX file).
 */
static void mdlTerminate(SimStruct *S)
{
}
/*______________________________________________________________________ */


#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism               */
#else
#include "cg_sfun.h"       /* Code generation registration function      */
#endif