/* ========================================================================
 * Environmental Systems Analysis with Matlab 
 * S. Marsili-Libelli
 *
 * (c) CRC Press 2016
 * ISBN 9781498706353
 * Matlab code revised by Emanuele El Basri and Giacomo Barni
 * ========================================================================
 *
 * Settler model.
 * C code for a library to be used in an S-function (MEX file) in the 
 * Matlab environment.
 *
 * ========================================================================
 */

#define S_FUNCTION_NAME Settler3_2step
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define Sin(element)   (*uPtrs_S[element])     /* Table of symbols       */
#define Xin(element)   (*uPtrs_X[element])     /* Table of symbols       */
#define Fin(element)   (*uPtrs_F[element])     /* Table of symbols       */
#define U21in(element) (*uPtrs_U21[element])   /* Table of symbols       */
#define U22in(element) (*uPtrs_U22[element])   /* Table of symbols       */

#define Dim_S    8          /* Dimension Soluble Components              */
#define Dim_X    6          /* Dimension Particulate components          */
#define Dim_F    1          /* Dimension Discharge                       */
#define Dim_U21  1          /* Dimension Discharge                       */
#define Dim_U22  1          /* Dimension Discharge                       */

#define X0_P(S)    ssGetSFcnParam(S,0)  /* Settler initial conditions    */
#define Sed_P(S)   ssGetSFcnParam(S,1)  /* Settler parameters            */

#define minimo(a,b)   (a<b) ? a:b
#define massimo(a,b)  (a>b) ? a:b

#define NumeroParametri  2	/* Total number of Parameters                */
#define NumeroStati      20	/* Number of continuous states of the system */

/* Function: mdlInitializeSizes ___________________________________________
 * Initializes the differential equation system, defines the number of 
 * states, inputs, outputs and parameters.
 */
static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumSFcnParams(S, NumeroParametri);  /* Expected number of params*/
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink       */
    }

    ssSetNumContStates(S, NumeroStati);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, 5)) return;
    ssSetInputPortWidth(S, 0, Dim_S);
    ssSetInputPortWidth(S, 1, Dim_X);
    ssSetInputPortWidth(S, 2, Dim_F);
    ssSetInputPortWidth(S, 3, Dim_U21);
    ssSetInputPortWidth(S, 4, Dim_U22);
    
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortDirectFeedThrough(S, 3, 1);
    ssSetInputPortDirectFeedThrough(S, 4, 1);


    if (!ssSetNumOutputPorts(S, 7)) return;
    ssSetOutputPortWidth(S, 0, Dim_S);
    ssSetOutputPortWidth(S, 1, Dim_X);
    ssSetOutputPortWidth(S, 2, Dim_F);
    ssSetOutputPortWidth(S, 3, Dim_S);
    ssSetOutputPortWidth(S, 4, Dim_X);
    ssSetOutputPortWidth(S, 5, Dim_F);
	ssSetOutputPortWidth(S, 6, NumeroStati);


    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);
}
/*______________________________________________________________________ */


/* Function: mdlInitializeSampleTimes ___________________________________*/
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, CONTINUOUS_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}
/*______________________________________________________________________ */


#define MDL_INITIALIZE_CONDITIONS
/* Function: mdlInitializeConditions ______________________________________
 * Initialize state x
 */
static void mdlInitializeConditions(SimStruct *S)
{
    real_T *x0 = ssGetContStates(S);  /* Pointer to the initial condition vector     */
    int_T  i;					      /* Variable used for the loops                 */

	if (mxGetM(X0_P(S))!=0)           /* If the initial condition vector is provided */
	{
		real_T *pr=mxGetPr(X0_P(S));  /* Pointer to the initial condition            */
		for (i=0;i<NumeroStati;i++)
		{
			*x0++=*pr++;              /* Get data form the previous vector           */
		}
	}
}
/*______________________________________________________________________ */


/* Function: mdlOutputs ___________________________________________________
 * Compute output y 
*/
static void mdlOutputs(SimStruct *S, int_T tid)
{
   InputRealPtrsType uPtrs_S   = ssGetInputPortRealSignalPtrs(S,0);        /* Pointer to the input:  Solubles                           */
	InputRealPtrsType uPtrs_X   = ssGetInputPortRealSignalPtrs(S,1);       /* Pointer to the input:  Particulate                        */
	InputRealPtrsType uPtrs_F   = ssGetInputPortRealSignalPtrs(S,2);       /* Pointer to the input:  Q_ing_sed                          */
	InputRealPtrsType uPtrs_U21 = ssGetInputPortRealSignalPtrs(S,3);       /* Pointer to the input:  Q_out_sed                          */
	InputRealPtrsType uPtrs_U22 = ssGetInputPortRealSignalPtrs(S,4);       /* Pointer to the input:  Q_ric_sed                          */
	
	real_T          *yo1      = ssGetOutputPortRealSignal(S,0);            /* Pointer to the output: Soluble components out             */
	real_T          *yo2      = ssGetOutputPortRealSignal(S,1);            /* Pointer to the output: Particulate components out         */
	real_T          *yo3      = ssGetOutputPortRealSignal(S,2);            /* Pointer to the output: Output discharge Q_out_sed         */ 
	real_T          *yr1      = ssGetOutputPortRealSignal(S,3);            /* Pointer to the output: Recycled soluble components        */
	real_T          *yr2      = ssGetOutputPortRealSignal(S,4);            /* Pointer to the output: Recycled particulate components    */
	real_T          *yr3      = ssGetOutputPortRealSignal(S,5);            /* Pointer to the output: Recycle discharge Q_ric_sed        */

	real_T          *ys       = ssGetOutputPortRealSignal(S,6);            /* Pointer to the output: State variables\Initial conditions */
	real_T		    *Param    = mxGetPr(Sed_P(S));                         /* Pointer to the settler parameters                         */

    real_T          *x        = ssGetContStates(S);                        /* Pointer to the state                                      */
	
	int_T           i;					                                   /* Variable used in the loop                                 */
	real_T			 accum;                                                /* Storage variable                                          */
	real_T			 Pf;
	real_T		    Sf;
	real_T			 Pmin;
	real_T			 vup;
	real_T			 vdn;
	real_T          espo [10];               
	real_T			 vs [10];
	real_T		    Js [10];

	Sf=0;
	for(i=0;i<Dim_S;i++)
		{
		  if (i!=7 & i!=2) {Sf=Sf+Sin(i);}                                 /* Sf=(SO2+SS+SNH4+SNO2+SNO3+SI) of the CSTR before the settler */
		}
	
   accum=0;
	for(i=0;i<Dim_X;i++)
		{
			accum=accum+Xin(i);                                            /* Total particulate components coming from the CSTR before the settler. */
		}
   Pf=accum;                                                               /* Pf = (XI + XS + XH + XSTO + Xns + Xnb)  */
	
	Pmin=Param[6]*Pf;

	vup=U21in(0)/Param[0];                                                 /*  vup = Q_out_sed/A          */  
	vdn=(U22in(0)+385)/Param[0];                                           /*  vdn = (Q_ric_sed + Q_w)/A  */

	/* Compute settling velocity */
	for(i=0;i<10;i++)                      
	{
		espo[i]=exp((-1)*(Param[4])*(x[10+i]-Pmin))-exp((-1)*(Param[5])*(x[10+i]-Pmin));
	}

	for (i=0;i<10;i++)
	{
     real_T Appo_a = Param[3]*espo[i];
     real_T Appo_b = Param[2];
     real_T Appo_c = minimo(Appo_b,Appo_a);
	  vs[i]=massimo(0,Appo_c);
	}

	/* Compute settler flux */
	for(i=0;i<10;i++)
	{
	  Js[i]=vs[i]*x[10+i];
	}

   /* Compute output */
	for(i=0;i<Dim_S;i++)                                                   /* Clarified output */
	{
	  yo1[i]=Sin(i);                                                       /* Soluble part     */
	}
	yo1[2]=0;
	
	for(i=0;i<Dim_X;i++)
	{
		yo2[i]=Xin(i)*x[19]/Pf;                                            /* Particulate part */
	}
	
	yo3[0]=U21in(0);                                                       /* Discharge        */

                     /* Recycled output  */
	yr1[0]=0;                                                              /* DO               */
	for(i=1;i<Dim_S;i++)                      
	{
	    yr1[i]=Sin(i);                                                       /* Soluble part   */
	}
	
	yr1[2]=0;
	for(i=0;i<Dim_X;i++)
	{
		yr2[i]=Xin(i)*x[10]/Pf;                                            /* Particulate part  */
	}
	
	yr3[0]=U22in(0);                                                       /* Discharge         */
	for(i=0;i<NumeroStati;i++)
	{
		ys[i]=x[i];							                               /* State variable: single plate sulubles and particulates */
	}
}
/*______________________________________________________________________ */


#define MDL_DERIVATIVES
/* Function: mdlDerivatives _______________________________________________
 * Compute the non-linear differential system.
 */
static void mdlDerivatives(SimStruct *S)
{
   
	InputRealPtrsType uPtrs_S   = ssGetInputPortRealSignalPtrs(S,0);       /* Pointer to input: Soluble         */
	InputRealPtrsType uPtrs_X   = ssGetInputPortRealSignalPtrs(S,1);       /* Pointer to input: Particulate     */
	InputRealPtrsType uPtrs_F   = ssGetInputPortRealSignalPtrs(S,2);       /* Pointer to input: Qing_sed        */
	InputRealPtrsType uPtrs_U21 = ssGetInputPortRealSignalPtrs(S,3);       /* Pointer to input: Qout_sed        */
	InputRealPtrsType uPtrs_U22 = ssGetInputPortRealSignalPtrs(S,4);       /* Pointer to input: Qric_sed        */
	
	real_T			  *Param    = mxGetPr(Sed_P(S));                       /* Pointer to the settler parameters */

  	real_T            *dx       = ssGetdX(S);		                       /* Pointer to the derivative         */
    real_T            *x        = ssGetContStates(S);	                   /* Pointer to the state              */

    int_T			        i;			                                   /* Variable used in the loop          */
	real_T				accum;           								   /* Storage variable                   */
	real_T				   Pf;
	real_T				   Sf;
	real_T				 Pmin;
	real_T		          vup;
	real_T				  vdn;
	real_T          espo [10];               
	real_T			  vs [10];
	real_T			  Js [10];
	real_T		       Jclar6;
	real_T			   Jclar7;
	real_T			   Jclar8;
	real_T			   Jclar9;
	real_T            Appo_D1;
    real_T            Appo_D2;
    real_T            Appo_D3;
    real_T            Appo_D4;
    real_T            Appo_D5;
    real_T            Appo_D6;

	Sf=0;
	for(i=0;i<Dim_S;i++)
		{
		   if (i!=7 & i!=2) {Sf=Sf+Sin(i);}                                /* Total input soluble components     */
		}
	
   accum=0;
	for(i=0;i<Dim_X;i++)
		{
			accum=accum+Xin(i);                                            /* Total input particulate components */
		}
   Pf=accum;
	
	Pmin=Param[6]*Pf;

	vup=U21in(0)/Param[0];      
	vdn=(U22in(0)+385)/Param[0];


	/* Compute settling velocity */
	for(i=0;i<10;i++)                      
	{
		espo[i]=exp((-1)*(Param[4])*(x[10+i]-Pmin))-exp((-1)*(Param[5])*(x[10+i]-Pmin));
	}

	for (i=0;i<10;i++)
	{
     real_T Appo_a = Param[3]*espo[i];
     real_T Appo_b = Param[2];
     real_T Appo_c = minimo(Appo_b,Appo_a);
	  vs[i]=massimo(0,Appo_c);
	}

	/* Compute settler flux */
	for(i=0;i<10;i++)
	{
	  Js[i]=vs[i]*x[10+i];
	}

	/* Compute the 4 Jclar for the plates 6,7, 8, 9, 10 ( Jclar6...Jclar9 ) */
   Jclar6=minimo(Js[6],Js[5]);
   Jclar7=minimo(Js[7],Js[6]);
   Jclar8=minimo(Js[8],Js[7]);
   Jclar9=minimo(Js[9],Js[8]);



   /* Compute derivatives */
   Appo_D1 = minimo(Js[0],Js[1]);
   Appo_D2 = minimo(Js[1],Js[2]);
   Appo_D3 = minimo(Js[2],Js[3]);
   Appo_D4 = minimo(Js[3],Js[4]);
   Appo_D5 = minimo(Js[4],Js[5]);
   Appo_D6 = minimo(Js[5],Js[6]);
        
		/* Soluble part equations */
    dx[0]=(vdn*(x[1]-x[0]))/Param[1];                                              /* Plate 1   */                        
	dx[1]=(vdn*(x[2]-x[1]))/Param[1];                                              /* Plate 2   */
	dx[2]=(vdn*(x[3]-x[2]))/Param[1];                                              /* Plate 3   */
	dx[3]=(vdn*(x[4]-x[3]))/Param[1];                                              /* Plate 4   */
	dx[4]=(vdn*(x[5]-x[4]))/Param[1];                                              /* Plate 5   */
	dx[5]=((Fin(0)*Sf/Param[0])-((vup+vdn)*x[5]))/Param[1];                        /* Plate 6   */
	dx[6]=(vup*(x[5]-x[6]))/Param[1];                                              /* Plate 7   */
	dx[7]=(vup*(x[6]-x[7]))/Param[1];                                              /* Plate 8   */
	dx[8]=(vup*(x[7]-x[8]))/Param[1];                                              /* Plate 9   */
	dx[9]=(vup*(x[8]-x[9]))/Param[1];                                              /* Plate 10  */
	
		/* Particulate part equations */
	dx[10]=(vdn*(x[11]-x[10])+Appo_D1)/Param[1];                                   /* Plate 1   */
	dx[11]=(vdn*(x[12]-x[11])+Appo_D2-Appo_D1)/Param[1];                           /* Plate 2   */
	dx[12]=(vdn*(x[13]-x[12])+Appo_D3-Appo_D2)/Param[1];                           /* Plate 3   */
	dx[13]=(vdn*(x[14]-x[13])+Appo_D4-Appo_D3)/Param[1];                           /* Plate 4   */
	dx[14]=(vdn*(x[15]-x[14])+Appo_D5-Appo_D4)/Param[1];                           /* Plate 5   */
	dx[15]=((Fin(0)*Pf/Param[0])-((vup+vdn)*x[15])-Appo_D5+Jclar6)/Param[1];       /* Plate 6   */
	dx[16]=(vup*(x[15]-x[16])+Jclar7-Jclar6)/Param[1];                             /* Plate 7   */
	dx[17]=(vup*(x[16]-x[17])+Jclar8-Jclar7)/Param[1];                             /* Plate 8   */
	dx[18]=(vup*(x[17]-x[18])+Jclar9-Jclar8)/Param[1];                             /* Plate 9   */
	dx[19]=(vup*(x[18]-x[19])-Jclar9)/Param[1];                                    /* Plate 10  */
}
/*______________________________________________________________________ */


/* Function: mdlTerminate _________________________________________________
 * Block to be executed when closing the S-function (MEX file).
 */
static void mdlTerminate(SimStruct *S)
{
}
/*______________________________________________________________________ */

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif