/* ========================================================================
 * Environmental Systems Analysis with Matlab 
 * S. Marsili-Libelli
 *
 * (c) CRC Press 2016
 * ISBN 9781498706353
 * Matlab code revised by Emanuele El Basri and Giacomo Barni
 * ========================================================================
 *
 * Splitter model.
 * The splitting percentages used are:      SS = 22% of COD
 *                                          SI = 13% of COD
 *                                          XI = 11% of COD
 *                                          XS = 54% of COD
 * C code for a library to be used in an S-function (MEX file) in the 
 * Matlab environment.
 *
 * ========================================================================
 */

#define S_FUNCTION_NAME Splitter3_2step
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define Dim_S    8          /* Dimension Soluble Components              */
#define Dim_X    6          /* Dimension Particulate components          */
#define Dim_F    1          /* Dimension Discharge                       */

#define NumeroParametri  0	/* Total number of Parameters                */
#define NumeroStati      0  /* Number of continuous states of the system */

/* Function: mdlInitializeSizes ___________________________________________
 * Initializes the differential equation system, defines the number of 
 * states, inputs, outputs and parameters.
 */
static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumSFcnParams(S, NumeroParametri);                                /* Expected number of params*/
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, NumeroStati);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, 4)) return;
    ssSetInputPortWidth(S, 0, 1);                                          /* Input COD */
    ssSetInputPortWidth(S, 1, 1);                                          /* Input NH4 */
    ssSetInputPortWidth(S, 2, 1);                                          /* Input Alk */
    ssSetInputPortWidth(S, 3, 1);                                          /* Input Q   */

    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
	ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortDirectFeedThrough(S, 3, 1);

    if (!ssSetNumOutputPorts(S, 3)) return;
    ssSetOutputPortWidth(S, 0, Dim_S);
    ssSetOutputPortWidth(S, 1, Dim_X);
    ssSetOutputPortWidth(S, 2, Dim_F);

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);
}
/*______________________________________________________________________ */


/* Function: mdlInitializeSampleTimes ___________________________________*/
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, CONTINUOUS_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}
/*______________________________________________________________________ */


#define MDL_INITIALIZE_CONDITIONS
/* Function: mdlInitializeConditions ______________________________________
 * Initialize state x
 */
static void mdlInitializeConditions(SimStruct *S)
{  
}
/*______________________________________________________________________ */


/* Function: mdlOutputs ___________________________________________________
 * Compute output y = x
*/
static void mdlOutputs(SimStruct *S, int_T tid)
{
    real_T            *y1    = ssGetOutputPortRealSignal(S,0);             /* Pointer to the soluble components output     */
	real_T            *y2    = ssGetOutputPortRealSignal(S,1);             /* Pointer to the particulate components output */
	real_T            *y3    = ssGetOutputPortRealSignal(S,2);             /* Pointer to the discharge output              */

	InputRealPtrsType uPtrs_COD = ssGetInputPortRealSignalPtrs(S,0);       /* Pointer to the COD inputs                    */
	InputRealPtrsType uPtrs_NH4 = ssGetInputPortRealSignalPtrs(S,1);       /* Pointer to the NH4 inputs                    */
	InputRealPtrsType uPtrs_Alk = ssGetInputPortRealSignalPtrs(S,2);       /* Pointer to the Alk inputs                    */
	InputRealPtrsType uPtrs_F = ssGetInputPortRealSignalPtrs(S,3);         /* Pointer to the Q inputs                      */

 /* Soluble components */
 
		    y1[0]=0;                     /* SO2    */
			y1[1]=*uPtrs_COD[0]*0.22;    /* SS     */
            y1[2]=0;                     /* SN2(g) */
			y1[3]=*uPtrs_NH4[0];         /* SNH4   */
			y1[4]=0;                     /* SNO2   */
			y1[5]=0;                     /* SNO3   */
			y1[6]=*uPtrs_COD[0]*0.13;    /* SI     */
			y1[7]=*uPtrs_Alk[0];         /* SALK   */

 /* Particulate components */

            y2[0]=*uPtrs_COD[0]*0.11;     /* XI    */
			y2[1]=*uPtrs_COD[0]*0.54;     /* XS    */
			y2[2]=0;                      /* XH    */
		    y2[3]=0;                      /* XSTO  */
			y2[4]=0;                      /* Xns   */
			y2[5]=0;                      /* Xnb   */

 /* Discharge */
			y3[0]=*uPtrs_F[0];
}
/*______________________________________________________________________ */


#define MDL_DERIVATIVES
/* Function: mdlDerivatives _______________________________________________
 * Compute the non-linear differential system.
 */
static void mdlDerivatives(SimStruct *S)
{
}
/*______________________________________________________________________ */


/* Function: mdlTerminate _________________________________________________
 * Block to be executed when closing the S-function (MEX file).
 */
static void mdlTerminate(SimStruct *S)
{
}
/*______________________________________________________________________ */


#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism               */
#else
#include "cg_sfun.h"       /* Code generation registration function      */
#endif