% *************************************************************************
%
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
% 
% Simulation of the two metabolic models for the microbial Monod kinetics:
% the Growth-Decay assumption considers all the substrate to be
% biodegradable, the Death-Regeneration hypothesis recycles part of the
% decayed cell.
% 
% Sect. 7.2.8 Fig. 7.23
%
% *************************************************************************

clear
close all
clc

%% Define model parameters
Ks   = 40;
Mmax = 0.4;
Y    = 0.5;
b    = 0.01;
fi   = 0.2;
Smo  = 10;
Xmo  = 0.5;
tend = 120;

b1 = b/(1-Y*(1-fi));

%% Compute the equilibrium point
[t, x, y] = sim('Monod_Batch_GD_DR', tend);

%% Figures
figure(1)
set(1, 'Position',[29 42 1012 578])

subplot(221)
plot(t,S_gd,'b', t,X_gd,'--b', 'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
axis tight
xlabel('time [h]')
ylabel('Conc. [mg/L]')
legend('Substrate', 'Biomass', 'location','north',...
       'orientation','horizontal')
   title('Groth-decay hypothesis')

subplot(222)
plot(t, DO_gd, 'LineWidth',2)
set(gca,'FontName','Arial', 'FontSize',12)
axis([0 tend 0 10])
xlabel('time [h]')
ylabel('Oxygen Uptake [mg {\ast}h/L]')
title('Groth-decay hypothesis')

subplot(223)
plot(t,S_dr,'b', t,X_dr,'--b','LineWidth',2)
set(gca,'FontName','Arial','FontSize',12)
axis tight
xlabel('time [h]')
ylabel('Conc. [mg/L]')
legend('Substrate', 'Biomass', 'location','north',...
       'orientation','horizontal')
   title('Death-regeneration hypothesis')

subplot(224)
plot(t, DO_dr, 'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
axis([0 tend 0 10])
xlabel('time [h]')
ylabel('Oxygen Uptake [mg {\ast}h/L]')
title('Death-regeneration hypothesis')