function  [sys, x0] = PIM_Kinetics(t, x, u, flag)
% 
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
% 
% S-FUNCTION implementation of the PIM
% Phosphorus kinetics according to John and Flynn,(2000).
% 
% Sect. 7.4.1.2 Figs. 7.55 - 7.56
%
% *************************************************************************
% 
%      [SYS X0] = PIM_KINETICS(T, X, U, FLAG)
% 
%    Input:
%           T        Description of the first input variable here.
%
%           X        Description of the second input variable here.
%
%           U        Description of the second input variable here.
%
%           FLAG     This input is used to select which action perform.
%                    If FLAG = 0 the code performs all initialization. This
%                    part is used only once and produces the six elements
%                    long vector SYS.
%                    If FLAG = 1 the code performs all initialization. This
%                    part is used only once and produces the six elements
%                    long vector SYS.
%                    If FLAG = 2 or 4 the code considers these flags as
%                    unimportant since there are no discrete states in the
%                    system. Output is therefore set to [].
%                    If FLAG = 3 the SIMULINK wants to know what the next 
%                    output will be. In the template system, y gets the
%                    current state x, (SYS =) Y = X.
% 
%    Output:
%           SYS      Is the six elements long vector of the states and the 
%                    outputs.
%
%           X0       Is the initial condition.
%

%% Call global variables
%  Parameter values and initial conditions are passed as external objects
%  via a global statement.
global P IC

if flag  == 0,
    sizes(1) = 4;   % number of continuous states
    sizes(2) = 0;   % number of discrete states
    sizes(3) = 4;   % number of system outputs
    sizes(4) = 1;   % number of system inputs
    sizes(5) = 0;   % number of discontinuous roots; unused feature, set to zero
    sizes(6) = 0;
	
    % Set the initial conditions on the states
    x0 = IC;
    sys = sizes';
end

if abs(flag) == 1    
    % Parameter vector unbundle
    KqP      = P(1);
    PolyPmax = P(2);
    Ksip     = P(3);
    Kps      = P(4);
    Kpt1     = P(5);
    Kpt2     = P(6);
    PTkSIP   = P(7);
    PTKs     = P(8);
    QoP      = P(9);
    SIPmax   = P(10);
    Ksop1    = P(11);
    Ksop2    = P(12);
    SOPmax   = P(13);
    Umax     = P(14);
	
    % Intermediate variables
    PAmax  = SOPmax*Umax;
    PTmax  = 4*PAmax;
    PolyP  = x(1);
	SIP    = x(2);
	SOP    = x(3);
	C      = x(4);
    PT     = PTmax*(u/(u + PTKs))*(1 - SIP/SIPmax)/((1 - SIP/SIPmax) +...
	         PTkSIP);
    PolyPs = 2*PAmax*(SIP/(SIP + Ksip))*((1 - PolyP/PolyPmax)^4)/(((1 -...
	         PolyP/PolyPmax)^4) + Kps);
    PolyPt = PAmax*(PolyP/(PolyP + Kpt1))*((1 + SIP/SIPmax)^4)/(((1 +...
	         SIP/SIPmax)^4) + Kpt2);
    SOPs   = PAmax*(SIP/(SIP + Ksop2))*((1 - SOP/SOPmax)^4)/(((1 -...
	         SOP/SOPmax)^4) + Ksop1);
    Cmu    = Umax*(SOP - QoP)/(SOP - QoP + KqP);
    dC     = C*Cmu;
    dPolyP = PolyP*Cmu;
    dSIP   = SIP*Cmu;
    dSOP   = SOP*Cmu;
	
    % Model equations
    dx(1) = PolyPs -PolyPt - dPolyP;
    dx(2) = PolyPt + PT - SOPs - PolyPs - dSIP;
    dx(3) = SOPs-dSOP;
    dx(4) = dC;
      sys = dx;
end

if flag == 2 || flag == 4
    sys = [];

end

if flag == 3
    for i = 1:4
        if x(i) < 0.001
            x(i) = 0.001;
        end
    end
    sys = x;
end

end