% *************************************************************************
%
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Launch script for the Phosporus Interaction Model (PIM).
% (John and Flynn, 2000)
%
% Sect. 7.4.1.2 Figs. 7.55 - 7.56
%
% *************************************************************************

clear
close all
clc

%% Set global variables
global P IC PTmax PTKs SIPmax PTkSIP

maxPolyP = 0.01; % Maximum PolyP level   % 0.024;

%% Parameter vector bundle
PolyPmax = 25.7*maxPolyP^2 + 1.33*maxPolyP; % Control constant for maximum 
                                            % PolyP level
KqP    = 0.01;     % Constant for quota control of growth
Ksip   = 0.0001;   % Half-saturation constant for synthesis of PolyP from 
                   % SIP
Kps    = 0.01;     % Constant for control of PolyP synthesis by PolyP
                   % concentration
Kpt1   = 0.0002;   % Half-saturation constant for turnover of PolyP
Kpt2   = 0.05;     % Constant for control of PolyP synthesis by SIP
                   % concentration
PTkSIP = 0.01;     % Control constant of trans-inhibition of P-transport by
                   % SIP concentration
PTKs   = 0.0155;   % Half-saturation constant for P-transport
QoP    = 0.003333; % Minimum P-quota
SIPmax = 0.004;    % Maximum size of SIP; set at 20% of SOPmax
Ksop1  = 0.005;    % Constant for control of SOP from SIP
Ksop2  = 0.0002;   % Half-saturation constant for synthesis of SOP from SIP
                   % Is set at 5% of SIPmax
SOPmax = 0.02;     % MaximumSOP
Umax   = 0.05;     % Maximum theoretical C-specific growth rate

PAmax = SOPmax*Umax;
PTmax = 4*PAmax;

IC = [0.0 0.0 0.0 0.1];

P = [KqP PolyPmax Ksip Kps Kpt1 Kpt2 PTkSIP PTKs QoP SIPmax Ksop1 Ksop2...
     SOPmax Umax];
Si = 0.05;  % External P (input)
tend = 300;

%% Run simulation
tic
[t, x, y] = sim('PIM_sink', tend);
toc

PC = (PolyP + SIP + SOP)./C;

%% Figures
figure(1)
set(1, 'Position',[16 40 1035 613])

subplot(221)
plot(tsim, PolyP)
xlim([0 100])
set(gca, 'FontName','Arial', 'FontSize',12)
title('Poly-P')
ylabel('[{\mu}g-PolyP/{\mu}g-C]')
xlabel('time [h]')

subplot(222)
plot(tsim, SIP)
xlim([0 100])
set(gca, 'FontName','Arial', 'FontSize',12)
title('SIP')
xlabel('time [h]')
ylabel('[{\mu}g-P/{\mu}g-C]')

subplot(223)
plot(tsim,SOP)
set(gca, 'FontName','Arial', 'FontSize',12)
title('SOP')
xlabel('time [h]')
ylabel('[mg-P/mg-C]')

subplot(224)
plot(tsim,C,'b',tsim,dC*100,':b')
set(gca, 'FontName','Arial', 'FontSize',12)
title('Biomass-C')
xlabel('time [h]')
ylabel('[{\mu}g-C]')
legend('C biomass', '100*C growth rate', 'location','south',...
       'orientation','horizontal')

figure(2)
set(2, 'Position',[180 40 1035 613])
subplot(211)
plot(tsim,Pex,'b', tsim,-10*Ptrans,':b')
set(gca, 'FontName','Arial', 'FontSize',12)
xlim([0 60])
xlabel('time [h]')
ylabel('[{\mu}g-P/mL]')
legend('ext-P', '10*P-transport')

subplot(212)
plot(tsim, PC)
set(gca, 'FontName','Arial', 'FontSize',12)
xlim([0 100])
xlabel('time [h]')
ylabel('Cell P:C [{\mu}g-P/{\mu}g-C]')