% *************************************************************************
%
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Launch script for the Sequencing Batch Reactor (SBR) model.
%
% Sect. 7.4.3.1 and Fig. 7.60
%
% *************************************************************************

clear
close all
clc
%% STATE VARIABLES IN THE SBR MODEL
%  x(1)  = SO2   Dissolved oxygen                                [g-O2/m^3]
%  x(2)  = SF    Rapidly biodegradable organic substrate        [g-COD/m^3]
%  x(3)  = SA    Fermentation products                          [g-COD/m^3]
%  x(4)  = SNH4  Ammonium nitrogen + ammonium ions                [g-N/m^3]
%  x(5)  = SNO2  Nitrites                                         [g-N/m^3]
%  x(6)  = SNO3  Nitrates                                         [g-N/m^3]
%  x(7)  = SPO4  Soluble inorganic phosphorus                     [g-P/m^3]
%  x(8)  = SI    Soluble inert organic material                 [g-COD/m^3]
%  x(9)  = SALK  Alkalinity                                 [mol HCO3-/m^3]
%  x(10) = XI    Particulate inert material                     [g-COD/m^3]
%  x(11) = XS    Slowly biodegradable substrate                 [g-COD/m^3]
%  x(12) = XH    Heterotrophic biomass                          [g-COD/m^3]
%  x(13) = XPAO  Phosphorus-accumulating biomass                [g-COD/m^3]
%  x(14) = XPP   Polyphosphates                                 [g-COD/m^3]
%  x(15) = XPHA  Organic compounds inside the biomass cell      [g-COD/m^3]
%  x(16) = Xns   Autotrophic biomass(Nitrosomonas)              [g-COD/m^3]
%  x(17) = Xnb   Autotrophic biomass(Nitrobacter)               [g-COD/m^3]
%  x(18) = XMeOH Metallic hydroxides                        [g-Fe(OH)3/m^3]
%  x(19) = XMeP  Metallic polyphosphates                      [g-FePO4/m^3]

%% Set global parameters of the biological model.
global p_idrolisi
global p_eterotrofi
global p_fosforo
global p_autotrofi
global p_precipitazione
global matrice
global Csat 
global Ka
global Ingr
global Ua
global cond_iniz
cond_iniz(1)  = 0.01; % Initial condition for Dissolved Oxygen  (0-4)
cond_iniz(2)  = 0.1;  % Initial condition for SF
cond_iniz(3)  = 10;   % Initial condition for SA                (0-10)
cond_iniz(4)  = 0.5;  % Initial condition for SNH4
cond_iniz(5)  = 5;    % Initial condition for SNO2
cond_iniz(6)  = 5;    % Initial condition for SNO3              (5-10)
cond_iniz(7)  = 0.4;  % Initial condition for SPO4
cond_iniz(8)  = 30;   % Initial condition for SI                (10-100)
cond_iniz(9)  = 20;   % Initial condition for SALK              (10-20)
cond_iniz(10) = 25;   % Initial condition for XI
cond_iniz(11) = 25;   % Initial condition for XS
cond_iniz(12) = 1500; % Initial condition for XH                (1000-2000)
cond_iniz(13) = 600; % Initial condition for XPAO 
cond_iniz(14) = 150;  % Initial condition for XPP               (10-20%XPAO)
cond_iniz(15) = 100;  % Initial condition for XPHA              (10-20%XPAO)
cond_iniz(16) = 80;   % Initial condition for Xns
cond_iniz(17) = 50;   % Initial condition for Xnb
cond_iniz(18) = 10;   % Initial condition for XMeOH
cond_iniz(19) = 10;   % Initial condition for XMeP
cond_iniz(20) = 4;    % Initial condition for volume

% The above initial conditions produce the steady state for
% the following process flows
Ufeed = 0.033;Usur = 0.062;Ufang = 0.00417;

%% Define parameters and initial conditions.
Ua   = 1;   % Input air flow (m^3/h)
Ka   = 0.3; % Oxygen transfer rate coefficient (1/h)
temp = 20;  % Process temperature (C)
% Dissolved Oxygen saturation
Csat = exp(7.7117 - 1.31403*log(temp + 45.93));
% Input concentrations
Ingr = [80 300 50 0 0 25 0 5 0 40];
t_cycle=360; % Cycle time (min)

%% Input parameters
%  Hydrolysis parameters
Kh      = 3.00*exp(0.04*(temp-20));  % Hydrolysis constant ratio                                 [1/d]
ETANO3i = 0.60;                      % Reduction factor for anoxic hydrolysis
ETAfe   = 0.10;                      % Reduction factor for anaerobic hydrolysis
KO2i    = 0.20;                      % Saturation/inhibition coefficient for the oxygen      [g-O2/m^3]
KNO3i   = 0.50;                      % Saturation/inhibition coefficient for the nitrates     [g-N/m^3]
KX      = 0.10*exp(-0.11*(temp-20)); % Saturation coefficient for the particulate COD      [g-COD/gCOD]
KNO2i   = 0.50;                      % Saturation/inhibition coefficient for the nitrites     [g-N/m^3]

p_idrolisi = [Kh, ETANO3i, ETAfe, KO2i, KNO3i, KX, KNO2i];

%  Heterotrophic organisms
MUH     = 6.00*exp(0.07*(temp-20));  % Substrate maximum growth rate                               [1/d] 
qfe     = 3.00*exp(0.07*(temp-20));  % Maximum fermentation ratio                      [g-COD/(g-COD*d)]
ETANO3e = 0.80;                      % Denitrification reduction factor iawq = 0.8 
bH      = 0.47*exp(0.07*(temp-20));  % Lysis constant ratio                                        [1/d]
KO2e    = 0.20;                      % Saturation/inhibition coefficient for the oxygen       [g-O2/m^3]
KF      = 4.00;                      % Saturation coefficient for the growth on SF           [g-COD/m^3]
Kfe     = 2.00;                      % Saturation coefficient for the fermentation of di SF  [g-COD/m^3] ben = 2.00
KAe     = 4.00;                      % Saturation coefficient for SA (acetate)               [g-COD/m^3] iawq = 4.00
KNO3e   = 0.50;                      % Saturation/inhibition coefficient for the nitrates      [g-N/m^3]
KNH4e   = 0.05;                      % Saturation coefficient for the ammonia (nutrient)       [g-N/M^3]
KPe     = 0.01;                      % Saturation coefficient the phosphorus (nutrient)        [g-P/m^3]
KALKe   = 0.10;                      % Saturation coefficient for the alkalinity         [mol HCO3-/m^3]
KNO2e   = 0.50;                      % Saturation/inhibition coefficient for the nitrites      [g-N/m^3]

p_eterotrofi = [MUH, qfe, ETANO3e, bH, KO2e, KF, Kfe, KAe, KNO3e, KNH4e,...
                KPe, KALKe, KNO2e];

%  Phosphorus-accumulating organisms (PAO)
qPHA    = 3.00*exp(0.04*(temp-20));  % Constant ratio for the production of XPHA                          [g-COD/g-PAO*d] ben=6.00
qPP     = 1.50*exp(0.04*(temp-20));  % Constant ratio for the production of XPP                          [g-PP/(g-PAO*d)] ben=5.00
MUPAO   = 1.00*exp(0.104*(temp-20)); % Maximum growth ratio                                                         [1/d] ben=4.00
bPAO    = 0.20*exp(0.07*(temp-20));  % Constant ratio for the lysis of XPAO                                         [1/d] ben=0.40
bPP     = 0.20*exp(0.07*(temp-20));  % Constant ratio for the lysis of XPP                                          [1/d]
bPHA    = 0.20*exp(0.07*(temp-20));  % Constant ratio for the lysis of XPHA                                         [1/d]
KO2f    = 0.20;                      % Saturation coefficient for the SO2                                      [g-O2/m^3]
KAf     = KAe;                       % Saturation coefficient for the SA                                      [g-COD/m^3]
KNH4f   = 0.05;                      % Saturation coefficient for the ammonium                                   [gN/m^3]
KPS     = 0.2;                       % Saturation coefficient for the phosphorus in the production of XPP       [g-P/m^3]
KPf     = 0.01;                      % Saturation coefficient for the phosphorus during the growth              [g-P/m^3]
KALKf   = 0.10;                      % Saturation coefficient for the alkalinity                          [mol HCO3-/m^3]
KPP     = 0.01;                      % Saturation coefficient for the polyphosphates                          [g-PP/gPAO]
KMAX    = 0.34;                      % Inhibition coefficient for the production of XPP                      [g-PP/g-PAO]
KIPP    = 0.02;                      % Inhibition coefficient for the production of XPP                      [g-PP/g-PAO]
KPHA    = 0.01;                      % Saturation coefficient for the XPHA                                   [g-PP/g-PAO]
ETANO3f = 0.60;                      % Reduction factor for the anoxic accumulation of XPP e XPHA                         iawq=0.60
KNO2f   = 0.50;                      % Saturation/inhibition coefficient for the nitrites                       [g-N/m^3]
KNO3f   = 0.50;                      % Saturation/inhibition coefficient for the nitrates                       [g-N/m^3]

p_fosforo = [qPHA, qPP, MUPAO, bPAO, bPP, bPHA, KO2f, KAf, KNH4f, KPS,...
             KPf, KALKf, KPP, KMAX, KIPP, KPHA, ETANO3f, KNO2f, KNO3f];

%  Autotrophic organisms
KO2a  = 0.50;                      % Saturation coefficient for SO2                         [g-O2/m^3]
KNH4  = 0.50;                      % Saturation coefficient for the ammonium                 [g-N/m^3]
KNH4I = 5.00;                      % Inhibition coefficient for the ammonium                 [g-N/m^3]
KNO2a = 0.50;                      % Saturation/inhibition coefficient for the nitrites      [g-N/m^3] iawq=0.50
KALKa = 0.50;                      % Saturation coefficient for the alkalinity         [mol HCO3-/m^3]
KPa   = 0.01;                      % Saturation coefficient for the phosphorus               [g-P/m^3]
MUNS  = 0.47*exp(0.095*(temp-15)); % Maximum growth ratio Nitrosomonas                           [1/d] ben=0.60
MUNB  = 0.78*exp(0.059*(temp-15)); % Maximum growth ratio Nitrobacter                            [1/d] ben=0.60
bNS   = 0.086*exp(0.11*(temp-15)); % Constant decay ratio Nitrosomonas                           [1/d] ben=0.04
bNB   = 0.086*exp(0.11*(temp-15)); % Constant decay ratio Nitrobacter                            [1/d] ben=0.04

p_autotrofi = [KO2a, KNH4, KNH4I, KALKa, KNO2a, KPa, MUNS, MUNB, bNS, bNB];

%  Precipitation
KPRE  = 1.00; % Constant precipitation ratio of the phosphorus [m^3/(g-Fe(OH3)*d)]
KALKp = 0.50; % Saturation coefficient for the alkalinity          [mol HCO3-/m^3]
KDIS  = 0.60; % Re-dissolution constant ratio                                [1/d]

p_precipitazione = [KPRE, KALKp, KDIS];

%  Stoichiometric constants
fp   = 0.08;  
YPO4 = 0.26;  % Demand of PP for the accumulation of PHA          [g-P/g-COD]
YPHA = 0.20;  % Demand of PHA for the accumulation of PP          [g-COD/g-P]
YPAO = 0.625; % Yield coefficient PAO (biomass/PHA)             [g-COD/g-COD]
fsi  = 0.10;  % Inert COD fraction in the particulate substrate [g-COD/g-COD]
YH   = 0.62;  % Heterotrophic yield factor                      [g-COD/g-COD]
Ynb  = 0.09;  % Nitrobacter yield factor (ben=0.20)               [g-COD/g-N] 
Yns  = 0.21;  % Nitrosomonas yield factor (Yns=0.14-0.21)         [g-COD/g-N]

c_stechiometrici = [fp, YPO4, YPHA, YPAO, fsi, YH, Ynb, Yns];

%  Conversion factors
iNSF = 0.03;  %  Content of N in SF                               [g-N/g-COD]
iXB  = 0.07;  %  Content of N in the biomass                      [g-N/g-COD]
iPSF = 0.01;  %  Content of P in SF                               [g-P/g-COD]
iPBM = 0.02;  %  Content of P in the biomass                      [g-P/g-COD]
iNXI = 0.03;  %  Content of N in XI                               [g-N/g-COD]
iPXI = 0.01;  %  Content of P in XI                               [g-P/g-COD]
iNBM = 0.07;  %  Content of N in the biomass                      [g-N/g-COD]
iNSI = 0.01;  %  Content of N in SI                               [g-N/g-COD]
iPSI = 0.00;  %  Content of P in SI                               [g-P/g-COD]
iNXS = 0.04;  %  Content of N in XS                               [g-N/g-COD]
iPXS = 0.01;  %  Content of P in XS        iawq=0.01              [g-P/g-COD]

f_conversione = [iNSF, iXB, iPSF, iPBM, iNXI, iPXI, iNBM, iNSI, iPSI,...
                 iNXS, iPXS]; % Conversion factors vector

%% Stoichiometric matrix             
%                       (1)    (2)    (3)          (4)                    (5)                    (6)     (7)    (8)     (9)(10)  (11)(12)(13)   (14)     (15)(16)(17)   (18)   (19)(20)
matrice = [              0, 1-fsi,     0,        0.01,                     0,                     0,      0,   fsi,  0.001,  0,   -1,  0,  0,     0,       0,  0,  0,     0,     0,  0 ; % (1)
                         0, 1-fsi,     0,        0.01,                     0,                     0,      0,   fsi,  0.001,  0,   -1,  0,  0,     0,       0,  0,  0,     0,     0,  0 ; % (2)
                         0, 1-fsi,     0,        0.01,                     0,                     0,      0,   fsi,  0.001,  0,   -1,  0,  0,     0,       0,  0,  0,     0,     0,  0 ; % (3)
                  1-(1/YH), -1/YH,     0,      -0.022,                     0,                     0, -0.004,     0, -0.001,  0,    0,  1,  0,     0,       0,  0,  0,     0,     0,  0 ; % (4)
                  1-(1/YH),     0, -1/YH,       -iNBM,                     0,                     0,  -0.02,     0,  0.021,  0,    0,  1,  0,     0,       0,  0,  0,     0,     0,  0 ; % (5)
                         0, -1/YH,     0,      -0.022,      (1-YH)/(1.14*YH),     -(1-YH)/(1.14*YH), -0.004,     0,  0.014,  0,    0,  1,  0,     0,       0,  0,  0,     0,     0,  0 ; % (6)
                         0, -1/YH,     0,      -0.022,     -(1-YH)/(1.72*YH),                     0, -0.004,     0,  0.014,  0,    0,  1,  0,     0,       0,  0,  0,     0,     0,  0 ; % (7)
                         0,     0, -1/YH,       -iNBM,      (1-YH)/(1.14*YH),     -(1-YH)/(1.14*YH),  -iPBM,     0,  0.036,  0,    0,  1,  0,     0,       0,  0,  0,     0,     0,  0 ; % (8)
                         0,     0, -1/YH,       -iNBM,     -(1-YH)/(1.72*YH),                     0,  -iPBM,     0,  0.036,  0,    0,  1,  0,     0,       0,  0,  0,     0,     0,  0 ; % (9)
                         0,    -1,     1,        iNSF,                     0,                     0,   iPSF,     0,  0.014,  0,    0,  0,  0,     0,       0,  0,  0,     0,     0,  0 ; % (10)
                         0,     0,     0,       0.031,                     0,                     0,   0.01,     0,  0.002, fp, 1-fp, -1,  0,     0,       0,  0,  0,     0,     0,  0 ; % (11)
                         0,     0,    -1,           0,                     0,                     0,   YPO4,     0, -0.004,  0,    0,  0,  0, -YPO4,       1,  0,  0,     0,     0,  0 ; % (12)
                     -YPHA,     0,     0,           0,                     0,                     0,     -1,     0,  0.048,  0,    0,  0,  0,     1,   -YPHA,  0,  0,     0,     0,  0 ; % (13)
                         0,     0,     0,           0,             YPHA/1.14,            -YPHA/1.14,     -1,     0,  0.048,  0,    0,  0,  0,     1,   -YPHA,  0,  0,     0,     0,  0 ; % (14)
                         0,     0,     0,           0,            -YPHA/1.72,                     0,     -1,     0,  0.048,  0,    0,  0,  0,     1,   -YPHA,  0,  0,     0,     0,  0 ; % (15)
                1-(1/YPAO),     0,     0,       -iNBM,                     0,                     0,  -iPBM,     0, -0.004,  0,    0,  0,  1,     0, -1/YPAO,  0,  0,     0,     0,  0 ; % (16)
                         0,     0,     0,       -iNBM,  (1-YPAO)/(1.14*YPAO), -(1-YPAO)/(1.14*YPAO),  -iPBM,     0, -0.004,  0,    0,  0,  1,     0, -1/YPAO,  0,  0,     0,     0,  0 ; % (17)
                         0,     0,     0,       -iNBM, -(1-YPAO)/(1.72*YPAO),                     0,  -iPBM,     0, -0.004,  0,    0,  0,  1,     0, -1/YPAO,  0,  0,     0,     0,  0 ; % (18)
                         0,     0,     0,       0.031,                     0,                     0,   0.01,     0,  0.002, fp, 1-fp,  0, -1,     0,       0,  0,  0,     0,     0,  0 ; % (19)
                         0,     0,     0,           0,                     0,                     0,      1,     0, -0.048,  0,    0,  0,  0,    -1,       0,  0,  0,     0,     0,  0 ; % (20)
                         0,     0,     1,           0,                     0,                     0,      0,     0, -0.016,  0,    0,  0,  0,     0,      -1,  0,  0,     0,     0,  0 ; % (21)
           -(3.43-Yns)/Yns,     0,     0, -iNBM-1/Yns,                 1/Yns,                     0,  -iPBM,     0,  -0.06,  0,    0,  0,  0,     0,       0,  1,  0,     0,     0,  0 ; % (22)
           -(1.14-Ynb)/Ynb,     0,     0,       -iNBM,                -1/Ynb,                  1/Ynb, -iPBM,     0,  -0.06,  0,    0,  0,  0,     0,       0,  0,  1,     0,     0,  0 ; % (23)
                         0,     0,     0,       0.031,                     0,                     0,   0.01,     0,  0.002, fp, 1-fp,  0,  0,     0,       0, -1,  0,     0,     0,  0 ; % (24)
                         0,     0,     0,       0.031,                     0,                     0,   0.01,     0,  0.002, fp, 1-fp,  0,  0,     0,       0,  0, -1,     0,     0,  0 ; % (25)
                         0,     0,     0,           0,                     0,                     0,     -1,     0,      0,  0,    0,  0,  0,     0,       0,  0,  0, -3.45,  4.87,  0 ; % (26) 
                         0,     0,     0,           0,                     0,                     0,      1,     0,      0,  0,    0,  0,  0,     0,       0,  0,  0,  3.45, -4.87,  0 ; % (27)
                         0,     0,     0,           0,                     0,                     0,      1,     0,      0,  0,    0,  0,  0,     0,       0,  0,  0,     0,     0,  1]; % (28)
        
matrice = matrice';

%% Run simulation
tic
[t, x, y] = sim('SBR_Complete', [0 360]);
toc

%% Figures
figure(1)
set(1,'Position',[20 30 647 695])

subplot(311)
fill(tsim, Y(:,1),[161/255 242/255 253/255])
set(gca, 'FontName','Arial', 'FontSize',11)
ylabel('DO [mg/L]')
axis tight
set(gca,'XTick',[0 30 150 300 360])

subplot(312)
Y(1,4)=0;Y(end,4)=0;Y(1,5)=0;Y(end,5)=0;Y(1,6)=0;Y(end,6)=0;
fill(tsim,Y(:,4),[161/255 242/255 172/255])
hold on
fill(tsim,Y(:,6),[161/255 253/255 253/255])
fill(tsim,Y(:,5),[234/255 245/255 169/255])
set(gca,'FontName','Arial','FontSize',11)
ylabel('Nitrogen [mg-N/L]')
legend('NH_4^+','NO_3^-','NO_2^-','location','north','orientation','horizontal')
axis tight
set(gca, 'XTick',[0 30 150 300 360])

subplot(313)
Y(1,7)=0;
fill(tsim, Y(:,7),[253/255 161/255 233/255])
set(gca, 'FontName','Arial', 'FontSize',12)
xlabel('time [min]')
ylabel('PO_4^{2-} [mg/L]')
axis tight
set(gca, 'XTick',[0 30 150 300 360])

figure(2)
set(2,'Position',[687 38 560 325])
plot(tsim,Y(:,12), tsim,Y(:,13), tsim,Y(:,16), tsim,Y(:,17))
set(gca, 'XTick',[0 30 150 300 360])
set(gca, 'FontName', 'Arial', 'FontSize',12)
xlim([0 360])
legend('X_H', 'PAO', 'AOB', 'NOB','location','northoutside','orientation','horizontal')
xlabel('time [min]')
ylabel('Microbial components [mg/L]')

figure(3)
set(3, 'Position',[687 459 560 267])
Ingres(1,:)=0;
Ingres(end,:)=0;
Col=['y','b','c','m'];
for kj = 1:4
    fill(tsim,Ingres(:, kj)/max(Ingres(:,kj)), Col(kj))
    hold on
    set(gca, 'FontName','Arial', 'FontSize',10)
    ylabel('ON/OFF')
    set(gca, 'FontName','Arial', 'FontSize',10)
    axis tight
end
legend('Feed', 'Aeration', 'Extraction', 'Sludge waste','location','northoutside','orientation','horizontal')
xlabel('time [min]')
set(gca, 'XTick', [0 30 150 300 360])        