% *************************************************************************
%
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
% 
% Simulation of a simple wastewater process composed of an oxidation tank 
% and secondary settler with a simple first-order dynamics and sludge
% recycle.
% The biomass is supposed to follow the death-regeneration metabolic model.
%
% Sect. 7.2.10 
% Compare the results with Fig. 7.31
%
% *************************************************************************

clear
close all
clc

load Input

%% Define kinetic parameters of the biomass
tfin = t_in(end);
Ks   = 40;      % Monod half velocity constant
Y    = 0.5;     % Monod Yield factor
Mu   = 0.45;     % Monod maximum growth rate
bh   = 0.03;    % Monod decay rate
fp   = 0.1;     % Fraction of mineralization 
r    = 0.8;     % Recycle ratio
w    = 0.2;     % Waste ratio
Csat = 9;       % Dissolved oxygen saturation
Ka   = 2;       % Oxygen transfer rate coefficient
F = 2;          % Input flowrate  [m^3/h]
V = 8000;       % Volune CSTR     [m^3]

%% Steady-state conditions

disp('Steady-state conditions')
Sm   = mean(S_in);      % Compute the average input
q   = F/V;           % Dilution rate rescaled to 1/h
qs = (1+r)*F*60/(3*V);  % Dilution of the settler
a   = q*w*(1+r)/(r+w);
So  = Ks*(bh+a)/(Mu-(bh+a))% Initial substrate (BOD)
Xo  = q*(Sm-(1+r)*So)/((1/Y)*(Mu*So/(Ks+So))-(1-fp)*bh) % Initial biomass
OUR = ((1-Y)/Y)*Mu*So*Xo/(Ks+So)
Co  = 2;                               % DO set-point
Ua  = (OUR+q*(1+r)*Co)/(Ka*(Csat-Co))  % Air flowrate

%% Initial conditions derived from steady-state
S_iniz  = So;% Initial substrate (BOD)
X_iniz  = Xo; % Initial biomass
DO_iniz = 2;% Initial DO
Xr_iniz = X_iniz*(1+r)/(r+w); % Initial biomass in secondary clarifier

%% Run simulation
[t, x, y]=sim('ASTP_simple_DO_dr', tfin);

S  = Mono(:,1);
X  = Mono(:,2);
DO = Mono(:,3);
Xr = Mono(:,4);

%% Figures
figure(1)
set(1, 'Position',[84 77 1040 605])

subplot(511)
plot(t, S_in, 'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
hold on
plot([0 tfin], [Sm Sm], ':r')
ylabel('S_i [mg/L]')
axis tight

subplot(512)
plot(t, S, 'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
hold on
plot([0 tfin], [So So], ':r') % Average output BOD
ylabel('BOD [mg/L]')
axis tight

subplot(513)
plot(t,X,'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
hold on
plot([0 tfin], [Xo Xo], ':r')
ylabel('X [mg/L]')
axis tight

subplot(514)
plot(t,DO,'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
hold on
plot([0 tfin], [Co Co], ':r')
ylabel('DO [mg/L]')
axis tight

subplot(515)
plot(t,Xr,'LineWidth',2)
set(gca, 'FontName','Arial', 'FontSize',12)
hold on
plot([0 tfin], [Xr_iniz Xr_iniz], ':r')
ylabel('X_r [mg/L]')
xlabel('time [h]')
axis tight