%% ************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% A simple model describing river respiration as a dynamical balance
% between photosynthetic oxygen productin and oxygen consumption for the
% oxydation of biodegradable material.
%
% Sect. 8.2 and Eq. (8.23)
%
% Please be patient while the script is run
% Wait for the results to appear on the screen
%
% *************************************************************************

clear
clc
close all

%% Set global variables
global t_exp DO_Measured Kr R a Is t

%% Load data
load DO_Sat;                          % Dissolved oxygen at saturation
load Solar_Rad;                       % Solar radiation
load DO_Measured;                     % Measured DO

tfin = DO_Measured(:,1);              % Simulation time
t_exp = DO_Measured(:,1);
Co = DO_Measured(1,2);                % Initial DO concentration

%% Insert initial values
prompt = {'Kr', 'R', 'a', 'Is'};
def = {'0.37', '1.28', '1', '1300'};
data = inputdlg(prompt, 'Initial model parameters', 1, def, 'on');
data = char(data);

Kr = str2num(data(1,:)); %  Re-oxygenation constant
R  = str2num(data(2,:)); %  Respiration
a  = str2num(data(3,:)); %  Radiation constant
Is = str2num(data(4,:)); %  Max solar radiation

Kr_in = Kr;              % Save initial values of Kr R a Is
R_in  = R;
a_in  = a;
Is_in = Is;

%% Run simulation for the initial values
[t, ~, ~] = sim('RRM', tfin); % Simulink model
C_interp = interp1(t, C, t_exp); % Missing data interpolation
C_in = C_interp;                 % Save output of simulated data

%% Initial dynamic sensitivity
%  Computation of perturbed trajectory
ds = 0.0001; % Perturbation

% Model sensitivity to Kr
Kr = Kr_in*(1+ds); % Perturbed Kr
[t, ~, y] = sim('RRM', tfin);
C_pert = interp1(t, C, t_exp);
C_sens_Kr = (C_pert - C_in)/(ds*Kr); % Initial sensitivity of C to Kr
% Compute sensitivity
Sens_Kr_in = sum(C_sens_Kr.^2);
% Reset initial values
Kr = Kr_in;
R  = R_in;
a  = a_in;
Is = Is_in;

% Model sensitivity to R
R = R_in*(1+ds); % Perturbed R
[t, ~, y] = sim('RRM',tfin);
C_pert = interp1(t, C, t_exp);
C_sens_R = (C_pert - C_in)/(ds*R); % Initial sensitivity of C to R
% Compute sensitivity
Sens_R_in = sum(C_sens_R.^2);
% Reset initial values
Kr = Kr_in;
R  = R_in;
a  = a_in;
Is = Is_in;

% Model sensitivity to a
a = a_in*(1+ds); % Perturbed a
[t, ~, y] = sim('RRM',tfin);
C_pert = interp1(t ,C, t_exp);
C_sens_a = (C_pert - C_in)/(ds*a); % Initial sensitivity of C to a
% Compute sensitivity
Sens_a_in = sum(C_sens_a.^2);
% Reset initial values
Kr = Kr_in;
R  = R_in;
a  = a_in;
Is = Is_in;

% Model sensitivity to Is
Is = Is_in*(1+ds); % Perturbed Is
[t, ~, y] = sim('RRM',tfin);
C_pert = interp1(t ,C, t_exp);
C_sens_Is = (C_pert-C_in)/(ds*Is); % Initial sensitivity of C to Is
% Compute sensitivity
Sens_Is_in = sum(C_sens_Is.^2);
% Reset initial values
Kr = Kr_in;
R  = R_in;
a  = a_in;
Is = Is_in;

%% Parametric calibration
p = [Kr R a Is]; % starting point
p_Opt = simplex('f_err', p); % Find optimal parameters

Kr = p_Opt(1);
R  = p_Opt(2);
a  = p_Opt(3);
Is = p_Opt(4);

[t, ~, y] = sim('RRM', tfin);
C_interp = interp1(t, C, t_exp); % Interpolate missing data
C_nom = C_interp;                % Save output of simulated data
Kr_nom = Kr                      % Save nominal Kr
R_nom  = R                       % Save nominal R
a_nom  = a                       % Save nominal a
Is_nom = Is                      % Save nominal Is

figure(1)  % Calibrated model vs. observations
set(1,'Position',[26 45 560 420])
hold on
plot(t, C, 'b')
set(gca, 'FontName','Arial', 'FontSize',12)
plot(t_exp, DO_Measured(:,2), 'o', 'MarkerSize',8, 'MarkerFaceColor','b')
set(gca,'Xtick',[0 6 12 18 24])
ylabel('DO [mg/L]')
xlabel('Time [h]')
legend('Model DO','Obs. DO')
title('Calibrated model vs. observations')


%% Static sensitivity
%  Static sensitivity input
n = 21;    % Number of perturbation cycles
p = 5/100; % Percent perturbation

%% Static sensitivity of Kr
i = 0;
E = [];
Krp = linspace(Kr_nom-Kr_nom/2*n*p, Kr_nom+Kr_nom/2*n*p, n);
for Kr = Krp
    i = i + 1;
    [t, ~, ~] = sim('RRM', tfin);
    C_pert = interp1(t, C, t_exp);
    SS_Kr = C_pert - C_nom;
    E(i) = sum((C_pert - C_nom).^2);
end
figure(2)
set(2,'Position',[602 45 560 420])
subplot(2,2,1)
plot(Krp, E)
xlabel('K_r')
ylabel('E')
axis tight

%% Static sensitivity of R
Kr = Kr_nom;
R  = R_nom;
a  = a_nom;
Is = Is_nom;

i = 0;
E = [];
Rp = linspace(R_nom-R_nom/2*n*p, R_nom+R_nom/2*n*p, n);
for R = Rp
    i = i + 1;
    [t, ~, y] = sim('RRM', tfin);
    C_pert = interp1(t, C, t_exp);
    SS_R = (C_pert - C_nom);
    E(i) = sum((C_pert - C_nom).^2);
end
subplot(2,2,2)
plot(Rp, E)
xlabel('R')
ylabel('E')
axis tight

%% Static sensitivity of a
Kr = Kr_nom;
R  = R_nom;
a  = a_nom;
Is = Is_nom;

i = 0;
E = [];
ap = linspace(a_nom-a_nom/2*n*p, a_nom+a_nom/2*n*p, n);
for a = ap
    i = i + 1;
    [t, ~, y] = sim('RRM', tfin);
    C_pert = interp1(t, C, t_exp);
    SS_a = (C_pert - C_nom);
    E(i) = sum((C_pert - C_nom).^2);
end
subplot(2,2,3)
plot(ap, E)
xlabel('a')
ylabel('E')
axis tight

%% Static sensitivity of Is
Kr = Kr_nom;
R  = R_nom;
a  = a_nom;
Is = Is_nom;

i = 0;
E = [];
Isp = linspace(Is_nom-Is_nom/2*n*p, Is_nom+Is_nom/2*n*p, n);
for Is=Isp
    i = i + 1;
    [t, ~, y] = sim('RRM', tfin);
    C_pert = interp1(t, C, t_exp);
    SS_Is = (C_pert - C_nom);
    E(i) = sum((C_pert - C_nom).^2);
end
subplot(2,2,4)
plot(Isp, E)
xlabel('I_s')
ylabel('E')
axis tight

%% Static sensitivity of the couple Kr R
Kr = Kr_nom;
R  = R_nom;
a  = a_nom;
Is = Is_nom;

i = 0;
j = 0;
E = [];
for Kr = Krp
    i = i+1;
    j=0;
    Par1(i) = Kr;
    for R = Rp
        j = j+1;
        Par2(j) = R;
        [t, ~, y] = sim('RRM',tfin);
        C_pert = interp1(t, C, t_exp);
        E(i,j) = sum((C_pert - C_nom).^2);
    end
end

%% Levels in the Kr - R parameter space
figure(3)
clf
set(3,'Position',[26 45 560 420])
P1 = linspace(0.5*Kr_nom, 1.5*Kr_nom, 100);
P2 = linspace(0.5*R_nom,  1.5*R_nom, 100);
E1 = interp2(Par1, Par2, E, P1, P2', 'spline');
[C, h] = contour(P1, P2, E1,[0.5 1 10 25 50], 'b', 'LineWidth',2);
set(gca, 'FontName','Arial', 'FontSize',12)
clabel(C, h)
xlabel('K_r')
ylabel('R')
hold on
plot(Kr_nom, R_nom,'ob','MarkerFaceColor','b','MarkerSize',8)
DO_var = var(DO_Measured(:,2));
n_exp = length(DO_Measured(:,2));
Ec = DO_var*(1+(2/(n_exp-2))*finv(0.95,2,n_exp-2));
[C, h] = contour(P1, P2, E1, Ec, ':r', 'LineWidth',2);
clabel(C, h)
text(0.42,1.92,'  95% Conf. Limit')
legend('E','Opt. Par.','E95%','orientation','horizontal','location','northoutside')


%% Static sensitivity of the couple Kr Is
Kr = Kr_nom;
R  = R_nom;
a  = a_nom;
Is = Is_nom;

i = 0;
j = 0;
E = [];
for Kr = Krp
    i = i + 1;
    j = 0;
    Par1(i) = Kr;
    for Is = Isp
        j = j + 1;
        Par2(j) = Is;
        [t, ~, y] = sim('RRM', tfin);
        C_pert = interp1(t, C, t_exp);
        E(i,j) = sum((C_pert - C_nom).^2);
    end
end

%% Levels in the R - Is parameter space
figure(5)
clf
set(5,'Position',[602 45 560 420])
P1 = linspace(0.5*Kr_nom, 1.5*Kr_nom, 100);
P2 = linspace(0.5*Is_nom, 1.5*Is_nom, 100);
E1 = interp2(Par1, Par2, E, P1, P2', 'spline');
[C, h] = contour(P1, P2, E1,[0.05 0.2 0.5 1 5], 'b', 'LineWidth',1);
clabel(C, h)
xlabel('K_r')
ylabel('I_s')
hold on
plot(Kr_nom, Is_nom, 'ob','MarkerFaceColor','b','MarkerSize',8)
[C, h] = contour(P1, P2, E1, Ec, ':r', 'LineWidth',2);
clabel(C, h)
text(0.44,3587,'  95% Conf. Limit')
axis tight
legend('E','Opt. Par.','E95%','orientation','horizontal','location','northoutside')

%% Dynamic sensitivity with respect to the optimal parameters
%  Computation of perturbed trajectory
ds = 0.0001; % Perturbation step

Kr = Kr_nom;
R  = R_nom;
a  = a_nom;
Is = Is_nom;

% Trajectory sensitivity to Kr
Kr = Kr_nom*(1+ds); % Perturbed Kr
[t, ~, y] = sim('RRM', tfin);
C_pert = interp1(t, C, t_exp);
C_sens_Kr = (C_pert - C_nom)/(ds*Kr_nom); % Sensitivity of C to Kr
% Computation of sensitivity
Sens_Kr_nom = sqrt(sum(C_sens_Kr.^2))

Kr = Kr_nom;
R  = R_nom;
a  = a_nom;
Is = Is_nom;

% Trajectory sensitivity to R
R = R_nom*(1+ds); % Perturbed R
[t, ~, y] = sim('RRM', tfin);
C_pert = interp1(t, C, t_exp);
C_sens_R = (C_pert - C_nom)/(ds*R_nom); % Sensitivity of C to R
% Computation of sensitivity
Sens_R_nom = sqrt(sum(C_sens_R.^2))

Kr = Kr_nom;
R  = R_nom;
a  = a_nom;
Is = Is_nom;

% Trajectory sensitivity to a
a = a_nom*(1+ds); % Perturbed a
[t, ~, y] = sim('RRM', tfin);
C_pert = interp1(t, C, t_exp);
C_sens_a = (C_pert - C_nom)/(ds*a_nom); % Sensitivity of C to a
% Computation of sensitivity
Sens_a_nom = sqrt(sum(C_sens_a.^2))

Kr = Kr_nom;
R  = R_nom;
a  = a_nom;
Is = Is_nom;

% Trajectory sensitivity to Is
Is = Is_nom*(1+ds); % Perturbed IS
[t, ~, y] = sim('RRM', tfin);
C_pert = interp1(t, C, t_exp);
C_sens_Is = ((C_pert - C_nom)/(ds*Is_nom))*10^4; % Sensitivity of C to Is
% Computation of sensitivity
Sens_Is_nom = sqrt(sum((C_sens_Is/10^4).^2) )

Kr=Kr_nom;
R=R_nom;
a=a_nom;
Is=Is_nom;

% Plot of the trajectory sensitivities
figure(7)
set(7,'Position',[24 44 814 718])

subplot(4,1,1)
plot(t_exp, C_sens_Kr, 'b', t_exp, C_nom, '-.r')
xlim([0 24])
set(gca, 'FontName','Arial', 'FontSize',12)
set(gca, 'Xtick', [0 6 12 18 24])
legend('Sens. K_r', 'DO','location','nw','orientation','horizontal')
xlabel('Time [h]')
ylabel('Nominal sens. K_r')

subplot(4,1,2)
plot(t_exp, C_sens_R, 'b', t_exp, C_nom,'-.r')
xlim([0 24])
set(gca, 'FontName','Arial', 'FontSize',12)
set(gca,'Xtick',[0 6 12 18 24])
legend('Sens. R','DO','location','e','orientation','horizontal')
xlabel('Time [h]')
ylabel('Nominal sens. R')

subplot(4,1,3)
plot(t_exp, C_sens_a, 'b', t_exp, C_nom, '-.r')
xlim([0 24])
set(gca, 'FontName','Arial', 'FontSize',12)
set(gca, 'Xtick',[0 6 12 18 24])
legend('Sens. a', 'DO','location','nw','orientation','horizontal')
xlabel('Time [h]')
ylabel('Nominal sens. a')

subplot(4,1,4)
plot(t_exp, C_sens_Is, 'b', t_exp, C_nom, '-.r')
xlim([0 24])
set(gca, 'FontName','Arial','FontSize',12)
set(gca, 'Xtick',[0 6 12 18 24])
legend('Sens. to I_s', 'DO','location','sw','orientation','horizontal')
xlabel('Time [h]')
ylabel('Sens. I_s')
subplot(411)

%% Model validtion
%  Verifiy with a spare dataset
load DO_validation % Load validation dataset
load Solar_Radv
load DO_Satv
Kr = Kr_nom;
R  = R_nom;
a  = a_nom;
Is = Is_nom;
Co = DO_validation(1,2);
DO_Sat=DO_Satv; % Use validation DO saturation values
Solar_Rad=Solar_Radv; % Use validation solar radiation values

% Run simulation with validation data
[t, x, y] = sim('RRM', tfin); 

figure(8)
set(8,'Position',[26 45 560 420])
% Validation data
plot(t, C, 'b')
set(gca, 'FontName','Arial', 'FontSize',12)
set(gca, 'Xtick',[0 6 12 18 24])
hold on
plot(t_exp, DO_validation(:,2), 'ob','MarkerFaceColor','b', 'MarkerSize',8)
ylabel('DO [mg/L]')
xlabel('Time [h]')
legend('Model DO', 'Valid. DO')
title('Validation model vs. observations')

%% Test on the model response
%  F-statistics on the parameters regression line

figure(9)
set(9,'Position',[604 46 676 420])
subplot(121)
[r, MC, SC, RC, Ftest] = F_test(C_nom, DO_Measured(:,2));
axis equal
legend('Calibration data','Regr. line','location','nw')
subplot(122)
[r, MC, SC, RC, Ftest] = F_test(C, DO_validation(:,2));
axis equal
legend('Validation data','Regr. line','1:1  line','location','nw')

% Variance Accounted For
v_err = var(DO_Measured(:,2) - C_nom); % Error variance
v_data = var(DO_Measured(:,2));        % Data variance
VAF = 100*(1-v_err/v_data)

% Residual analysis
resid = (DO_Measured(:,2) - C_nom);
DO_var = var(resid);
DO_std = sqrt(DO_var);
% Residual_analysis(resid,10)

%% Confidence test on the parameters

% Fisher matrix
F = ones(4, 4);
F(1,1) = sum(SS_Kr.*SS_Kr);
F(1,2) = sum(SS_Kr.*SS_R);
F(1,3) = sum(SS_Kr.*SS_a);
F(1,4) = sum(SS_Kr.*SS_Is);
F(2,1) = F(1, 2);
F(2,2) = sum(SS_R.*SS_R);
F(2,3) = sum(SS_R.*SS_a);
F(2,4) = sum(SS_R.*SS_Is);
F(3,1) = F(1, 3);
F(3,2) = F(2, 3);
F(3,3) = sum(SS_a.*SS_a);
F(3,4) = sum(SS_a.*SS_Is);
F(4,1) = F(1, 4);
F(4,2) = F(2, 4);
F(4,3) = F(3, 4);
F(4,4) = sum(SS_Is.*SS_Is);
disp('Fisher matrix')
F = (1/DO_var)*F;
disp(F)

% Covariance matrix
disp('Covariance matrix')
C = inv(F); % ~F^-1 considering the measurements error matrix Qk = I
disp(C)

% Parameters confidence band estimation
l = size(DO_Measured);
l = l(1,1);
% t-Student because measurements are < 3
band_Kr = tinv(0.95,l)*sqrt(C(1,1));
band_R = tinv(0.95,l)*sqrt(C(2,2));
band_a = tinv(0.95,l)*sqrt(C(3,3));
band_Is = tinv(0.95,l)*sqrt(C(4,4));
disp(['Estimated confidence band for Kr =  ', num2str(band_Kr)])
disp(['Estimated confidence band for R  =  ', num2str(band_R)])
disp(['Estimated confidence band for a  =  ', num2str(band_a)])
disp(['Estimated confidence band for Is =  ', num2str(band_Is)])