%% ************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% 
% *************************************************************************
%
%  Fitting the carrying capacity surface of several Selenastrum batches
%  as a function of the added quantities of N abd P
%
%  Sect. 8.6 and Fig. 8.45
%
% *************************************************************************

clear
close all
clc
%%
% Bioavailable nutrients estimation on the basis of known additions
% The carrying capacity surface is plotted
% as a function of the initial N and P cocnentrations.
%
% The half-velocity constant were estimated on the basis
% of experimental observations.

global Ak Kn Kp n p dn dp K G

Kn=5.6280;
Kp=0.1681;
G=1.5099;

% Data input

prompt={'Insert the N addition dN       [mg/l]:',...
    'Insert the P addition dP      [mg/l]:'};
def={'1','0.05'};
dati=inputdlg(prompt,'Defining the calibration quadrangle',1,def);
dati=char(dati);
dn=str2num(dati(1,:));
dp=str2num(dati(2,:));


prompt={'Sample without additions (N,P):',
   'Sample with N addition (N+dN, P):',
   'Sample with P addition (N, P+dP):',
   'Sample with both N and P addition (N+dN, P+dP).'};
def={'1494000','1524000','2003000','2878000'};
dati=inputdlg(prompt,'Final optical density values [n cell/ml]',1,def);
dati=char(dati);


K(1)=str2num(dati(1,:));      %N-P
K(2)=str2num(dati(3,:));      %N-Pdp
K(3)=str2num(dati(2,:));      %Ndn-P
K(4)=str2num(dati(4,:));      %Ndn-Pdp

Ak=K./1e7;

if max(K)<=(3e6)
    N=0.0001:0.1:4;
    P=0.0001:0.01:0.4;
 else
    N=0.0001:0.2:10;
    P=0.0001:0.01:0.4;
 end
 
[x,y]=meshgrid(N,P);
N1=x./(Kn+x);P1=y./(Kp+y);
M=G./((1./N1)+(1./P1));

Z=[0.01 0.025 0.05 0.075 0.1 0.125 0.15 0.175 0.2 0.225 0.25 0.275];


Par=[0.944 0.157]'; % Initial N and P estimates

figure(1)
clf;
colormap(cool)
hold on;
shg;
rotate3d;
Xmax=max(N);
Ymax=max(P);
Zmax=1.2*K_max(Xmax,Ymax);
contour(x,y,M,Z)
axis([0 Xmax 0 Ymax 0 Zmax]);
title('Finding the bioavailable nutrients...');


figure(1)
hold off;
set(1,'Position',[26 45 560 420])
% Model calibration with nutrients addition
[ParCal,options]=simplex('F_Nutrients',Par);
    
    plot3(n,p,0,'ob','MarkerFaceColor','r','MarkerSize',8);
    plot3(n+dn,p,0,'ob','MarkerFaceColor','r','MarkerSize',8);
    plot3(n,p+dp,0,'ob','MarkerFaceColor','r','MarkerSize',8);
    plot3(n+dn,p+dp,0,'ob','MarkerFaceColor','r','MarkerSize',8);
    plot3([n n+dn],[p p],[0 0],'--b');
    plot3([n n+dn],[p+dp p+dp],[0 0],'--b');
    plot3([n+dn n+dn],[p+dp p],[0 0],'--b');
    plot3([n n],[p p+dp],[0 0],'--b');
 title('Bioavailable nutrients');
   
xlabel('N [mg/l] ')
ylabel('P [mg/l]')
zlabel('Final NCell/ml *10^7')
grid;

figure(2)
set(2,'Position',[605 46 560 420])
surf(x,y,M)
shading interp;
colormap(hsv)
hold on;
grid on;

% axis([0 Xmax 0 Ymax 0 Zmax]);
view(-20,40);
title('Carrying capacity surface')
xlabel('Nitrogen [mg/l]')
ylabel('Phosphorus [mg/l]')
zlabel('Final NCell/ml *10^7')
plot3(n,p,K_max(n,p),'dr','MarkerFaceColor','y','MarkerSize',10);
text(n,p,0.95*K_max(n,p),'  Final value','VerticalAlignment','top');
    plot3(n+dn,p,K_max(n+dn,p),'or','MarkerFaceColor','r','MarkerSize',8);
    plot3(n,p+dp,K_max(n,p+dp),'or','MarkerFaceColor','r','MarkerSize',8);
    plot3(n+dn,p+dp,K_max(n+dn,p+dp),'or','MarkerFaceColor','r','MarkerSize',8);
    

ParBest=ParCal;
Nitrogen=ParCal(1)
Phosphorus=ParCal(2)
