%% ************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% *************************************************************************
%
% Computation of the photoperiod, total daily radiation and instantaneous 
% radiation, based on astronomical formulas Eqs. (8.2) - (8.6).
%
% Sect. 8.1.3 and Fig. 8.7
%
% *************************************************************************

clear
clc
close all

AA=[0 31 28 31 30 31 30 31 31 30 31 30];
mesi=cumsum(AA);

n=[1:365];
Latitude=42;                % Latitude
lat=Latitude*2*pi/360;      % Latitude in radians
y=(2*pi)*(n-21-58)/365;     % Year angle
% Solar declination
  d=0.38092-0.76996*cos(y)+23.265*sin(y)+0.36958*cos(2*y)+0.10868*sin(2*y); 
% Photoperiod  
  f=acos(-tan(lat)*tan(d*(2*pi/360)))/pi;% Fotoperiodo

figure(1)
set(1,'Position',[19 61 675 591])
   % Photoperiod over the year
     subplot(311)
     plot(n,f)
     set(gca,'FontName','Arial','FontSize',11)
     hold on
     xlabel('Calendar month')
     ylabel('Photoperiod')
     xlim([0 365])
     ylim([0.3 0.7])
     set(gca,'Xtick',mesi)
     set(gca,'XTickLabel',{'          Jan';'        Feb';'         Mar';'         Apr';'         May';...
         '         Jun';'        Jul';'        Aug';'         Sep';'         Oct';...
         '        Nov';'          Dec'})
   % Solar radiation over the year
     subplot(312)
     % The yearly solar radiation is fitted to the ENEA data
     % with the code in the Solar_radiation-calibration folder
     % Here are the four parameters of Eq.(8.5) calibrated for a location
     % situated at 41.7 N, 11E.
     Par=[3949.5 2504.6 0.0172 -26.54];
     In=Par(1)+Par(2)*sin(Par(3)*n+Par(4));
     plot(n,In)
     set(gca,'FontName','Arial','FontSize',11)
     xlim([0 365])
     xlabel('Calendar month')
     ylabel('Daily Sol. Rad. (Wh/m2)')
     set(gca,'Xtick',mesi)
     set(gca,'XTickLabel',{'          Jan';'        Feb';'         Mar';'         Apr';'         May';...
         '         Jun';'        Jul';'        Aug';'         Sep';'         Oct';...
         '        Nov';'          Dec'})

% The third graph shows the variation of daily radiation (yellow filled
% curve) while its integral divided by 24 equals the total daily radiation
% for the same day, corresponding to the blue dot in the second graph.
% The integration is performed by the Daily_Rad.mdl Simulink model.
n=172; % June 21st
% Year angle for June 21st
y=(2*pi)*(n-21-58)/365;        
% Solar declination for June 21st
d=0.38092-0.76996*cos(y)+23.265*sin(y)+0.36958*cos(2*y)+0.10868*sin(2*y); 
% Photoperiod for June 21st
f=acos(-tan(lat)*tan(d*(2*pi/360)))/pi;
% Initialize data vectors
T=[];IIt=[];
% Total daily radiation for June 21st
In=Par(1)+Par(2)*sin(Par(3)*n+Par(4));
subplot(312)
hold on
plot(n,In,'o','MarkerFaceColor','b','MarkerSize',6)
text(n,0.9*In,['In= ',num2str(In)],'HorizontalAlignment','center','VerticalAlignment','top')
% Computation of instantaneous solar radiation
  for t=0:1/48:1;   
        It=(In/f)*(1+cos((t-0.5)*2*pi/f));
        Iz1=find(t<0.5-f/2);Iz2=find(t>0.5+f/2);
        It(Iz1)=0;It(Iz2)=0;
    IIt=[IIt It];T=[T t];
  end
  subplot(313)
% Integrate the instantaneous solar radiation
% using the Simulink model named Daily_Rad.mdl
  [t,x,y]=sim('Daily_Rad',[0 1]);
 
% Jn  comes from the Simulink model
% used to integrate the instantaneous solar radiation
fill(24*t,IIt1/(24),'y')
set(gca,'FontName','Arial','FontSize',11)
hold on
plot(24*t,Jn/(24),':b')
plot(24,Jn(end)/(24),'o','MarkerFaceColor','b','MarkerSize',6)
legend('Inst. Sol. Rad.','Total daily sol. rad.','location','nw')
text(20,Jn(end)/24,['In= ',num2str(Jn(end))],'VerticalAlignment','bottom')
set(gca,'XTick',[0 3 6 9 12 15 18 21 24])
xlim([0 24])
xlabel('time of day (h)')
ylabel('Inst. Sol. Rad. (W/m^2)')
title('Daily radiation pattern for June 21st')