function [Y, f]=Freq_Analysis(t, q, mult, plotlog)
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Frequency analysis of a sampled time-series.
%
% *************************************************************************
%
%      [Y F] = FREQ_ANALYSIS(T, Q, MULT, PLOTLOG)
%
%    Input:
%           T        Vector of the time.
%
%           Q        Vector of the time-indexed data.
%
%           MULT     Downsampling decimation.
%
%           PLOTLOG  Is a switch for the plot options:
%                       1 = linear frq. plot;
%                       2 = semilogy plot (solid line);
%                       3 = log-log plot  (dashed line).
%
%    Output:
%           Y        Spectrum components.
%
%           F        Frequency basis.
%

indp = 1:mult:length(t);
tp = t(indp);
qp = q(indp);

%% Get the time base
dt = diff(tp);
Ts = dt(1); % average sampling interval [s]
Fs = 1/Ts;  % Sampling frquency        [Hz]

%% Frequency analysis
L = length(qp);
N_samples = 2^(nextpow2(L) - 1); % Next power of 2 from length of samples
Y = fft(qp, N_samples)/L;
f = Fs/2*linspace(0, 1, N_samples/2 + 1);

%% Figures
subplot(121)
plot(tp, qp, 'b')
axis tight
set(gca, 'FontName','Arial', 'FontSize',12)
xlabel('time [h]')
ylabel('Data')
title('Time-series samples')
subplot(122)
switch plotlog
    case 1
        plot(f, abs(Y(1:N_samples/2+1)/abs(Y(1))))
        axis tight
    case 2
        semilogy(f, abs(Y(1:N_samples/2+1)/abs(Y(1))))
        axis tight
    case 3
        loglog(f, abs(Y(1:N_samples/2+1)/abs(Y(1))), 'r')
        xlim([1/48 max(f)])
end
set(gca, 'FontName','Arial', 'FontSize',12)
xlabel('Freq. [1/h]')
ylabel('Normalized power spectrum')
title(['Frequency spectrum for T_s = ',num2str(Ts,'%6.3f'),' h'])

end