function Fs = Freq_spectrum(time_file, data_file, Ts, fig_num, line_type)
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
% © CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% This function returns the frequency spectrum of the time series provided
% and plots the results.
%
% *************************************************************************
%
%      FS = FREQ_SPECTRUM(TIME_FILE, DATA_FILE, TS, FIG_NUM, LINE_TYPE)
%
%    Input:
%           TIME_FILE  Vector of the time intervals of the series.
%
%           DATA_FILE  Vector of the time series values.
%
%           TS         Sampling interval.
%
%           FIG_NUM    Number of figure in which plot the results.
%
%           LINE_TYPE  Line type specifier for the plot.
%
%    Output:
%           FS         Frequency spectrum of the time series.
%

%% Check data consistency
if length(time_file) ~= length(data_file),
    disp('Data files must have the same length')
    return
end

%% Check line type
if line_type < 1
    line_type = 1;
end
if line_type > 5
    line_type = 5;
end

%% Check constraint on frequency axis
%  If Freq_Flag = 1 don't compute frequency, Frequency is entered as the
%  fifth input argument.

L = length(time_file); % Length of signal samples

%  Get the time base
Fs = 1/Ts; % Nyquist sampling frequency

%% Frequency analysis
N_samples = 2^(nextpow2(L) - 1); % Next power of 2 from sample length
Y = fft(data_file, N_samples)/L;
f = Fs/2*linspace(0, 1, N_samples/2+1);

%% Plot single-sided amplitude spectrum.
figure(fig_num)
switch line_type
    case 1
        semilogy(f, 2*abs(Y(1:N_samples/2+1)), 'b')
    case 2
        semilogy(f, 2*abs(Y(1:N_samples/2+1)), ':b')
    case 3
        semilogy(f, 2*abs(Y(1:N_samples/2+1)), '.-r', 'LineWidth',2)
    case 4
        semilogy(f, 2*abs(Y(1:N_samples/2+1)),'--k', 'LineWidth',2)
    case 5
        semilogy(f, 2*abs(Y(1:N_samples/2+1)),'-.k', 'LineWidth',2)
end

xlabel('Frequency [1/t]')
ylabel('Power Spectrum')

end