function [U, V] = FCM(X, m, c)
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% FC-means Fuzzy Clustering.
%
% *************************************************************************
%
%      [U, V] = FCM(X, M, C)
%
%    Input:
%           X        N-by-P matrix of the N data of P features to be
%                    classified.
%
%           M        Fuzziness parameter of the partition.
%
%           C        Number of clusters.
%
%    Output:
%           U        C-by-N partition matrix of data X with respect to the
%                    clusters provided by V.
%
%           V        C-by-P matrix containing the C centroids of the
%                    clusters.
%

epss = 10e-4;
it_max = 100;

toll = 1e-7;           % Tolerance with which a centroid and an observation
toll = toll^(2/(m-1)); % are considered coincident.

[n, p] = size(X);

%% Initialize iterative cycle
nrm = 10; % Norm
loop = 0;
code = 1;
Unew = [eye(c), rand(c, n-c)];    % Unew must respect 2 constraints:
[Mx, Imax] = max(Unew(:, c+1:n)); % a) The sum of each column must be 1;
Unew = [eye(c), zeros(c,n-c)];    % b) The rows must be different.
for i = c+1:n                     %    Putting the identity in the first C
    Unew(Imax(i-c), i) = 1;       %    columns b is satisfied; the other
end                               %    columns are random chosen and made
                                  %    unitary.

Unew = (0.5*(2-sqrt(2))/c)*ones(c,n) + (0.5*sqrt(2))*Unew; % from Bezdek 
                                                           % pag 109

%% Iterative cycle
while (nrm > epss) && (loop <= it_max)
    U = Unew;
    loop = loop + 1;
    
    %% Compute matrix V of the centroids
    Um = U.^m;
    S = Um*X;           % The rows of this matrix are the centroids numerators
    Den = Um*ones(n,1); % Den is a column vector in which each row is the 
                        % sum of the components of the corresponding row of
                        % the matrix Um.
    
    
    for i = 1:c                 % Each row of V is divided by the
        V(i,:) = S(i,:)/Den(i); % corresponding element of Den.
    end
    
    %% Compute matrix of the distances D2
    for k = 1:n
        for i = 1:c
            D2(i,k) = (norm((V(i,:) - X(k,:)),2))^(2/(m-1));
        end
    end
    
    %% Update partition matrix U
    Unew = zeros(c,n);
    for k = 1:n
        ind = 0;
        for i = 1:c            % Ik contains the indexes of the centroids 
            if D2(i,k) < toll  % that coincide less than TOLL with the k-th
                ind = ind + 1; % observation. If there are any the vector 
                Ik(ind) = i;   % is not created.
            end
        end                    % If IND = 0 it means that no centroid 
        if ind == 0            % coincides with the k-th observation.
            aux = ones(1,c)*(ones(c,1)./D2(:,k));
            Unew(:,k) = ones(c,1)./(D2(:,k)*aux);
        else
            Unew(Ik,k) = ones(ind,1)/ind;
        end
        clear Ik;
    end
    nrm = max(max(abs(Unew - U))); % Error of the norm of U at each iter.
end
if nrm < epss
    code = 0;
end

end