function [f,dd] = fgrade(x,v,P,Pi)
% 
% Environmental Systems Analysis with Matlab 
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
% 
% F = FGRADE(X,V,M) calculates membership grade of elements in vector
% X into clusters (multi-dimensional fuzzy sets) defined by means V
% and distance norm-inducing matrix M. Both V and M are obtained from
% GKFAST via GKPROJ. For details on meaning of V and M and for the
% algorithm, see GKFAST and the description of the Gustafson-Kessel
% method.
%
%     Input:
%       X  ... data matrix (M x N, M - number of data, N - data dimension)
%       V  ... cluster means matrix (C x N, C - number of clusters)
%       M  ... matrices inducing the distance norm, calculated as
%              M = det(P).^(1/N)*inv(P), matrices for all clusters are
%              concatenated  in one matrix M = [M1;M2;...Mc]
%       p  ... optional parameter p > 1, determines the fuzziness
%              of clusters, must be the same as in GKFAST, default 2
%     Output:
%       F  ... membership grades

% (c) Robert Babuska, May 1994

if nargin < 3, P = []; end;

[mx,nx] = size(x);
[c,nc] = size(v);
x1 = ones(mx,1);

  for j = 1 : c,                        % for all clusters
    xv = x - x1*v(j,:);
% Calculate distances
    if isempty(P), 
      p = eye(nx); 
    else
      p = P((j-1)*nx+(1:nx),:);
    end;
    d(:,j) = (sqrt(det(p))/Pi(j))*exp(sum((xv*inv(p).*xv)')'/2);
  end;
% Calculate "similarities" based on distance only
  dd = 1./(1+d);
%  dd = d;
% Calculate membership grades
  d0 = (d == 0);                                % find zero distances
  c0 = max(d0')';                               
  d(c0,:) = zeros(sum(c0),c);                   % set d(i,:) to 0
  d(d0) = ones(sum(sum(d0)),1);                 % set d(d0)'s to 1
  d = 1./d;
  f = (d ./ (sum(d')'*ones(1,c)));

