% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
%  This exercise demonstrates the Fuzzy Maximum Likelihood Estimate (FMLE)
%  clustering algorithm.
%
%          Input:
%       X  ... M by N data matrix,
%              M is the number of data points and N data dimension
%       F0 ... either an initial fuzzy partition matrix, or the
%              number of clusters. In the latter case, a default
%              partition matrix is generated.
%       m  ... optional parameter m > 1, determines the fuzziness
%              of clustering, for m close to 1 clusters become crisp,
%              default value is 2
%       e  ... optional termination tolerance, the algorithm stops
%              when max(max(F(k-1) - F(k))) <= e, default tolerance 1e-3
%       s  ... optional parameter for plotting intermediate results
%              (only for 2D data), default 0 - i.e. no plot, set
%              to 1 to show the clustering process on-line,
%	       set to 2 to speed-up the plots for ordered data
%       rho .. 1xM vector of expected cluster volumes (default a unit vector)
%
%     Output:
%       F  ... fuzzy partition matrix
%       C  ... cluster means matrix
%       P  ... cluster covariance matrices concatenated in one matrix
%              P = [P1;P2;...Pk], where k is number of clusters
%       V  ... eigenvectors of the covariance matrices, corresponding
%              to the smallest eigenvalues
%       D  ... eigenvalues of the covariance matrices
%       J  ... history of the clustering criterion J
%       M  ... matrices inducing the distance norm, calculated as
%              M = det(P).^(1/N)*inv(P), M = [M1;M2;...Mk]
%
% The original algorithm by Robert Babuska, 1994-96
% public domain software available at http://www.dcsc.tudelft.nl/~rbabuska/
% was modified by SML.
%
% Sect. 4.5.9
%
% *************************************************************************

close all
clear
clc

% Data input
disp('------------------------------------------')
disp('--- Fuzzy Maximum Likelihood Estimates ---')
disp('------------------------------------------')
disp(' ')
F_I = input('Do you want to load data from a file (1) or by direct input (2)? ');
if F_I == 1
    
    % Get the data from file
    g = uigetfile('FMLE_*.mat', 'Load data');
    eval(['load ' g]);
    
    % Plot the data
    figure(1)
    set(1, 'Position',[29 45 593 526])
    axis square
    hold on
    N = size(X,1);
    x1min = floor(min(X(:,1)));
    x1max = ceil(max(X(:,1)));
    x2min = floor(min(X(:,2)));
    x2max = ceil(max(X(:,2)));
    Ex = [x1min x1max x2min x2max];
    axis(Ex);
    set(gca, 'FontName','Arial', 'FontSize',12)
    for i = 1:N
        plot(X(i,1), X(i,2), 'ok', 'MarkerSize',7,...
            'MarkerFaceColor',[0.83 0.82 0.78])
    end
    Xlim = [x1min x1max];
    Ylim = [x2min x2max];
    
else if F_I==2,
        % Enter data with the mouse
        figure(1)
        TTL = title('Right-click to exit data entry');
        set(1, 'Position',[29 45 593 526])
        set(gca, 'FontName','Arial', 'FontSize',12)
        x1min = 0;
        x1max = 1;
        x2min = 0;
        x2max = 1;
        axis([x1min x1max x2min x2max])
        Xlim = [x1min x1max];
        Ylim = [x2min x2max];
        xlabel('x_1')
        ylabel('x_2')
        axis square
        axis(axis)
        hold on
        X = [];
        i = 0;
        butt = 1;
        while butt == 1
            i = i + 1;
            [x, y, butt] = ginput(1);
            plot(x, y, 'ok', 'MarkerSize',7, ...
                'MarkerFaceColor',[0.83 0.82 0.78])
            X = [X; x y];
        end
        delete(TTL); clear TTL
        %% Save the data
        Save_data = input('Do you want to save your data? (y/n) ', 's');
        if Save_data == 'y',
            Fname = input('Name of data file ', 's');
            Fn = char(['FMLE_', Fname]);
            Mn = 'X';
            eval(['save ', Fn, ' ', Mn])
        end
    end
end

m = input('Fuzzy exponent: ');
c = input('Number of clusters: ');
tol = 1e-4;
s = 0;
[f, v, P, V, D, J, M, Pi] = fmle(X, c, m, tol, s);
figure(1)
for i = 1:c
    plot(v(i,1), v(i,2), 'sk', 'MarkerFaceColor','k', 'MarkerSize',10)
end
GK_Alfa_contour(Xlim, Ylim, v, m, P);
xlabel('x_1')
ylabel('x_2')
disp('  ')
disp('----------------------------')
Hn = Fuzzy_entropy(f');
[g r] = Sep_Coef(X, f', v);
title(['H_n = ', num2str(Hn, '%8.4f'), '  ','g = ', num2str(g, '%8.4f'),...
    '  ', 'm = ', num2str(m, '%4.1f')])
FHV = fhyperv(X, f, v, m);
disp('  ')
disp(['Fuzzy Hypervolume ', num2str(FHV)])
disp('----------------------------')