function [f, C, P, V, D, J, M, Pi] = fmle(x, f0, m, e, s, rho, seed)
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Fuzzy maximum likelihood estimation clustering algorithm.
%
% (c) Robert Babuska, May 1994
%
% *************************************************************************
%
%      [F, C, P, V, D, J, M, Pi] = FMLE(X, F0, m, e, s, RHO, SEED)
%
%    Input:
%           X        M-by-N data matrix. M is the number of data points and
%                    N data dimension.
%
%           F0       Either an initial fuzzy partition matrix, or the
%                    number of clusters. In the latter case, a default
%                    partition matrix is generated.
%
%           M        a dummy parameter, defined for syntactic compatibility
%                    with GKFAST.
%
%           E        Optional termination tolerance, the algorithm stops
%                    when max(max(F(k-1) - F(k))) <= e, default tol. 1e-3.
%
%           S        optional parameter for plotting intermediate results
%                    (only for 2D data), default 0 - i.e. no plot, set
%                    to 1 to show the clustering process on-line,
%	                 set to 2 to speed-up the plots for ordered data.
%
%           RHO      Optional vector of cluster volumes (unit vector by
%                    default).
%
%           SEED     .
%
%    Output:
%           F        Fuzzy partition matrix.
%
%           C        Cluster means matrix.
%
%           P        Cluster covariance matrices concatenated in one matrix
%                    P = [P1;P2;...;Pk], where k is number of clusters.
%
%           V        Eigenvectors of the covariance matrices, corresponding
%                    to the smallest eigenvalues.
%
%           D        Eigenvalues of the covariance matrices.
%
%           J        History of the clustering criterion J.
%
%           M        Matrices inducing the distance norm, calculated as
%                    M = det(P).^(1/N)*inv(P), M = [M1;M2;...Mk]
%                    (defined for compatibility with GKFAST).
%
%           Pi       .
%

if nargin < 3
    m = 2;
elseif isempty(m)
    m = 2;
end
if nargin < 4
    e = 1e-2;
elseif isempty(e)
    e = 1e-2;
end
if nargin < 5
    s = 0;
elseif isempty(s)
    s = 0;
end

[mx,nx] = size(x);
[mf0,c] = size(f0);
x1 = ones(mx,1);
inx = 1/nx;
if c == mf0 && c == 1
    c = f0;
    
    % Initialize fuzzy partition matrix
    if nargin == 7
        rand('seed', seed);
    end
    mm = mean(x);
    aa = max(abs(x - ones(mx,1)*mm));
    C = 2*(ones(c,1)*aa).*(rand(c,nx)-0.5) + ones(c,1)*mm;
    
    % Calculate f0
    for j = 1 : c,                        % for all clusters
        xv = x - x1*C(j,:);
        d(:,j) = sum((xv.^2)')';
    end
    
    d0 = (d == 0);                        % find zero distances
    c0 = max(d0')';
    d(c0,:) = zeros(sum(c0),c);           % set d(i,:) to 0
    d(d0) = ones(sum(sum(d0)),1);         % set d(d0)'s to 1
    d = d.^(-1/(m-1));
    f0 = (d ./ (sum(d')'*ones(1,c)));
    
end

f = zeros(mx,c);                % partition matrix
d = f;                          % distance matrix
P = zeros(c*nx,nx);             % covariance matrix
M = P;                          % norm-inducing matrix
V = zeros(c,nx);                % eigenvectors
D = V;                          % eigenvalues
C = V;                          % cluster prototypes
iter = 0;                       % iteration counter

if nargin < 6
    rho = ones(1,c);
elseif isempty(rho)
    rho = ones(1,c);
end

% initialize graphics
if s ~= 0  &&  nx < 3
    subplot(211);
    lines = [C(:,1)*x(:,1)' + C(:,2)*ones(1,mx)]';
    mask = find(f0 < 0.2);              % find membership degrees < 0.2
    lines(mask) = NaN*ones(size(mask)); % mask with NaN's for plots
    H1 = plot(x(:,1), x(:,2), 'go', C(:,1), C(:,2), 'r*', x(:,1), lines,...
             'EraseMode','xor');
    title('Function approximation');
    xlabel('x'); ylabel('y');
    minx = min(x(:,1));
    maxx = max(x(:,1));
    miny = min(x(:,2));
    maxy = max(x(:,2));
    ma = 0.3*max(abs(x(:,2)));
    axis([minx maxx miny-ma maxy+ma]);
    subplot(212);
    if s == 1,
        H2 = plot(x(:,1), f0, 'o', 'EraseMode','xor');
    else
        H2 = plot(x(:,1), f0, 'EraseMode','xor');
    end;
    title('Membership functions');
    xlabel('x');
    ylabel('Membership');
    set(gcf, 'UserData', [H1;H2]);
end;

% Iterate
while  max(max(f0-f)) > e
    iter = iter + 1;
    f = f0;
    fm = f;
    sumf = sum(fm);
    % Calculate centers
    C = (fm'*x)./(sumf'*ones(1,nx));
    for j = 1 : c,                        % for all clusters
        xv = x - x1*C(j,:);
        % Calculate covariance matrix
        p = ones(nx,1)*f(:,j)'.*xv'*xv/sum(f(:,j));
        % Calculate distances
        if det(p) == 0
            f;
            return;
        end;
        M = (det(p)/rho(j))^inx*inv(p);
        Pi(j) = sumf(j)/mx;
        d(:,j) = (sqrt(det(p))/Pi(j))*exp(sum((xv*inv(p).*xv)')'/2);
        % Calculate eigen vectors and cluster prototypes (lines)
        if s ~= 0 && nx < 3,
            [ev,ed] = eig(p); ed = diag(ed);
            ev = ev(:,ed == min(ed));
            lines(:,j) = -x(:,1)*ev(1)/ev(2) + C(j,:)*ev/ev(2);
            mask = find(f0 < 0.2);          % find membership degrees < 0.2
            lines(mask) = NaN*ones(size(mask)); % mask with NaN's for plots
        end
    end
    J(iter) = sum(sum(f0.*d));
    % Update f0
    d0 = (d == 0);                                % find zero distances
    c0 = max(d0')';
    d(c0,:) = zeros(sum(c0),c);                   % set d(i,:) to 0
    d(d0) = ones(sum(sum(d0)),1);                 % set d(d0)'s to 1
    d = 1./d;
    f0 = (d ./ (sum(d')'*ones(1,c)));
    
    % Plot intermediate results
    if s ~= 0
        fprintf('Iteration count = %d, J = %f\n\n',iter,max(max(f-f0))),
        if nx < 3
            H = get(gcf,'UserData');
            set(H(2),'xdata',C(:,1),'ydata',C(:,2));
            for i = 1 : c
                set(H(2+i),'ydata',lines(:,i));
                set(H(2+c+i),'ydata',f0(:,i));
            end
            drawnow;
        end
    end;
end

f = f0;
fm = f.^m;
sumf = sum(fm);


% calculate P,V,D,M
for j = 1 : c                     % for all clusters
    xv = x - ones(mx,1)*C(j,:);
    % Calculate covariance matrix
    p = ones(nx,1)*fm(:,j)'.*xv'*xv/sumf(j);
    % Calculate eigen values and eigen vectors
    [ev, ed] = eig(p);
    ed = diag(ed)';
    ev = ev(:, ed == min(ed));
    % Concatenate cluster info in one matrix
    P((j-1)*nx+(1:nx),:) = p;
    M((j-1)*nx+(1:nx),:) = (det(p)/rho(j)).^(1/nx)*inv(p);
    V(j,:) = ev';
    D(j,:) = ed;
end

return

% addition a la Michel
rp = rank(p(1:nx-1,1:nx-1));
if rp < nx-1,
    [q, r, perm] = qr(p(1:nx-1,1:nx-1));
    [ind, jnd] = find(perm);
    xmask(j, ind(nx-1:-1:rp+1)) = zeros(1,nx-1-rp);
end

% Calculate distances
M = (det(p(xmask(j,:),xmask(j,:)))/rho(j)).^(1/sum(xmask(j,:)))*inv(p(xmask(j,:),xmask(j,:)));
for i = 1 : mx
    d(i,j) = (x(i,xmask(j,:)) - C(j,xmask(j,:)))*M*(x(i,xmask(j,:))...
              -C(j,xmask(j,:)))';
end

end