% *************************************************************************
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% This exercise demonstrates the Gustafson-Kessel. The original algorithm
% by Robert Babuska, 1994-96 public domain software available at
% http://www.dcsc.tudelft.nl/~rbabuska/ was modified by SML and Matteo
% Masi in the adaptive volume version.
%
% Sect. 4.5.8.a
%
% *************************************************************************

clear
close all
clc

% Data input
disp('-----------------------------------')
disp('--- Gustafson-Kessel Clustering ---')
disp('--- with adaptive cluster volume --')
disp('-----------------------------------')
disp(' ')
F_I = input('Do you want to load data from a file (1) or by direct input (2)? ');
if F_I == 1
    % Get the data from file
    g = uigetfile('GK*.mat','Load data');
    eval(['load ' g]);
    % Plot the data
    figure(1)
    set(1, 'Position',[29 45 593 526])
    axis square
    hold on
    N = size(X,1);
    x1min = floor(min(X(:,1)));
    x1max = ceil(max(X(:,1)));
    x2min = floor(min(X(:,2)));
    x2max = ceil(max(X(:,2)));
    Ex = [x1min x1max x2min x2max];
    axis(Ex);
    set(gca, 'FontName','Arial', 'FontSize',12)
    xlabel('x_1')
    ylabel('x_2')
    hold on
    N = size(X,1);
    for i = 1:N
        plot(X(i,1), X(i,2), 'ok', 'MarkerSize',7,...
            'MarkerFaceColor',[0.83 0.82 0.78])
    end
    Xlim = [x1min x1max];
    Ylim = [x2min x2max];
else
    if F_I == 2,
        % Enter data with the mouse
        figure(1)
        TTL=title('right-click to exit data entry');
        set(1, 'Position',[29 45 593 526])
        xmin = 0;
        xmax = 1;
        ymin = 0;
        ymax = 1;
        axis([xmin xmax ymin ymax])
        axis square
        axis(axis)
        set(gca, 'FontName','Arial', 'FontSize',12)
        xlabel('x_1')
        ylabel('x_2')
        hold on
        X=[];
        i=0;
        butt = 1;
        while butt == 1
            i = i + 1;
            [x, y, butt] = ginput(1);
            plot(x, y, 'ok', 'MarkerSize',7,...
                'MarkerFaceColor',[0.83 0.82 0.78])
            X = [X; x y];
        end
        delete(TTL); clear TTL
        % Save the data for later
        sf = input('Do you want to save the data? (y/n) ', 's');
        if sf == 'y',
            Fname = input('Name of data file ', 's');
            Fn = char(Fname);
            eval(['save ', Fn, ' X'])
        end
    end
end
m = input('Fuzzy exponent: ');
c = input('Number of clusters: ');
V_Flag = input('Adaptive volume? (y/n): ', 's');
disp('  ')

% Parameters
tol = 1e-3; % Tolerance

if V_Flag == 'n'
    %% GK standard
    [U_GK, v_GK, F_GK, V_GK, D_GK, J_GK, A_GK] = gkfast(X, c, m, tol);
    figure(1)
    set(gca, 'FontName','Arial', 'FontSize',14)
    rho = ones(1, c);
    plot(v_GK(:,1), v_GK(:,2), 'sk', 'MarkerFaceColor','k', 'MarkerSize',10)
    fuzzy_contour(X, 1, 2, U_GK, v_GK, c, m, F_GK, 'GK', rho)
    axis tight
    xlabel('x_1')
    ylabel('x_2')
    disp('----------------------------')
    Hn = Fuzzy_entropy(U_GK');
    [g r] = Sep_Coef(X, U_GK', v_GK);
    title(['H_n = ', num2str(Hn, '%8.4f'), '  ', 'g = ', num2str(g, '%8.4f'),...
        '  ', 'm = ', num2str(m, '%4.1f')])
    FHV = fhyperv(X, U_GK, v_GK, m);
    disp(['Fuzzy Hypervolume ', num2str(FHV)])
    disp('----------------------------')
end

if V_Flag == 'y'
    %% GK VOLUME
    [U_GK2, v_GK2, F_GK2, V, D, J_GKvol, A,rho] = gkvolume(X, c, m, tol);
    figure(1)
    set(gca, 'FontName','Arial', 'FontSize',14)
    fuzzy_contour(X, 1, 2, U_GK2, v_GK2, c, m, F_GK2, 'GK', rho)
    plot(v_GK2(:,1), v_GK2(:,2), 'sk', 'MarkerFaceColor','k', 'MarkerSize',10)
    axis tight
    xlabel('x_1')
    ylabel('x_2')
    disp(' ')
    disp('----------------------------')
    Hn = Fuzzy_entropy(U_GK2');
    [g r] = Sep_Coef(X, U_GK2', v_GK2);
    title(['H_n = ', num2str(Hn, '%8.4f'), '  ', 'g = ', num2str(g,'%8.4f'),...
        '  ', 'm = ', num2str(m, '%4.1f')])
    FHV = fhyperv(X, U_GK2, v_GK2, m);
    disp(['Fuzzy Hypervolume ', num2str(FHV)])
    disp('----------------------------')
end

% Save the data
if F_I == 2,
    Save_data = input('Do you want to save your data? (y/n) ', 's');
    if Save_data == 'y',
        Fname = input('Name of data file ', 's');
        Fn = char(['FC_', Fname]);
        Mn = 'X';
        eval(['save ', Fn, ' ', Mn])
    end
end