function [P, V, D, M, detP] = fcov(x, f, v, m, rho)
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% FCOV compute fuzzy covariance matrices for a fuzzy partition, the
% eigenvalues and eigenvectors corresponding to largest eigen values.
%
% (c) Robert Babuska, 1994-96
%
% *************************************************************************
%
%      [P, V, D, M] = FCOV(X, F, V, M, RHO)
%
%    Input:
%           X        Data matrix.
%
%           F        Fuzzy partition matrix.
%
%           V        Prototype (means) matrix.
%
%           M        Fuzziness coefficient.
%
%           RHO      optional vector of cluster volumes (unit vector by
%                    default).
%
%    Output:
%           P        cluster covariance matrices concatenated in one matrix
%                    P = [P1; P2; ...; Pk], where k is number of clusters.
%
%           V        Eigenvectors of the covariance matrices, corresponding
%                    to the smallest eigenvalues.
%
%           D        eigenvalues of the covariance matrices.
%
%           M        matrices inducing the distance norm, calculated as
%                    M = det(P).^(1/N)*inv(P), M = [M1;M2;...Mk].
%

[mx,nx] = size(x);
c = size(v, 1);
P = zeros(c*nx, nx); % Covariance matrix
M = P;               % Norm-inducing matrix
V = zeros(c, nx);    % Eigenvectors
D = V;               % Eigenvalues
detP = zeros(1,c);	 % Determinants of Pi's

if nargin < 5
    rho = ones(1,c);
elseif isempty(rho)
    rho = ones(1, c);
end

fm = f.^m;
sumf = sum(fm);

%% Calculate P, V, D, M
for j = 1 : c % For all clusters
    xv = x - ones(mx,1)*v(j,:);
    % Calculate covariance matrix
    p = ones(nx,1)*fm(:,j)'.*xv'*xv/sumf(j);
    % Calculate eigen values and eigen vectors
    [ev, ed] = eig(p); ed = diag(ed)';
    ev = ev(:,ed == min(ed));
    % Concatenate cluster info in one matrix
    P((j-1)*nx+(1:nx),:) = p;
    detP(j) = det(p);
    M((j-1)*nx+(1:nx),:) = (detP(j)/rho(j)).^(1/nx)*inv(p);
    V(j,:) = ev';
    D(j,:) = ed;
end

end