function [f, dd] = fgrades(x, v, op, P, method, f_fmle, rho)
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Calculates membership grade of elements in vector X into clusters
% (multi-dimensional fuzzy sets) defined by means V and distance
% norm-inducing matrix M. Both V and M are obtained from GKFAST or FMLE.
% For details on meaning of V and M and for the algorithm, see GKFAST and
% the description of the Gustafson-Kessel method.
%
% (c) Robert Babuska, 1994-96
% Modified with adaptive cluster volumes
% Stefano Marsili-Libelli and Matteo Masi (March 2011)
%
% *************************************************************************
%
%      [F, DD] = FGRADES(X, V, OP, P, METHOD, F_FMLE, RHO)
%
%    Input:
%           X        Data matrix (M x N, with M number of data and N data
%                    dimension).
%
%           V        Cluster means matrix (C-by-N, C - number of clusters).
%
%           OP       optional parameter OP > 1, determines the fuzziness
%                    of clusters, must be the same as in GKFAST, default 2.
%
%           P        Fuzzy covariance matrices.
%
%           METHOD   'GK' or 'FMLE' (default 'GK').
%
%           F_FMLE   Fuzzy partition matrix (FMLE only).
%
%           RHO      1-by-c vector of cluster volumes (needed if GKVOLUME
%                    is used).
%
%    Output:
%           F        Matrix of membership grades.
%
%           DD       Similarities based on the distance.
%

if nargin < 3
    op = 2;
elseif ~all(size(op))
    op = 2;
end

if nargin < 4
    P = [];
end

if nargin < 5
    method = 'GK';
end

if nargin < 6
    f_fmle = [];
end

if nargin < 7
    rho = ones(1, c);
end

[mx, nx] = size(x);
[c, nc] = size(v);
nP = size(P, 2);
x1 = ones(mx, 1);

switch lower(method)
    case 'fmle'
        % Rearrange P in a multidimesional array
        for j = 1:c
            P1(:,:,j) = P(nc*(j-1)+1:nc*j,:);
        end
        P = P1;
        for j = 1:c,
            xv = x - x1*v(j,1:nx);
            a = 1/mx*sum(f_fmle(:,j));
            S = P(:,:,j);
            d(:,j) = (det(S)^(1/2))/a*exp(1/2*sum((xv*inv(S).*xv),2));
        end
        
    case 'gk'
        for j = 1:c,                        % for all clusters
            xv = x - x1*v(j,1:nx);
            if nx > 1,
                % Calculate distances
                if isempty(P),
                    m = eye(nx);
                else
                    P1=P(:,:,j);
                    m = (det(P1)/rho(j)).^(1/nx)*inv(P1);
                end
                d(:,j) = sum((xv*m(1:nx,1:nx).*xv)')';
            else
                d(:,j) = 1./P(j)*xv.^2;
            end
        end
end

% Calculate "similarities" based on distance only
dd = 1./(1+d);
% Calculate membership grades
d0 = (d == 0);                                % find zero distances
c0 = max(d0')';
d(find(c0~=0),:) = zeros(sum(c0),c);                   % set d(i,:) to 0
d(find(d0~=0)) = ones(sum(sum(d0)),1);                 % set d(d0)'s to 1
d = d.^(-1/(op-1));
f = (d ./ (sum(d,2)*ones(1,c)));

end