function [U,v,F,V,D,J,A] = gkfast(X,c,m,e,rho)
%
% Environmental Systems Analysis with Matlab
% S. Marsili-Libelli
%
%  CRC Press 2016
% ISBN 9781498706353
% Matlab code revised by Emanuele El Basri and Giacomo Barni
% *************************************************************************
%
% Gustafson-Kessel clustering algorithm (with fuzzy covariance matrix).
%
% *************************************************************************
%
%      [U, v, F, V, D, J, A] = GKFAST(X, c, m, e, rho)
%
%     Input:
%       X  ... N by P data matrix,
%              N is the number of data points and P data dimension
%       c  ... number of clusters.
%       m  ... optional parameter m > 1, determines the fuzziness
%              of clustering, for m close to 1 clusters become crisp,
%              default value is 2
%       e  ... optional termination tolerance, the algorithm stops
%              when max(max(U(k-1) - U(k))) <= e, default tolerance 1e-3
%       rho .. 1xc vector of expected cluster volumes (default a unit vector)
%
%     Output:
%       U  ... fuzzy partition matrix
%       v  ... cluster means matrix
%       F  ... cluster covariance matrices concatenated in one matrix
%              F = [f1;f2;...fi], where i is number of clusters
%       V  ... eigenvectors of the covariance matrices, corresponding
%              to the smallest eigenvalues
%       D  ... eigenvalues of the covariance matrices
%       J  ... history of the clustering criterion J
%       A  ... matrices inducing the distance norm, calculated as
%              A = det(F).^(1/N)*inv(F), A = [A1;A2;...Ai]
%


if nargin < 3
    m = 2;
elseif isempty(m)
    m = 2;
end

if nargin < 4
    e = 1e-2;
elseif isempty(e)
    e = 1e-2;
end

if nargin < 5
    rho = ones(1,c);
elseif isempty(s)
    rho = ones(1,c);
end

% Data dimensions
[N, p] = size(X);

% Initialize cluster centers
mm = mean(X);
aa = max(abs(X - ones(N,1)*mm));
v = 2*(ones(c,1)*aa).*(rand(c,p)-0.5) + ones(c,1)*mm; % Random cluster centers

% Initialize fuzzy partition matrix U0
for i = 1:c
    Xv = X - ones(N,1)*v(i,:); % Xv = X - vi
    d(:,i) = sum((Xv.^2),2);
end;
d = (d+1e-10).^(-1/(m-1));
U0 = (d ./ (sum(d,2)*ones(1,c)));  % Initial partition matrix

U = zeros(N,c);    % Initialize matrix U
iter = 0;          % Iteration counter

%% Iterate
while  max(max(U0-U)) > e
    iter = iter + 1;
    U = U0;
    Um = U.^m;
    sumU = sum(Um);
    % Calculate new centers
    v = (Um'*X)./(sumU'*ones(1, p));
    
    for i = 1:c,
        Xv = X - ones(N,1)*v(i,:);
        % Calculate fuzzy covariance matrix
        f = ones(p,1)*Um(:,i)'.*Xv'*Xv/sumU(i);
        % Calculate distances
        A = (det(f)/rho(i))^(1/p)*inv(f);
        d(:,i) = sum((Xv*A.*Xv),2);
    end;
    J(iter) = sum(sum(U0.*d));
    % Update U0
    d = (d+1e-10).^(-1/(m-1));
    U0 = (d ./ (sum(d,2)*ones(1,c)));
end



%% Calculate F,V,D,A
F = zeros(p, p, c);   % fuzzy covariance matrix (multidim. array)
A = F;                % norm-inducing matrix
V = zeros(c, p);      % eigenvectors
D = V;                % eigenvalues
Um = U.^m; sumU = sum(Um);

for i = 1:c
    Xv = X - ones(N,1)*v(i,:);
    % Calculate covariance matrix
    f = ones(p,1)*Um(:,i)'.*Xv'*Xv/sumU(i);
    % Calculate eigen values and eigen vectors
    [ev,ed] = eig(f); ed = diag(ed)';
    ev = ev(:,ed == min(ed));
    % Put cluster info in one matrix
    F(:,:,i) = f;
    A(:,:,i) = (det(f)/rho(i)).^(1/p)*inv(f);
    V(i,:) = ev';
    D(i,:) = ed;
end

end